@@ -43,7 +43,7 @@ C_MODE_START
 extern unsigned int mysql_server_last_errno;
 extern char mysql_server_last_error[MYSQL_ERRMSG_SIZE];
 static my_bool emb_read_query_result(MYSQL *mysql);
-static void emb_free_embedded_thd(MYSQL *mysql);
+static void free_embedded_thd(MYSQL *mysql);
 static bool embedded_print_errors= 0;
 
 extern "C" void unireg_clear(int exit_code)
@@ -121,7 +121,7 @@ emb_advanced_command(MYSQL *mysql, enum enum_server_command command,
       thd->killed= NOT_KILLED;
     else
     {
-      emb_free_embedded_thd(mysql);
+      free_embedded_thd(mysql);
       thd= 0;
     }
   }
@@ -431,7 +431,7 @@ int emb_unbuffered_fetch(MYSQL *mysql, char **row)
   return 0;
 }
 
-static void emb_free_embedded_thd(MYSQL *mysql)
+static void free_embedded_thd(MYSQL *mysql)
 {
   THD *thd= (THD*)mysql->thd;
   server_threads.erase(thd);
@@ -454,12 +454,25 @@ static MYSQL_RES * emb_store_result(MYSQL *mysql)
   return mysql_store_result(mysql);
 }
 
-int emb_read_change_user_result(MYSQL *mysql)
+static int emb_read_change_user_result(MYSQL *mysql)
 {
   mysql->net.read_pos= (uchar*)""; // fake an OK packet
   return mysql_errno(mysql) ? (int)packet_error : 1 /* length of the OK packet */;
 }
 
+
+static void emb_on_close_free(MYSQL *mysql)
+{
+  my_free(mysql->info_buffer);
+  mysql->info_buffer= 0;
+  if (mysql->thd)
+  {
+    free_embedded_thd(mysql);
+    mysql->thd= 0;
+  }
+}
+
+
 MYSQL_METHODS embedded_methods= 
 {
   emb_read_query_result,
@@ -469,12 +482,12 @@ MYSQL_METHODS embedded_methods=
   emb_fetch_lengths, 
   emb_flush_use_result,
   emb_read_change_user_result,
+  emb_on_close_free,
   emb_list_fields,
   emb_read_prepare_result,
   emb_stmt_execute,
   emb_read_binary_rows,
   emb_unbuffered_fetch,
-  emb_free_embedded_thd,
   emb_read_statistics,
   emb_read_query_result,
   emb_read_rows_from_cursor
@@ -516,7 +529,6 @@ int init_embedded_server(int argc, char **argv, char **groups)
    */
   int *argcp;
   char ***argvp;
-  int fake_argc = 1;
   char *fake_argv[] = { (char *)"", 0 };
   const char *fake_groups[] = { "server", "embedded", 0 };
   my_bool acl_error;
@@ -529,16 +541,14 @@ int init_embedded_server(int argc, char **argv, char **groups)
   if (init_early_variables())
     return 1;
 
-  if (argc)
+  if (!argc)
   {
-    argcp= &argc;
-    argvp= (char***) &argv;
-  }
-  else
-  {
-    argcp= &fake_argc;
-    argvp= (char ***) &fake_argv;
+    argc= 1;
+    argv= fake_argv;
   }
+  argcp= &argc;
+  argvp= &argv;
+
   if (!groups)
     groups= (char**) fake_groups;
 
@@ -568,6 +578,8 @@ int init_embedded_server(int argc, char **argv, char **groups)
   if (ho_error != 0)
     return 1;
 
+  my_timer_init(&sys_timer_info);
+
   if (init_common_variables())
   {
     mysql_server_end();
@@ -696,8 +708,7 @@ void *create_embedded_thd(int client_flag)
 
   if (thd->variables.max_join_size == HA_POS_ERROR)
     thd->variables.option_bits |= OPTION_BIG_SELECTS;
-  thd->proc_info=0;				// Remove 'login'
-  thd->set_command(COM_SLEEP);
+  thd->mark_connection_idle();
   thd->set_time();
   thd->init_for_queries();
   thd->client_capabilities= client_flag;
@@ -1113,7 +1124,7 @@ bool Protocol::send_result_set_metadata(List<Item> *list, uint flags)
 
   for (uint pos= 0 ; (item= it++); pos++)
   {
-    if (prot.store_field_metadata(thd, item, pos))
+    if (prot.store_item_metadata(thd, item, pos))
       goto err;
   }
 
@@ -1228,8 +1239,7 @@ bool Protocol_binary::write()
     @retval FALSE Success
 */
 
-bool
-net_send_ok(THD *thd,
+bool Protocol::net_send_ok(THD *thd,
             uint server_status, uint statement_warn_count,
             ulonglong affected_rows, ulonglong id, const char *message,
             bool, bool)
@@ -1264,7 +1274,7 @@ net_send_ok(THD *thd,
 */
 
 bool
-net_send_eof(THD *thd, uint server_status, uint statement_warn_count)
+Protocol::net_send_eof(THD *thd, uint server_status, uint statement_warn_count)
 {
   bool error= write_eof_packet(thd, server_status, statement_warn_count);
   thd->cur_data= 0;
@@ -1272,8 +1282,8 @@ net_send_eof(THD *thd, uint server_status, uint statement_warn_count)
 }
 
 
-bool net_send_error_packet(THD *thd, uint sql_errno, const char *err,
-                           const char *sqlstate)
+bool Protocol::net_send_error_packet(THD *thd, uint sql_errno, const char *err,
+                                     const char *sqlstate)
 {
   uint error;
   char converted_err[MYSQL_ERRMSG_SIZE];
