@@ -79,6 +79,12 @@ my_bool my_getopt_skip_unknown= 0;
 */
 my_bool my_getopt_prefix_matching= 1;
 
+/*
+   This is a flag that can be set in client programs. 1 means that
+   handle_options()  will not initialize options to default values.
+*/
+my_bool my_handle_options_init_variables = 1;
+
 static void default_reporter(enum loglevel level,
                              const char *format, ...)
 {
@@ -212,7 +218,8 @@ int handle_options(int *argc, char ***argv,
   DBUG_ASSERT(argv && *argv);
   (*argc)--; /* Skip the program name */
   (*argv)++; /*      --- || ----      */
-  init_variables(longopts, init_one_value);
+  if (my_handle_options_init_variables)
+    init_variables(longopts, init_one_value);
 
   /*
     Search for args_separator, if found, then the first part of the
