@@ -18,7 +18,9 @@
 #include <my_global.h>
 #include <string.h>
 
+#define template _template /* bug in WolfSSL 4.4.0, see also violite.h */
 #include <openssl/evp.h>
+#undef template
 #include <openssl/aes.h>
 #include <openssl/err.h>
 #include <openssl/rand.h>
@@ -88,24 +90,8 @@ public:
   }
   virtual int finish(uchar *dst, uint *dlen)
   {
-#ifdef HAVE_WOLFSSL
-     /*
-       Bug in WolfSSL - sometimes EVP_CipherFinal_ex
-       returns success without setting destination length
-       when it should return error.
-       We catch it by presetting invalid value for length,
-       and checking if it has changed after the call.
-
-       See https://github.com/wolfSSL/wolfssl/issues/2224
-     */
-    *dlen= UINT_MAX;
-#endif
     if (EVP_CipherFinal_ex(ctx, dst, (int*)dlen) != 1)
       return MY_AES_BAD_DATA;
-#ifdef HAVE_WOLFSSL
-    if (*dlen == UINT_MAX)
-      return MY_AES_BAD_DATA;
-#endif
     return MY_AES_OK;
   }
 };
@@ -118,7 +104,7 @@ public:
   uchar oiv[MY_AES_BLOCK_SIZE];
 
   MyCTX_nopad() : MyCTX() { }
-  ~MyCTX_nopad() { }
+  ~MyCTX_nopad() = default;
 
   int init(const EVP_CIPHER *cipher, int encrypt, const uchar *key, uint klen,
            const uchar *iv, uint ivlen)
