@@ -35,8 +35,8 @@ public:
   static Create_func_trt<TRT_FIELD> s_singleton;
 
 protected:
-  Create_func_trt<TRT_FIELD>() {}
-  virtual ~Create_func_trt<TRT_FIELD>() {}
+  Create_func_trt() = default;
+  virtual ~Create_func_trt() = default;
 };
 
 template<TR_table::field_id_t TRT_FIELD>
@@ -131,8 +131,8 @@ public:
   static Create_func_trt_trx_sees<Item_func_trt_trx_seesX> s_singleton;
 
 protected:
-  Create_func_trt_trx_sees<Item_func_trt_trx_seesX>() {}
-  virtual ~Create_func_trt_trx_sees<Item_func_trt_trx_seesX>() {}
+  Create_func_trt_trx_sees() = default;
+  virtual ~Create_func_trt_trx_sees() = default;
 };
 
 template<class X>
@@ -140,7 +140,7 @@ Create_func_trt_trx_sees<X> Create_func_trt_trx_sees<X>::s_singleton;
 
 #define BUILDER(F) & F::s_singleton
 
-static Native_func_registry func_array[] =
+static const Native_func_registry func_array_vers[] =
 {
   { { C_STRING_WITH_LEN("TRT_BEGIN_TS") }, BUILDER(Create_func_trt<TR_table::FLD_BEGIN_TS>)},
   { { C_STRING_WITH_LEN("TRT_COMMIT_ID") }, BUILDER(Create_func_trt<TR_table::FLD_COMMIT_ID>)},
@@ -164,7 +164,7 @@ static int versioning_plugin_init(void *p __attribute__ ((unused)))
 {
   DBUG_ENTER("versioning_plugin_init");
   // No need in locking since we so far single-threaded
-  int res= item_create_append(func_array);
+  int res= native_functions_hash.append(func_array_vers);
   if (res)
   {
     my_message(ER_PLUGIN_IS_NOT_LOADED, "Can't append function array" , MYF(0));
@@ -177,7 +177,7 @@ static int versioning_plugin_init(void *p __attribute__ ((unused)))
 static int versioning_plugin_deinit(void *p __attribute__ ((unused)))
 {
   DBUG_ENTER("versioning_plugin_deinit");
-  (void) item_create_remove(func_array);
+  (void) native_functions_hash.remove(func_array_vers);
   DBUG_RETURN(0);
 }
 
