@@ -29,6 +29,7 @@ logging=init
 want_syslog=0
 syslog_tag=
 user='@MYSQLD_USER@'
+group='@MYSQLD_GROUP@'
 pid_file=
 err_log=
 err_log_base=
@@ -73,12 +74,14 @@ usage () {
         cat <<EOF
 Usage: $0 [OPTIONS]
   --no-defaults              Don't read the system defaults file
-  --core-file-size=LIMIT     Limit core files to the specified size
   --defaults-file=FILE       Use the specified defaults file
   --defaults-extra-file=FILE Also use defaults from the specified file
+  --defaults-group-suffix=X  Additionally read default groups with X appended
+                             as a suffix
   --ledir=DIRECTORY          Look for mysqld in the specified directory
   --open-files-limit=LIMIT   Limit the number of open files
   --crash-script=FILE        Script to call when mysqld crashes
+  --core-file-size=LIMIT     Limit core files to the specified size
   --timezone=TZ              Set the system timezone
   --malloc-lib=LIB           Preload shared library LIB if available
   --mysqld=FILE              Use the specified file as mysqld
@@ -126,7 +129,7 @@ log_generic () {
   case $logging in
     init) ;;  # Just echo the message, don't save it anywhere
     file)
-      if [ -n "$helper" ]; then
+      if [ "$helper_exist" -eq "0" ]; then
         echo "$msg" | "$helper" "$user" log "$err_log"
       fi
     ;;
@@ -150,7 +153,7 @@ eval_log_error () {
   local cmd="$1"
   case $logging in
     file)
-     if [ -n "$helper" ]; then
+     if [ "$helper_exist" -eq "0" ]; then
         cmd="$cmd 2>&1 | "`shell_quote_string "$helper"`" $user log "`shell_quote_string "$err_log"`
      fi
      ;;
@@ -235,7 +238,9 @@ wsrep_recover_position() {
   fi
 
   if [ -f $wr_logfile ]; then
-    [ "$euid" = "0" ] && chown $user $wr_logfile
+    # NOTE! Do not change ownership of the temporary file, as on newer kernel
+    # versions fs.protected_regular is set to '2' and redirecting output with >
+    # as root to a file not owned by root will fail with "Permission denied"
     chmod 600 $wr_logfile
   else
     log_error "WSREP: mktemp failed"
@@ -250,6 +255,11 @@ wsrep_recover_position() {
 
   eval "$mysqld_cmd --wsrep_recover $wr_options 2> $wr_logfile"
 
+  if [ ! -s "$wr_logfile" ]; then
+    log_error "Log file $wr_logfile was empty, cannot proceed. Is system running fs.protected_regular?"
+    exit 1
+  fi
+
   local rp="$(grep 'WSREP: Recovered position:' $wr_logfile)"
   if [ -z "$rp" ]; then
     local skipped="$(grep WSREP $wr_logfile | grep 'skipping position recovery')"
@@ -290,6 +300,7 @@ parse_arguments() {
       --pid[-_]file=*) pid_file="$val" ;;
       --plugin[-_]dir=*) PLUGIN_DIR="$val" ;;
       --user=*) user="$val"; SET_USER=1 ;;
+      --group=*) group="$val"; SET_USER=1 ;;
       --log[-_]basename=*|--hostname=*|--loose[-_]log[-_]basename=*)
         pid_file="$val.pid";
 	err_log_base="$val";
@@ -430,25 +441,10 @@ mysqld_ld_preload_text() {
   echo "$text"
 }
 
-
-mysql_config=
-get_mysql_config() {
-  if [ -z "$mysql_config" ]; then
-    mysql_config=`echo "$0" | sed 's,/[^/][^/]*$,/mysql_config,'`
-    if [ ! -x "$mysql_config" ]; then
-      log_error "Can not run mysql_config $@ from '$mysql_config'"
-      exit 1
-    fi
-  fi
-
-  "$mysql_config" "$@"
-}
-
-
 # set_malloc_lib LIB
 # - If LIB is empty, do nothing and return
-# - If LIB starts with 'tcmalloc' or 'jemalloc', look for the shared library in
-#   /usr/lib, /usr/lib64 and then pkglibdir.
+# - If LIB starts with 'tcmalloc' or 'jemalloc', look for the shared library
+#   using `ldconfig`.
 #   tcmalloc is part of the Google perftools project.
 # - If LIB is an absolute path, assume it is a malloc shared library
 #
@@ -456,28 +452,29 @@ get_mysql_config() {
 # running mysqld.  See ld.so for details.
 set_malloc_lib() {
   malloc_lib="$1"
-
   if expr "$malloc_lib" : "\(tcmalloc\|jemalloc\)" > /dev/null ; then
-    pkglibdir=`get_mysql_config --variable=pkglibdir`
-    where=''
-    # This list is kept intentionally simple.  Simply set --malloc-lib
-    # to a full path if another location is desired.
-    for libdir in /usr/lib /usr/lib64 "$pkglibdir" "$pkglibdir/mysql"; do
-       tmp=`echo "$libdir/lib$malloc_lib.so".[0-9]`
-       where="$where $libdir"
-       # log_notice "DEBUG: Checking for malloc lib '$tmp'"
-       [ -r "$tmp" ] || continue
-       malloc_lib="$tmp"
-       where=''
-       break
-    done
-
-    if [ -n "$where" ]; then
-      log_error "no shared library for lib$malloc_lib.so.[0-9] found in$where"
+    export PATH=$PATH:/sbin
+    if ! command -v ldconfig > /dev/null 2>&1
+    then
+      log_error "ldconfig command not found, required for ldconfig -p"
       exit 1
     fi
-  fi
+    # format from ldconfig:
+    # "libjemalloc.so.1 (libc6,x86-64) => /usr/lib/x86_64-linux-gnu/libjemalloc.so.1"
+    libmalloc_path="$(ldconfig -p | sed -n "/lib${malloc_lib}/p" | cut -d '>' -f2)"
 
+    if [ -z "$libmalloc_path" ]; then
+      log_error "no shared library for lib$malloc_lib.so.[0-9] found."
+      exit 1
+    fi
+
+    for f in $libmalloc_path; do
+      if [ -f "$f" ]; then
+        malloc_lib=$f # get the first path if many
+        break
+      fi
+    done
+  fi
   # Allow --malloc-lib='' to override other settings
   [ -z  "$malloc_lib" ] && return
 
@@ -494,7 +491,6 @@ set_malloc_lib() {
       exit 1
       ;;
   esac
-
   add_mysqld_ld_preload "$malloc_lib"
 }
 
@@ -541,10 +537,9 @@ fi
 
 helper=`find_in_bin mysqld_safe_helper`
 print_defaults=`find_in_bin my_print_defaults`
-
 # Check if helper exists
-$helper --help >/dev/null 2>&1 || helper=""
-
+command -v $helper --help >/dev/null 2>&1
+helper_exist=$?
 #
 # Second, try to find the data directory
 #
@@ -708,6 +703,8 @@ then
   if test "$user" != "root" -o $SET_USER = 1
   then
     USER_OPTION="--user=$user"
+    # To be used if/when we enable --system-group as an option to mysqld
+    GROUP_OPTION="--group=$group"
   fi
   if test -n "$open_files"
   then
@@ -730,7 +727,12 @@ then
     log_error "Fatal error Can't create database directory '$mysql_unix_port'"
     exit 1
   fi
-  chown $user $mysql_unix_port_dir
+  if [ "$user" -a "$group" ]; then
+    chown $user:$group $mysql_unix_port_dir
+  else
+    [ "$user" ] && chown $user $mysql_unix_port_dir
+    [ "$group" ] && chgrp $group $mysql_unix_port_dir
+  fi
   chmod 755 $mysql_unix_port_dir
 fi
 
@@ -951,7 +953,6 @@ fi
 
 # Avoid 'nohup: ignoring input' warning
 test -n "$NOHUP_NICENESS" && cmd="$cmd < /dev/null"
-
 log_notice "Starting $MYSQLD daemon with databases from $DATADIR"
 
 # variable to track the current number of "fast" (a.k.a. subsecond) restarts
