@@ -33,11 +33,7 @@ struct TABLE;
 class Event_queue_element_for_exec
 {
 public:
-  Event_queue_element_for_exec()
-  {
-    dbname.str= NULL; dbname.length= 0;
-    name.str= NULL; name.length= 0;
-  };
+  Event_queue_element_for_exec() : dbname{nullptr, 0}, name{nullptr, 0} {}
   ~Event_queue_element_for_exec();
 
   bool
