@@ -538,7 +538,7 @@ Event_queue::empty_queue()
 void
 Event_queue::dbug_dump_queue(my_time_t when)
 {
-#ifndef DBUG_OFF
+#ifdef DBUG_TRACE
   my_time_t now= when;
   Event_queue_element *et;
   uint i;
@@ -650,7 +650,6 @@ Event_queue::get_top_for_execution_if_time(THD *thd,
       top->status= Event_parse_data::DISABLED;
     DBUG_PRINT("info", ("event %s status is %d", top->name.str, top->status));
 
-    top->execution_count++;
     (*event_name)->dropped= top->dropped;
     /*
       Save new values of last_executed timestamp and event status on stack
