@@ -30,6 +30,10 @@
 #include "sql_base.h"
 #include "log_event.h"
 
+#ifdef WITH_WSREP
+#include "wsrep_trans_observer.h" /* wsrep_start_transaction() */
+#endif
+
 /*
   Table flags we should inherit and disable from the original engine.
   We add HA_STATS_RECORDS_IS_EXACT as ha_sequence::info() will ensure
@@ -199,6 +203,7 @@ int ha_sequence::write_row(const uchar *buf)
   int error;
   sequence_definition tmp_seq;
   bool sequence_locked;
+  THD *thd= table->in_use;
   DBUG_ENTER("ha_sequence::write_row");
   DBUG_ASSERT(table->record[0] == buf);
 
@@ -235,8 +240,8 @@ int ha_sequence::write_row(const uchar *buf)
         on master and slaves
       - Check that the new row is an accurate SEQUENCE object
     */
-
-    THD *thd= table->in_use;
+    /* mark a full binlog image insert to force non-parallel slave */
+    thd->transaction.stmt.mark_trans_did_ddl();
     if (table->s->tmp_table == NO_TMP_TABLE &&
         thd->mdl_context.upgrade_shared_lock(table->mdl_ticket,
                                              MDL_EXCLUSIVE,
@@ -255,6 +260,29 @@ int ha_sequence::write_row(const uchar *buf)
     sequence->write_lock(table);
   }
 
+#ifdef WITH_WSREP
+  if (WSREP_ON && WSREP(thd) && wsrep_thd_is_local(thd))
+  {
+    if (sequence_locked &&
+        (wsrep_thd_is_SR(thd) || wsrep_streaming_enabled(thd)))
+    {
+      my_error(ER_NOT_SUPPORTED_YET, MYF(0),
+               "SEQUENCEs with streaming replication in Galera cluster");
+      DBUG_RETURN(HA_ERR_UNSUPPORTED);
+    }
+
+    /*
+       We need to start Galera transaction for select NEXT VALUE FOR
+       sequence if it is not yet started. Note that ALTER is handled
+       as TOI.
+    */
+    if (!thd->wsrep_trx().active())
+    {
+      wsrep_start_transaction(thd, thd->wsrep_next_trx_id());
+    }
+  }
+#endif
+
   if (likely(!(error= file->update_first_row(buf))))
   {
     Log_func *log_func= Write_rows_log_event::binlog_row_logging_function;
@@ -421,6 +449,9 @@ static int sequence_initialize(void *p)
                                HTON_HIDDEN |
                                HTON_TEMPORARY_NOT_SUPPORTED |
                                HTON_ALTER_NOT_SUPPORTED |
+#ifdef WITH_WSREP
+                               HTON_WSREP_REPLICATION |
+#endif
                                HTON_NO_PARTITION);
   DBUG_RETURN(0);
 }
