@@ -144,11 +144,26 @@ class Item_json_str_multipath: public Item_str_func
 protected:
   json_path_with_flags *paths;
   String *tmp_paths;
+private:
+  /**
+    Number of paths returned by calling virtual method get_n_paths() and
+    remembered inside fix_fields(). It is used by the virtual destructor
+    ~Item_json_str_multipath() to iterate along allocated memory chunks stored
+    in the array tmp_paths and free every of them. The virtual method
+    get_n_paths() can't be used for this goal from within virtual destructor.
+    We could get rid of the virtual method get_n_paths() and store the number
+    of paths directly in the constructor of classes derived from the class
+    Item_json_str_multipath but presence of the method get_n_paths() allows
+    to check invariant that the number of arguments not changed between
+    sequential runs of the same prepared statement that seems to be useful.
+  */
+  uint n_paths;
 public:
   Item_json_str_multipath(THD *thd, List<Item> &list):
-    Item_str_func(thd, list), tmp_paths(0) {}
+    Item_str_func(thd, list), paths(NULL), tmp_paths(0), n_paths(0) {}
+  virtual ~Item_json_str_multipath();
+
   bool fix_fields(THD *thd, Item **ref);
-  void cleanup();
   virtual uint get_n_paths() const = 0;
   bool is_json_type() { return true; }
 };
@@ -208,10 +223,10 @@ protected:
 public:
   Item_func_json_contains_path(THD *thd, List<Item> &list):
     Item_bool_func(thd, list), tmp_paths(0) {}
+  virtual ~Item_func_json_contains_path();
   const char *func_name() const { return "json_contains_path"; }
   bool fix_fields(THD *thd, Item **ref);
   bool fix_length_and_dec();
-  void cleanup();
   longlong val_int();
   Item *get_copy(THD *thd)
   { return get_item_copy<Item_func_json_contains_path>(thd, this); }
@@ -310,6 +325,11 @@ class Item_func_json_length: public Item_long_func
 {
   bool check_arguments() const
   {
+    if (arg_count == 0 || arg_count > 2)
+    {
+      my_error(ER_WRONG_PARAMCOUNT_TO_NATIVE_FCT, MYF(0), func_name());
+      return true;
+    }
     return args[0]->check_type_can_return_text(func_name()) ||
            (arg_count > 1 &&
             args[1]->check_type_general_purpose_string(func_name()));
