@@ -368,8 +368,6 @@ static void get_service_config()
 */
 static void change_service_config()
 {
-  char defaults_file[MAX_PATH];
-  char default_character_set[64];
   char buf[MAX_PATH];
   char commandline[3 * MAX_PATH + 19];
   int i;
@@ -392,22 +390,6 @@ static void change_service_config()
   */
   WritePrivateProfileString("mysqld", "basedir",NULL, service_properties.inifile);
 
-  /* 
-    Replace default-character-set  with character-set-server, to avoid 
-    "default-character-set is deprecated and will be replaced ..."
-    message.
-  */
-  default_character_set[0]= 0;
-  GetPrivateProfileString("mysqld", "default-character-set", NULL,
-    default_character_set, sizeof(default_character_set), defaults_file);
-  if (default_character_set[0])
-  {
-    WritePrivateProfileString("mysqld", "default-character-set", NULL, 
-      defaults_file);
-    WritePrivateProfileString("mysqld", "character-set-server",
-      default_character_set, defaults_file);
-  }
-
   sprintf(defaults_file_param,"--defaults-file=%s", service_properties.inifile);
   sprintf_s(commandline, "\"%s\" \"%s\" \"%s\"", mysqld_path, 
    defaults_file_param, opt_service);
@@ -569,8 +551,8 @@ int main(int argc, char **argv)
     if (WaitForSingleObject(mysqld_process, 0) != WAIT_TIMEOUT)
       die("mysqld.exe did not start");
 
-    if (run_tool(P_WAIT, mysqladmin_path, "--protocol=pipe",
-      socket_param, "ping",  NULL) == 0)
+    if (run_tool(P_WAIT, mysqladmin_path, "--protocol=pipe", socket_param,
+                 "ping", "--no-beep", NULL) == 0)
     {
       break;
     }
