@@ -12,7 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
 
 #ifndef _SP_PCONTEXT_H_
 #define _SP_PCONTEXT_H_
@@ -22,7 +22,6 @@
 #endif
 
 #include "sql_string.h"                         // LEX_STRING
-#include "mysql_com.h"                          // enum_field_types
 #include "field.h"                              // Create_field
 #include "sql_array.h"                          // Dynamic_array
 
@@ -372,7 +371,28 @@ public:
   class Lex_for_loop: public Lex_for_loop_st
   {
   public:
-    Lex_for_loop() { init(); }
+    /*
+      The label poiting to the body start,
+      either explicit or automatically generated.
+      Used during generation of "ITERATE loop_label"
+      to check if "loop_label" is a FOR loop label.
+      - In case of a FOR loop, some additional code
+        (cursor fetch or iteger increment) is generated before
+        the backward jump to the beginning of the loop body.
+      - In case of other loop types (WHILE, REPEAT)
+        only the jump is generated.
+    */
+    const sp_label *m_start_label;
+
+    Lex_for_loop()
+     :m_start_label(NULL)
+    { Lex_for_loop_st::init(); }
+
+    Lex_for_loop(const Lex_for_loop_st &for_loop, const sp_label *start)
+     :m_start_label(start)
+    {
+      Lex_for_loop_st::operator=(for_loop);
+    }
   };
 
 public:
@@ -592,13 +612,7 @@ public:
                                      bool current_scope_only) const;
 
   sp_condition_value *
-  find_declared_or_predefined_condition(const LEX_CSTRING *name) const
-  {
-    sp_condition_value *p= find_condition(name, false);
-    if (p)
-      return p;
-    return find_predefined_condition(name);
-  }
+  find_declared_or_predefined_condition(THD *thd, const LEX_CSTRING *name) const;
 
   bool declare_condition(THD *thd, const LEX_CSTRING *name,
                                    sp_condition_value *val)
@@ -686,9 +700,9 @@ public:
 
   void set_for_loop(const Lex_for_loop_st &for_loop)
   {
-    m_for_loop.init(for_loop);
+    m_for_loop= Lex_for_loop(for_loop, last_label());
   }
-  const Lex_for_loop_st &for_loop()
+  const Lex_for_loop &for_loop()
   {
     return m_for_loop;
   }
