@@ -25,6 +25,12 @@
 #include "transaction.h"
 #include "lock.h"
 #include "sql_acl.h"
+#ifdef WITH_WSREP
+#include "wsrep_mysqld.h"
+bool wsrep_check_sequence(THD* thd,
+                          const sequence_definition *seq,
+                          const bool used_engine);
+#endif
 
 struct Field_definition
 {
@@ -183,7 +189,8 @@ void sequence_definition::store_fields(TABLE *table)
     true        Failure
 */
 
-bool check_sequence_fields(LEX *lex, List<Create_field> *fields)
+bool check_sequence_fields(LEX *lex, List<Create_field> *fields,
+                           const LEX_CSTRING db, const LEX_CSTRING table_name)
 {
   Create_field *field;
   List_iterator_fast<Create_field> it(*fields);
@@ -231,8 +238,7 @@ bool check_sequence_fields(LEX *lex, List<Create_field> *fields)
 
 err:
   my_error(ER_SEQUENCE_INVALID_TABLE_STRUCTURE, MYF(0),
-           lex->first_select_lex()->table_list.first->db.str,
-           lex->first_select_lex()->table_list.first->table_name.str, reason);
+           db.str, table_name.str, reason);
   DBUG_RETURN(TRUE);
 }
 
@@ -297,7 +303,8 @@ bool sequence_insert(THD *thd, LEX *lex, TABLE_LIST *org_table_list)
   Query_tables_list query_tables_list_backup;
   TABLE_LIST table_list;                        // For sequence table
   DBUG_ENTER("sequence_insert");
-
+  DBUG_EXECUTE_IF("kill_query_on_sequence_insert",
+                  thd->set_killed(KILL_QUERY););
   /*
     seq is 0 if sequence was created with CREATE TABLE instead of
     CREATE SEQUENCE
@@ -308,11 +315,6 @@ bool sequence_insert(THD *thd, LEX *lex, TABLE_LIST *org_table_list)
       DBUG_RETURN(TRUE);
   }
 
-#ifdef WITH_WSREP
-  if (WSREP_ON && seq->cache != 0)
-    WSREP_WARN("CREATE SEQUENCES declared without `NOCACHE` will not behave correctly in galera cluster.");
-#endif
-
   /* If not temporary table */
   if (!temporary_table)
   {
@@ -902,6 +904,20 @@ end:
   DBUG_RETURN(error);
 }
 
+#if defined(HAVE_REPLICATION)
+class wait_for_commit_raii
+{
+private:
+  THD *m_thd;
+  wait_for_commit *m_wfc;
+
+public:
+  wait_for_commit_raii(THD* thd) :
+      m_thd(thd), m_wfc(thd->suspend_subsequent_commits())
+    {}
+  ~wait_for_commit_raii() { m_thd->resume_subsequent_commits(m_wfc); }
+};
+#endif
 
 bool Sql_cmd_alter_sequence::execute(THD *thd)
 {
@@ -911,10 +927,14 @@ bool Sql_cmd_alter_sequence::execute(THD *thd)
   TABLE_LIST *first_table= lex->query_tables;
   TABLE *table;
   sequence_definition *new_seq= lex->create_info.seq_create_info;
+  uint saved_used_fields= new_seq->used_fields;
   SEQUENCE *seq;
   No_such_table_error_handler no_such_table_handler;
   DBUG_ENTER("Sql_cmd_alter_sequence::execute");
-
+#if defined(HAVE_REPLICATION)
+  /* No wakeup():s of subsequent commits is allowed in this function. */
+  wait_for_commit_raii suspend_wfc(thd);
+#endif
 
   if (check_access(thd, ALTER_ACL, first_table->db.str,
                    &first_table->grant.privilege,
@@ -922,21 +942,25 @@ bool Sql_cmd_alter_sequence::execute(THD *thd)
                    0, 0))
     DBUG_RETURN(TRUE);                  /* purecov: inspected */
 
-#ifdef WITH_WSREP
-  if (WSREP_ON && new_seq->cache != 0)
-    WSREP_WARN("ALTER SEQUENCES declared without `NOCACHE` will not behave correctly in galera cluster.");
-#endif
-
   if (check_grant(thd, ALTER_ACL, first_table, FALSE, 1, FALSE))
     DBUG_RETURN(TRUE);                  /* purecov: inspected */
 
 #ifdef WITH_WSREP
-  if (WSREP_ON && WSREP(thd) &&
-      wsrep_to_isolation_begin(thd, first_table->db.str,
-	                       first_table->table_name.str,
-		               first_table))
-    DBUG_RETURN(TRUE);
+  if (WSREP(thd) && wsrep_thd_is_local(thd))
+  {
+    const bool used_engine= lex->create_info.used_fields & HA_CREATE_USED_ENGINE;
+    if (wsrep_check_sequence(thd, new_seq, used_engine))
+      DBUG_RETURN(TRUE);
+
+    if (wsrep_to_isolation_begin(thd, first_table->db.str,
+                                 first_table->table_name.str,
+                                 first_table))
+    {
+      DBUG_RETURN(TRUE);
+    }
+  }
 #endif /* WITH_WSREP */
+
   if (if_exists())
     thd->push_internal_handler(&no_such_table_handler);
   error= open_and_lock_tables(thd, first_table, FALSE, 0);
@@ -1013,11 +1037,17 @@ bool Sql_cmd_alter_sequence::execute(THD *thd)
     error= 1;
   if (trans_commit_implicit(thd))
     error= 1;
+  DBUG_EXECUTE_IF("hold_worker_on_schedule",
+                  {
+                    /* delay binlogging of a parent trx in rpl_parallel_seq */
+                    my_sleep(100000);
+                  });
   if (likely(!error))
     error= write_bin_log(thd, 1, thd->query(), thd->query_length());
   if (likely(!error))
     my_ok(thd);
 
 end:
+  new_seq->used_fields= saved_used_fields;
   DBUG_RETURN(error);
 }
