@@ -122,11 +122,11 @@ int select_unit::send_data(List<Item> &values)
     table->null_catch_flags= CHECK_ROW_FOR_NULLS_TO_REJECT;
   if (intersect_mark)
   {
-    fill_record(thd, table, table->field + 1, values, TRUE, FALSE);
+    fill_record(thd, table, table->field + 1, values, true, false, true);
     table->field[0]->store((ulonglong) curr_step, 1);
   }
   else
-    fill_record(thd, table, table->field, values, TRUE, FALSE);
+    fill_record(thd, table, table->field, values, true, false, true);
   if (unlikely(thd->is_error()))
   {
     rc= 1;
@@ -551,15 +551,18 @@ int select_union_direct::send_data(List<Item> &items)
 {
   if (!limit)
     return false;
-  limit--;
-  if (offset)
+  if (!unit->offset_limit_cnt)
   {
-    offset--;
-    return false;
+    limit--;
+    if (offset)
+    {
+      offset--;
+      return false;
+    }
+    send_records++;
   }
 
-  send_records++;
-  fill_record(thd, table, table->field, items, true, false);
+  fill_record(thd, table, table->field, items, true, false, true);
   if (unlikely(thd->is_error()))
     return true; /* purecov: inspected */
 
@@ -633,15 +636,6 @@ st_select_lex_unit::init_prepare_fake_select_lex(THD *thd_arg,
          order= order->next)
       order->item= &order->item_ptr;
   }
-  for (ORDER *order= global_parameters()->order_list.first;
-       order;
-       order=order->next)
-  {
-    (*order->item)->walk(&Item::change_context_processor, 0,
-                         &fake_select_lex->context);
-    (*order->item)->walk(&Item::set_fake_select_as_master_processor, 0,
-                         fake_select_lex);
-  }
 }
 
 
@@ -1664,9 +1658,9 @@ bool st_select_lex_unit::exec()
           the current result.
         */
         push_warning_printf(thd, Sql_condition::WARN_LEVEL_WARN,
-                            ER_QUERY_EXCEEDED_ROWS_EXAMINED_LIMIT,
-                            ER_THD(thd, ER_QUERY_EXCEEDED_ROWS_EXAMINED_LIMIT),
-                            thd->accessed_rows_and_keys,
+                            ER_QUERY_RESULT_INCOMPLETE,
+                            ER_THD(thd, ER_QUERY_RESULT_INCOMPLETE),
+                            "LIMIT ROWS EXAMINED",
                             thd->lex->limit_rows_examined->val_uint());
         thd->reset_killed();
         break;
