@@ -124,12 +124,17 @@ static void re_init_net_server_extension(THD *thd)
 
 #endif /* HAVE_PSI_INTERFACE */
 
+static inline bool has_unread_compressed_data(const NET *net)
+{
+  return net->compress && net->remain_in_buf;
+}
 
 static inline void set_thd_idle(THD *thd)
 {
   thd->net.reading_or_writing= 1;
 #ifdef HAVE_PSI_INTERFACE
-  net_before_header_psi(&thd->net, thd, 0);
+  if (!has_unread_compressed_data(&thd->net))
+    net_before_header_psi(&thd->net, thd, 0);
 #endif
 }
 
@@ -209,12 +214,11 @@ void tp_callback(TP_connection *c)
 
 error:
   c->thd= 0;
-  delete c;
-
   if (thd)
   {
     threadpool_remove_connection(thd);
   }
+  delete c;
   worker_context.restore();
 }
 
@@ -248,8 +252,8 @@ static THD* threadpool_add_connection(CONNECT *connect, void *scheduler_data)
     }
     return NULL;
   }
-  delete connect;
-  server_threads.insert(thd);
+  server_threads.insert(thd); // Make THD visible in show processlist
+  delete connect; // must be after server_threads.insert, see close_connections()
   thd->set_mysys_var(mysys_var);
   thd->event_scheduler.data= scheduler_data;
 
@@ -323,6 +327,14 @@ static void handle_wait_timeout(THD *thd)
   thd->net.error= 2;
 }
 
+/** Check if some client data is cached in thd->net or thd->net.vio */
+static bool has_unread_data(THD* thd)
+{
+  NET *net= &thd->net;
+  Vio *vio= net->vio;
+  return vio->has_data(vio) || has_unread_compressed_data(net);
+}
+
 
 /**
  Process a single client request or a single batch.
@@ -357,7 +369,6 @@ static int threadpool_process_request(THD *thd)
   */
   for(;;)
   {
-    Vio *vio;
     thd->net.reading_or_writing= 0;
     if (mysql_audit_release_required(thd))
       mysql_audit_release(thd);
@@ -373,8 +384,7 @@ static int threadpool_process_request(THD *thd)
 
     set_thd_idle(thd);
 
-    vio= thd->net.vio;
-    if (!vio->has_data(vio))
+    if (!has_unread_data(thd))
     { 
       /* More info on this debug sync is in sql_parse.cc*/
       DEBUG_SYNC(thd, "before_do_command_net_read");
@@ -475,11 +485,25 @@ void tp_timeout_handler(TP_connection *c)
 {
   if (c->state != TP_STATE_IDLE)
     return;
-  THD *thd=c->thd;
+  THD *thd= c->thd;
   mysql_mutex_lock(&thd->LOCK_thd_kill);
-  thd->set_killed_no_mutex(KILL_WAIT_TIMEOUT);
-  c->priority= TP_PRIORITY_HIGH;
-  post_kill_notification(thd);
+  Vio *vio= thd->net.vio;
+  if (vio && (vio_pending(vio) > 0 || vio->has_data(vio)) &&
+      c->state == TP_STATE_IDLE)
+  {
+    /*
+     There is some data on that connection, i.e
+     i.e there was no inactivity timeout.
+     Don't kill.
+    */
+    c->state= TP_STATE_PENDING;
+  }
+  else if (c->state == TP_STATE_IDLE)
+  {
+    thd->set_killed_no_mutex(KILL_WAIT_TIMEOUT);
+    c->priority= TP_PRIORITY_HIGH;
+    post_kill_notification(thd);
+  }
   mysql_mutex_unlock(&thd->LOCK_thd_kill);
 }
 
