@@ -1,4 +1,4 @@
-/* Copyright 2018 Codership Oy <info@codership.com>
+/* Copyright 2018-2023 Codership Oy <info@codership.com>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 #include "wsrep_schema.h"
 #include "wsrep_xid.h"
 #include "wsrep_trans_observer.h"
+#include "wsrep_server_state.h"
 
 #include "sql_class.h" /* THD */
 #include "transaction.h"
@@ -44,7 +45,7 @@ public:
     , m_server_status(thd->server_status)
   {
     m_thd->variables.option_bits&= ~OPTION_BEGIN;
-    m_thd->server_status&= ~SERVER_STATUS_IN_TRANS;
+    m_thd->server_status&= ~(SERVER_STATUS_IN_TRANS | SERVER_STATUS_IN_TRANS_READONLY);
     m_thd->wsrep_cs().enter_toi_mode(ws_meta);
   }
   ~Wsrep_non_trans_mode()
@@ -142,11 +143,12 @@ Wsrep_high_priority_service::Wsrep_high_priority_service(THD* thd)
 
   /* Disable general logging on applier threads */
   thd->variables.option_bits |= OPTION_LOG_OFF;
-  /* Enable binlogging if opt_log_slave_updates is set */
-  if (opt_log_slave_updates)
-    thd->variables.option_bits|= OPTION_BIN_LOG;
-  else
-    thd->variables.option_bits&= ~(OPTION_BIN_LOG);
+
+  /* enable binlogging regardless of log_slave_updates setting
+     this is for ensuring that both local and applier transaction go through
+     same commit ordering algorithm in group commit control
+   */
+  thd->variables.option_bits|= OPTION_BIN_LOG;
 
   thd->net.vio= 0;
   thd->reset_db(&db_str);
@@ -195,6 +197,7 @@ int Wsrep_high_priority_service::start_transaction(
   const wsrep::ws_handle& ws_handle, const wsrep::ws_meta& ws_meta)
 {
   DBUG_ENTER(" Wsrep_high_priority_service::start_transaction");
+  m_thd->set_time();
   DBUG_RETURN(m_thd->wsrep_cs().start_transaction(ws_handle, ws_meta) ||
               trans_begin(m_thd));
 }
@@ -274,7 +277,9 @@ int Wsrep_high_priority_service::append_fragment_and_commit(
   ret= ret || trans_commit(m_thd);
 
   m_thd->wsrep_cs().after_applying();
-  m_thd->mdl_context.release_transactional_locks();
+  m_thd->release_transactional_locks();
+
+  free_root(m_thd->mem_root, MYF(MY_KEEP_PREALLOC));
 
   thd_proc_info(m_thd, "wsrep applier committed");
 
@@ -299,16 +304,22 @@ int Wsrep_high_priority_service::commit(const wsrep::ws_handle& ws_handle,
   DBUG_ASSERT(thd->wsrep_trx().active());
   thd->wsrep_cs().prepare_for_ordering(ws_handle, ws_meta, true);
   thd_proc_info(thd, "committing");
+  int ret=0;
 
   const bool is_ordered= !ws_meta.seqno().is_undefined();
-  int ret= trans_commit(thd);
+
+  if (!thd->transaction.stmt.is_empty())
+    ret= trans_commit_stmt(thd);
+
+  if (ret == 0)
+    ret= trans_commit(thd);
 
   if (ret == 0)
   {
     m_rgi->cleanup_context(thd, 0);
   }
 
-  m_thd->mdl_context.release_transactional_locks();
+  m_thd->release_transactional_locks();
 
   thd_proc_info(thd, "wsrep applier committed");
 
@@ -334,6 +345,8 @@ int Wsrep_high_priority_service::commit(const wsrep::ws_handle& ws_handle,
 
   thd->lex->sql_command= SQLCOM_END;
 
+  free_root(thd->mem_root, MYF(MY_KEEP_PREALLOC));
+
   must_exit_= check_exit_status();
   DBUG_RETURN(ret);
 }
@@ -344,8 +357,10 @@ int Wsrep_high_priority_service::rollback(const wsrep::ws_handle& ws_handle,
   DBUG_ENTER("Wsrep_high_priority_service::rollback");
   m_thd->wsrep_cs().prepare_for_ordering(ws_handle, ws_meta, false);
   int ret= (trans_rollback_stmt(m_thd) || trans_rollback(m_thd));
-  m_thd->mdl_context.release_transactional_locks();
-  m_thd->mdl_context.release_explicit_locks();
+  m_thd->release_transactional_locks();
+
+  free_root(m_thd->mem_root, MYF(MY_KEEP_PREALLOC));
+
   DBUG_RETURN(ret);
 }
 
@@ -365,6 +380,19 @@ int Wsrep_high_priority_service::apply_toi(const wsrep::ws_meta& ws_meta,
   WSREP_DEBUG("Wsrep_high_priority_service::apply_toi: %lld",
               client_state.toi_meta().seqno().get());
 
+#ifdef ENABLED_DEBUG_SYNC
+  DBUG_EXECUTE_IF("sync.wsrep_apply_toi",
+                  {
+                    const char act[]=
+                      "now "
+                      "SIGNAL sync.wsrep_apply_toi_reached "
+                      "WAIT_FOR signal.wsrep_apply_toi";
+                    DBUG_ASSERT(!debug_sync_set_action(thd,
+                                                       STRING_WITH_LEN(act)));
+                  };);
+#endif
+
+  thd->set_time();
   int ret= wsrep_apply_events(thd, m_rli, data.data(), data.size());
   if (ret != 0 || thd->wsrep_has_ignored_error)
   {
@@ -413,6 +441,17 @@ int Wsrep_high_priority_service::log_dummy_write_set(const wsrep::ws_handle& ws_
   DBUG_PRINT("info",
              ("Wsrep_high_priority_service::log_dummy_write_set: seqno=%lld",
               ws_meta.seqno().get()));
+#ifdef ENABLED_DEBUG_SYNC
+  DBUG_EXECUTE_IF("sync.wsrep_log_dummy_write_set",
+                  {
+                    const char act[]=
+                      "now "
+                      "SIGNAL sync.wsrep_log_dummy_write_set_reached ";
+                    DBUG_ASSERT(!debug_sync_set_action(m_thd,
+                                                       STRING_WITH_LEN(act)));
+                  };);
+#endif
+
   if (ws_meta.ordered())
   {
     wsrep::client_state& cs(m_thd->wsrep_cs());
@@ -429,7 +468,27 @@ int Wsrep_high_priority_service::log_dummy_write_set(const wsrep::ws_handle& ws_
       cs.before_rollback();
       cs.after_rollback();
     }
+
+    if (!WSREP_EMULATE_BINLOG(m_thd))
+    {
+      wsrep_register_for_group_commit(m_thd);
+      /* wait_for_prior_commit() ensures that all preceding transactions
+         have been committed and seqno has been synced into
+         storage engine. We don't release commit order here yet to
+         avoid following transactions to sync seqno before
+         wsrep_set_SE_checkpoint() below returns. This effectively pauses
+         group commit for the checkpoint operation, but is the only way to
+         ensure proper ordering. */
+      m_thd->wait_for_prior_commit();
+    }
+
+    WSREP_DEBUG("checkpointing dummy write set %lld", ws_meta.seqno().get());
     wsrep_set_SE_checkpoint(ws_meta.gtid());
+
+    if (!WSREP_EMULATE_BINLOG(m_thd))
+    {
+      wsrep_unregister_from_group_commit(m_thd);
+    }
     ret= ret || cs.provider().commit_order_leave(ws_handle, ws_meta, err);
     cs.after_applying();
   }
@@ -477,6 +536,7 @@ int Wsrep_applier_service::apply_write_set(const wsrep::ws_meta& ws_meta,
   THD* thd= m_thd;
 
   thd->variables.option_bits |= OPTION_BEGIN;
+  thd->variables.option_bits |= OPTION_GTID_BEGIN;
   thd->variables.option_bits |= OPTION_NOT_AUTOCOMMIT;
   DBUG_ASSERT(thd->wsrep_trx().active());
   DBUG_ASSERT(thd->wsrep_trx().state() == wsrep::transaction::s_executing);
@@ -485,6 +545,7 @@ int Wsrep_applier_service::apply_write_set(const wsrep::ws_meta& ws_meta,
   /* moved dbug sync point here, after possible THD switch for SR transactions
      has ben done
   */
+#ifdef ENABLED_DEBUG_SYNC
   /* Allow tests to block the applier thread using the DBUG facilities */
   DBUG_EXECUTE_IF("sync.wsrep_apply_cb",
                  {
@@ -495,6 +556,7 @@ int Wsrep_applier_service::apply_write_set(const wsrep::ws_meta& ws_meta,
                    DBUG_ASSERT(!debug_sync_set_action(thd,
                                                       STRING_WITH_LEN(act)));
                  };);
+#endif
 
   wsrep_setup_uk_and_fk_checks(thd);
 
@@ -511,6 +573,8 @@ int Wsrep_applier_service::apply_write_set(const wsrep::ws_meta& ws_meta,
     thd->wsrep_cs().fragment_applied(ws_meta.seqno());
   }
   thd_proc_info(thd, "wsrep applied write set");
+
+  thd->variables.option_bits &= ~OPTION_GTID_BEGIN;
   DBUG_RETURN(ret);
 }
 
@@ -562,6 +626,9 @@ Wsrep_replayer_service::Wsrep_replayer_service(THD* replayer_thd, THD* orig_thd)
      transactional locks */
   DBUG_ASSERT(!orig_thd->mdl_context.has_transactional_locks());
 
+  replayer_thd->system_thread_info.rpl_sql_info=
+    new rpl_sql_thread_info(replayer_thd->wsrep_rgi->rli->mi->rpl_filter);
+
   /* Make a shadow copy of diagnostics area and reset */
   m_da_shadow.status= orig_thd->get_stmt_da()->status();
   if (m_da_shadow.status == Diagnostics_area::DA_OK)
@@ -600,35 +667,35 @@ Wsrep_replayer_service::Wsrep_replayer_service(THD* replayer_thd, THD* orig_thd)
 
 Wsrep_replayer_service::~Wsrep_replayer_service()
 {
-  THD* replayer_thd= m_thd;
-  THD* orig_thd= m_orig_thd;
-
   /* Switch execution context back to original. */
-  wsrep_after_apply(replayer_thd);
-  wsrep_after_command_ignore_result(replayer_thd);
-  wsrep_close(replayer_thd);
-  wsrep_reset_threadvars(replayer_thd);
-  wsrep_store_threadvars(orig_thd);
+  wsrep_after_apply(m_thd);
+  wsrep_after_command_ignore_result(m_thd);
+  wsrep_close(m_thd);
+  wsrep_reset_threadvars(m_thd);
+  wsrep_store_threadvars(m_orig_thd);
 
-  DBUG_ASSERT(!orig_thd->get_stmt_da()->is_sent());
-  DBUG_ASSERT(!orig_thd->get_stmt_da()->is_set());
+  DBUG_ASSERT(!m_orig_thd->get_stmt_da()->is_sent());
+  DBUG_ASSERT(!m_orig_thd->get_stmt_da()->is_set());
+
+  delete m_thd->system_thread_info.rpl_sql_info;
+  m_thd->system_thread_info.rpl_sql_info= nullptr;
 
   if (m_replay_status == wsrep::provider::success)
   {
-    DBUG_ASSERT(replayer_thd->wsrep_cs().current_error() == wsrep::e_success);
-    orig_thd->reset_kill_query();
-    my_ok(orig_thd, m_da_shadow.affected_rows, m_da_shadow.last_insert_id);
+    DBUG_ASSERT(m_thd->wsrep_cs().current_error() == wsrep::e_success);
+    m_orig_thd->reset_kill_query();
+    my_ok(m_orig_thd, m_da_shadow.affected_rows, m_da_shadow.last_insert_id);
   }
   else if (m_replay_status == wsrep::provider::error_certification_failed)
   {
-    wsrep_override_error(orig_thd, ER_LOCK_DEADLOCK);
+    wsrep_override_error(m_orig_thd, ER_LOCK_DEADLOCK);
   }
   else
   {
     DBUG_ASSERT(0);
     WSREP_ERROR("trx_replay failed for: %d, schema: %s, query: %s",
                 m_replay_status,
-                orig_thd->db.str, WSREP_QUERY(orig_thd));
+                m_orig_thd->db.str, wsrep_thd_query(m_orig_thd));
     unireg_abort(1);
   }
 }
@@ -643,6 +710,19 @@ int Wsrep_replayer_service::apply_write_set(const wsrep::ws_meta& ws_meta,
   DBUG_ASSERT(thd->wsrep_trx().active());
   DBUG_ASSERT(thd->wsrep_trx().state() == wsrep::transaction::s_replaying);
 
+#ifdef ENABLED_DEBUG_SYNC
+  /* Allow tests to block the replayer thread using the DBUG facilities */
+  DBUG_EXECUTE_IF("sync.wsrep_replay_cb",
+                 {
+                   const char act[]=
+                     "now "
+                     "SIGNAL sync.wsrep_replay_cb_reached "
+                     "WAIT_FOR signal.wsrep_replay_cb";
+                   DBUG_ASSERT(!debug_sync_set_action(thd,
+                                                      STRING_WITH_LEN(act)));
+                 };);
+#endif
+
   wsrep_setup_uk_and_fk_checks(thd);
 
   int ret= 0;
