@@ -39,7 +39,7 @@ static void init_service_thd(THD* thd, char* thread_stack)
   thd->thread_stack= thread_stack;
   thd->real_id= pthread_self();
   thd->prior_thr_create_utime= thd->start_utime= microsecond_interval_timer();
-  thd->set_command(COM_SLEEP);
+  thd->mark_connection_idle();
   thd->reset_for_next_command(true);
 }
 
@@ -143,9 +143,13 @@ void Wsrep_server_service::release_high_priority_service(wsrep::high_priority_se
   wsrep_delete_threadvars();
 }
 
-void Wsrep_server_service::background_rollback(wsrep::client_state& client_state)
+void Wsrep_server_service::background_rollback(
+    wsrep::unique_lock<wsrep::mutex> &lock WSREP_UNUSED,
+    wsrep::client_state &client_state)
 {
-  Wsrep_client_state& cs= static_cast<Wsrep_client_state&>(client_state);
+  DBUG_ASSERT(lock.owns_lock());
+  Wsrep_client_state &cs= static_cast<Wsrep_client_state &>(client_state);
+  mysql_mutex_assert_owner(&cs.thd()->LOCK_thd_data);
   wsrep_fire_rollbacker(cs.thd());
 }
 
@@ -338,6 +342,7 @@ void Wsrep_server_service::log_state_change(
   case Wsrep_server_state::s_synced:
     wsrep_ready= TRUE;
     WSREP_INFO("Synchronized with group, ready for connections");
+    wsrep_ready_set(true);
     /* fall through */
   case Wsrep_server_state::s_joined:
   case Wsrep_server_state::s_donor:
@@ -345,16 +350,16 @@ void Wsrep_server_service::log_state_change(
     break;
   case Wsrep_server_state::s_connected:
     wsrep_cluster_status= "non-Primary";
-    wsrep_ready= FALSE;
+    wsrep_ready_set(false);
     wsrep_connected= TRUE;
     break;
   case Wsrep_server_state::s_disconnected:
-    wsrep_ready= FALSE;
+    wsrep_ready_set(false);
     wsrep_connected= FALSE;
     wsrep_cluster_status= "Disconnected";
     break;
   default:
-    wsrep_ready= FALSE;
+    wsrep_ready_set(false);
     wsrep_cluster_status= "non-Primary";
     break;
   }
