@@ -1,4 +1,4 @@
-/* Copyright 2008-2015 Codership Oy <http://www.codership.com>
+/* Copyright 2008-2022 Codership Oy <http://www.codership.com>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 #include <cstdio>
 #include <cstdlib>
 #include "wsrep_trans_observer.h"
+#include "wsrep_server_state.h"
 
 ulong   wsrep_reject_queries;
 
@@ -119,8 +120,7 @@ bool wsrep_on_update (sys_var *self, THD* thd, enum_var_type var_type)
 
       if (wsrep_init())
       {
-        my_error(ER_CANT_OPEN_LIBRARY, MYF(0), tmp, my_error, "wsrep_init failed");
-        //rcode= true;
+        my_error(ER_CANT_OPEN_LIBRARY, MYF(0), tmp, errno, "wsrep_init failed");
         saved_wsrep_on= false;
       }
 
@@ -752,12 +752,30 @@ bool wsrep_slave_threads_update (sys_var *self, THD* thd, enum_var_type type)
   if (wsrep_slave_count_change > 0)
   {
     WSREP_DEBUG("Creating %d applier threads, total %ld", wsrep_slave_count_change, wsrep_slave_threads);
+    wsrep_thread_create_failed.store(false, std::memory_order_relaxed);
     res= wsrep_create_appliers(wsrep_slave_count_change, true);
+    mysql_mutex_unlock(&LOCK_global_system_variables);
+    mysql_mutex_unlock(&LOCK_wsrep_slave_threads);
+    // Thread creation and execution is asyncronous, therefore we need
+    // wait them to be started or error produced
+    while (wsrep_running_applier_threads != (ulong)wsrep_slave_threads &&
+	   !wsrep_thread_create_failed.load(std::memory_order_relaxed))
+    {
+	    my_sleep(1000);
+    }
+
+    mysql_mutex_lock(&LOCK_global_system_variables);
+
+    if (wsrep_thread_create_failed.load(std::memory_order_relaxed)) {
+      wsrep_slave_threads= wsrep_running_applier_threads;
+      return true;
+    }
+
     WSREP_DEBUG("Running %lu applier threads", wsrep_running_applier_threads);
     wsrep_slave_count_change = 0;
   }
-
-  mysql_mutex_unlock(&LOCK_wsrep_slave_threads);
+  else
+     mysql_mutex_unlock(&LOCK_wsrep_slave_threads);
 
   return res;
 }
@@ -794,7 +812,7 @@ bool wsrep_desync_check (sys_var *self, THD* thd, set_var* var)
     ret= Wsrep_server_state::instance().provider().desync();
     if (ret) {
       WSREP_WARN ("SET desync failed %d for schema: %s, query: %s", ret,
-                  thd->db.str, WSREP_QUERY(thd));
+                  thd->db.str, wsrep_thd_query(thd));
       my_error (ER_CANNOT_USER, MYF(0), "'desync'", thd->query());
       return true;
     }
