@@ -1,11 +1,11 @@
 /************* TabJDBC C++ Program Source Code File (.CPP) *************/
 /* PROGRAM NAME: TABJDBC                                               */
 /* -------------                                                       */
-/*  Version 1.2                                                        */
+/*  Version 1.3                                                        */
 /*                                                                     */
 /* COPYRIGHT:                                                          */
 /* ----------                                                          */
-/*  (C) Copyright to the author Olivier BERTRAND          2016-2017    */
+/*  (C) Copyright to the author Olivier BERTRAND          2016-2019    */
 /*                                                                     */
 /* WHAT THIS PROGRAM DOES:                                             */
 /* -----------------------                                             */
@@ -38,7 +38,7 @@
 #include "my_global.h"
 #include "sql_class.h"
 #include "sql_servers.h"
-#if defined(__WIN__)
+#if defined(_WIN32)
 #include <io.h>
 #include <fcntl.h>
 #if defined(__BORLANDC__)
@@ -72,7 +72,6 @@
 #include "tabext.h"
 #include "tabjdbc.h"
 #include "tabmul.h"
-//#include "reldef.h"
 #include "tabcol.h"
 #include "valblk.h"
 #include "ha_connect.h"
@@ -89,6 +88,9 @@ extern int num_read, num_there, num_eq[2];                // Statistics
 /*  External function.                                                 */
 /***********************************************************************/
 bool ExactInfo(void);
+#if defined(DEVELOPMENT)
+extern char *GetUserVariable(PGLOBAL g, const uchar *varname);
+#endif  // DEVELOPMENT
 
 /* -------------------------- Class JDBCDEF -------------------------- */
 
@@ -147,10 +149,6 @@ int JDBCDEF::ParseURL(PGLOBAL g, char *url, bool b)
 				return RC_FX;
 
 			Tabname = p;
-//  } else if (b) {
-//	  // Otherwise, straight server name, 
-//	  Tabname = GetStringCatInfo(g, "Name", NULL);
-//	  Tabname = GetStringCatInfo(g, "Tabname", Tabname);
 		} // endif
 
 		if (trace(1))
@@ -161,10 +159,15 @@ int JDBCDEF::ParseURL(PGLOBAL g, char *url, bool b)
 
 		// get_server_by_name() clones the server if exists
 		if (!(server= get_server_by_name(current_thd->mem_root, url, &server_buffer))) {
-			sprintf(g->Message, "Server %s does not exist!", url);
+			snprintf(g->Message, sizeof(g->Message), "Server %s does not exist!", url);
 			return RC_FX;
 		} // endif server
 
+#if defined(DEVELOPMENT)
+		if (*server->host == '@') {
+			Url = GetUserVariable(g, (const uchar*)&server->host[1]);
+		} else
+#endif // 0
 		if (strncmp(server->host, "jdbc:", 5)) {
 			// Now make the required URL
 			Url = (PSZ)PlugSubAlloc(g, NULL, 0);
@@ -185,12 +188,18 @@ int JDBCDEF::ParseURL(PGLOBAL g, char *url, bool b)
 		} else		 // host is a URL
 			Url = PlugDup(g, server->host);
 
-		if (server->username)
+		if (!Tabschema && server->db)
+			Tabschema = PlugDup(g, server->db);
+
+		if (!Username && server->username)
 			Username = PlugDup(g, server->username);
 
-		if (server->password)
+		if (!Password && server->password)
 			Password = PlugDup(g, server->password);
 
+		Driver = PlugDup(g, GetListOption(g, "Driver", server->owner, NULL));
+		Wrapname = PlugDup(g, GetListOption(g, "Wrapper", server->owner, NULL));
+		Memory = atoi(GetListOption(g, "Memory", server->owner, "0"));
 		return RC_NF;
 	} // endif
 
@@ -208,7 +217,6 @@ bool JDBCDEF::DefineAM(PGLOBAL g, LPCSTR am, int poff)
 	if (EXTDEF::DefineAM(g, am, poff))
 		return true;
 
-	Driver = GetStringCatInfo(g, "Driver", NULL);
 	Desc = Url = GetStringCatInfo(g, "Connect", NULL);
 
 	if (!Url && !Catfunc) {
@@ -216,7 +224,7 @@ bool JDBCDEF::DefineAM(PGLOBAL g, LPCSTR am, int poff)
 		Url = GetStringCatInfo(g, "Url", NULL);
 
 		if (!Url) {
-			sprintf(g->Message, "Missing URL for JDBC table %s", Name);
+			snprintf(g->Message, sizeof(g->Message), "Missing URL for JDBC table %s", Name);
 			return true;
 		} // endif Url
 
@@ -224,11 +232,14 @@ bool JDBCDEF::DefineAM(PGLOBAL g, LPCSTR am, int poff)
 
 	if (Url)
 		if ((rc = ParseURL(g, Url)) == RC_FX) {
-			sprintf(g->Message, "Wrong JDBC URL %s", Url);
+			snprintf(g->Message, sizeof(g->Message), "Wrong JDBC URL %s", Url);
 			return true;
 		} // endif rc
 
-	Wrapname = GetStringCatInfo(g, "Wrapper", NULL);
+	// Default values may have been set in ParseURL
+	Memory = GetIntCatInfo("Memory", Memory);
+	Driver = GetStringCatInfo(g, "Driver", Driver);
+	Wrapname = GetStringCatInfo(g, "Wrapper", Wrapname);
 	return false;
 } // end of DefineAM
 
@@ -266,7 +277,7 @@ PTDB JDBCDEF::GetTable(PGLOBAL g, MODE m)
 			if (Multiple == 1)
 				tdbp = new(g)TDBMUL(tdbp);
 			else if (Multiple == 2)
-				strcpy(g->Message, "NO_JDBC_MUL");
+				safe_strcpy(g->Message, sizeof(g->Message), "NO_JDBC_MUL");
 
 		} // endswitch Catfunc
 
@@ -375,7 +386,7 @@ bool TDBJDBC::MakeInsert(PGLOBAL g)
 
 	for (colp = Columns; colp; colp = colp->GetNext())
 		if (colp->IsSpecial()) {
-			strcpy(g->Message, "No JDBC special columns");
+			safe_strcpy(g->Message, sizeof(g->Message), "No JDBC special columns");
 			return true;
 		} else {
 			// Column name can be encoded in UTF-8
@@ -449,7 +460,7 @@ bool TDBJDBC::MakeInsert(PGLOBAL g)
 	} // endfor colp
 
 	if ((Query->Append(") VALUES ("))) {
-		strcpy(g->Message, "MakeInsert: Out of memory");
+		safe_strcpy(g->Message, sizeof(g->Message), "MakeInsert: Out of memory");
 		return true;
 	} else // in case prepared statement fails
 		pos = Query->GetLength();
@@ -459,7 +470,7 @@ bool TDBJDBC::MakeInsert(PGLOBAL g)
 		Query->Append("?,");
 
 	if (Query->IsTruncated()) {
-		strcpy(g->Message, "MakeInsert: Out of memory");
+		safe_strcpy(g->Message, sizeof(g->Message), "MakeInsert: Out of memory");
 		return true;
 	} else
 		Query->RepLast(')');
@@ -521,12 +532,15 @@ int TDBJDBC::Cardinality(PGLOBAL g)
 
 		// Table name can be encoded in UTF-8
 		Decode(TableName, tbn, sizeof(tbn));
-		strcpy(qry, "SELECT COUNT(*) FROM ");
+		safe_strcpy(qry, sizeof(qry), "SELECT COUNT(*) FROM ");
 
-		if (Quote)
-			strcat(strcat(strcat(qry, Quote), tbn), Quote);
+		if (Quote) {
+			safe_strcat(qry, sizeof(qry), Quote);
+			safe_strcat(qry, sizeof(qry), tbn);
+			safe_strcat(qry, sizeof(qry), Quote);
+		}
 		else
-			strcat(qry, tbn);
+			safe_strcat(qry, sizeof(qry), tbn);
 
 		// Allocate a Count(*) column (must not use the default constructor)
 		Cnp = new(g)JDBCCOL;
@@ -558,33 +572,42 @@ bool TDBJDBC::OpenDB(PGLOBAL g)
 		     this, Tdb_No, Use, Mode);
 
 	if (Use == USE_OPEN) {
-		/*******************************************************************/
-		/*  Table already open, just replace it at its beginning.          */
-		/*******************************************************************/
-		if (Memory == 1) {
-			if ((Qrp = Jcp->AllocateResult(g, this)))
-				Memory = 2;            // Must be filled
-			else
-				Memory = 0;            // Allocation failed, don't use it
+		if (Mode == MODE_READ || Mode == MODE_READX) {
+			/*****************************************************************/
+			/*  Table already open, just replace it at its beginning.        */
+			/*****************************************************************/
+			if (Memory == 1) {
+				if ((Qrp = Jcp->AllocateResult(g, this)))
+					Memory = 2;            // Must be filled
+				else
+					Memory = 0;            // Allocation failed, don't use it
 
-		} else if (Memory == 2)
-			Memory = 3;              // Ok to use memory result
+			} else if (Memory == 2)
+				Memory = 3;              // Ok to use memory result
 
-		if (Memory < 3) {
-			// Method will depend on cursor type
-			if ((Rbuf = Query ? Jcp->Rewind(Query->GetStr()) : 0) < 0)
-				if (Mode != MODE_READX) {
-					Jcp->Close();
-				  return true;
-			  } else
-				  Rbuf = 0;
+			if (Memory < 3) {
+				// Method will depend on cursor type
+				if ((Rbuf = Query ? Jcp->Rewind(Query->GetStr()) : 0) < 0)
+					if (Mode != MODE_READX) {
+						Jcp->Close();
+						return true;
+					} else
+						Rbuf = 0;
 
-		} else
-			Rbuf = Qrp->Nblin;
+			} else
+				Rbuf = Qrp->Nblin;
+
+			CurNum = 0;
+			Fpos = 0;
+			Curpos = 1;
+		} else if (Mode == MODE_UPDATE || Mode == MODE_DELETE) {
+			// new update coming from a trigger or procedure
+			Query = NULL;
+			SetCondFil(NULL);
+			Qrystr = To_Def->GetStringCatInfo(g, "Query_String", "?");
+		} else {  //if (Mode == MODE_INSERT)
+		} // endif Mode
 
-		CurNum = 0;
-		Fpos = 0;
-		Curpos = 1;
 		return false;
 	} // endif use
 
@@ -624,7 +647,9 @@ bool TDBJDBC::OpenDB(PGLOBAL g)
 				Cnp->InitValue(g);
 
 				if ((n = Jcp->GetResultSize(Query->GetStr(), Cnp)) < 0) {
-					sprintf(g->Message, "Cannot get result size rc=%d", n);
+					char* msg = PlugDup(g, g->Message);
+
+					snprintf(g->Message, sizeof(g->Message), "Get result size: %s (rc=%d)", msg, n);
 					return true;
 				} else if (n) {
 					Jcp->m_Rows = n;
@@ -632,7 +657,7 @@ bool TDBJDBC::OpenDB(PGLOBAL g)
 					if ((Qrp = Jcp->AllocateResult(g, this)))
 						Memory = 2;            // Must be filled
 					else {
-						strcpy(g->Message, "Result set memory allocation failed");
+						safe_strcpy(g->Message, sizeof(g->Message), "Result set memory allocation failed");
 						return true;
 					} // endif n
 
@@ -659,7 +684,7 @@ bool TDBJDBC::OpenDB(PGLOBAL g)
 #if 0
 		if (!(rc = MakeInsert(g))) {
 			if (Nparm != Jcp->PrepareSQL(Query->GetStr())) {
-				strcpy(g->Message, MSG(PARM_CNT_MISS));
+				safe_strcpy(g->Message, sizeof(g->Message), MSG(PARM_CNT_MISS));
 				rc = true;
 			} else
 				rc = BindParameters(g);
@@ -670,7 +695,7 @@ bool TDBJDBC::OpenDB(PGLOBAL g)
 	} else if (Mode == MODE_UPDATE || Mode == MODE_DELETE) {
 		rc = false;  // wait for CheckCond before calling MakeCommand(g);
 	} else
-		sprintf(g->Message, "Invalid mode %d", Mode);
+		snprintf(g->Message, sizeof(g->Message), "Invalid mode %d", Mode);
 
 	if (rc) {
 		Jcp->Close();
@@ -711,12 +736,12 @@ bool TDBJDBC::SetRecpos(PGLOBAL g, int recpos)
 		  CurNum = recpos;
 			Fpos = recpos;
 		} else {
-			strcpy(g->Message, "Scrolling out of row set NIY");
+			safe_strcpy(g->Message, sizeof(g->Message), "Scrolling out of row set NIY");
 			return true;
 		} // endif recpos
 
 	} else {
-		strcpy(g->Message, "This action requires a scrollable cursor");
+		safe_strcpy(g->Message, sizeof(g->Message), "This action requires a scrollable cursor");
 		return true;
 	} // endif's
 
@@ -762,7 +787,7 @@ bool TDBJDBC::ReadKey(PGLOBAL g, OPVAL op, const key_range *kr)
 
 			if (To_CondFil)
 				if (Query->Append(" AND ") || Query->Append(To_CondFil->Body)) {
-					strcpy(g->Message, "Readkey: Out of memory");
+					safe_strcpy(g->Message, sizeof(g->Message), "Readkey: Out of memory");
 					return true;
 				} // endif Append
 
@@ -895,7 +920,7 @@ int TDBJDBC::WriteDB(PGLOBAL g)
 	} // endfor colp
 
 	if (unlikely(Query->IsTruncated())) {
-		strcpy(g->Message, "WriteDB: Out of memory");
+		safe_strcpy(g->Message, sizeof(g->Message), "WriteDB: Out of memory");
 		return RC_FX;
 	} // endif Query
 
@@ -927,7 +952,7 @@ int TDBJDBC::DeleteDB(PGLOBAL g, int irc)
 		// Send the DELETE (all) command to the remote table
 		if (Jcp->ExecuteUpdate(Query->GetStr()) == RC_OK) {
 			AftRows = Jcp->m_Aff;
-			sprintf(g->Message, "%s: %d affected rows", TableName, AftRows);
+			snprintf(g->Message, sizeof(g->Message), "%s: %d affected rows", TableName, AftRows);
 
 			if (trace(1))
 				htrc("%s\n", g->Message);
@@ -955,7 +980,7 @@ void TDBJDBC::CloseDB(PGLOBAL g)
 
 	if (!Werr && 
 		(Mode == MODE_INSERT || Mode == MODE_UPDATE || Mode == MODE_DELETE)) {
-		sprintf(g->Message, "%s: %d affected rows", TableName, AftRows);
+		snprintf(g->Message, sizeof(g->Message), "%s: %d affected rows", TableName, AftRows);
 
 		if (trace(1))
 			htrc("%s\n", g->Message);
@@ -1088,13 +1113,13 @@ PCMD TDBXJDC::MakeCMD(PGLOBAL g)
 				(To_CondFil->Op == OP_EQ || To_CondFil->Op == OP_IN)) {
 				xcmd = To_CondFil->Cmds;
 			} else
-				strcpy(g->Message, "Invalid command specification filter");
+				safe_strcpy(g->Message, sizeof(g->Message), "Invalid command specification filter");
 
 		} else
-			strcpy(g->Message, "No command column in select list");
+			safe_strcpy(g->Message, sizeof(g->Message), "No command column in select list");
 
 	} else if (!Srcdef)
-		strcpy(g->Message, "No Srcdef default command");
+		safe_strcpy(g->Message, sizeof(g->Message), "No Srcdef default command");
 	else
 		xcmd = new(g) CMD(g, Srcdef);
 
@@ -1127,7 +1152,7 @@ bool TDBXJDC::OpenDB(PGLOBAL g)
 		this, Tdb_No, Use, Mode);
 
 	if (Use == USE_OPEN) {
-		strcpy(g->Message, "Multiple execution is not allowed");
+		safe_strcpy(g->Message, sizeof(g->Message), "Multiple execution is not allowed");
 		return true;
 	} // endif use
 
@@ -1149,7 +1174,7 @@ bool TDBXJDC::OpenDB(PGLOBAL g)
 	Use = USE_OPEN;       // Do it now in case we are recursively called
 
 	if (Mode != MODE_READ && Mode != MODE_READX) {
-		strcpy(g->Message, "No INSERT/DELETE/UPDATE of XJDBC tables");
+		safe_strcpy(g->Message, sizeof(g->Message), "No INSERT/DELETE/UPDATE of XJDBC tables");
 		return true;
 	} // endif Mode
 
@@ -1157,8 +1182,9 @@ bool TDBXJDC::OpenDB(PGLOBAL g)
 	/*  Get the command to execute.                                      */
 	/*********************************************************************/
 	if (!(Cmdlist = MakeCMD(g))) {
-		Jcp->Close();
-		return true;
+		// Next lines commented out because of CHECK TABLE
+		//Jcp->Close();
+		//return true;
 	} // endif Query
 
 	Rows = 1;
@@ -1189,8 +1215,10 @@ int TDBXJDC::ReadDB(PGLOBAL g)
 		Fpos++;                // Used for progress info
 		Cmdlist = (Nerr > Mxr) ? NULL : Cmdlist->Next;
 		return RC_OK;
-	} else
+	} else {
+		PushWarning(g, this, 1);
 		return RC_EF;
+	}	// endif Cmdlist
 
 } // end of ReadDB
 
@@ -1199,7 +1227,7 @@ int TDBXJDC::ReadDB(PGLOBAL g)
 /***********************************************************************/
 int TDBXJDC::WriteDB(PGLOBAL g)
 {
-	strcpy(g->Message, "Execsrc tables are read only");
+	safe_strcpy(g->Message, sizeof(g->Message), "Execsrc tables are read only");
 	return RC_FX;
 } // end of DeleteDB
 
@@ -1208,7 +1236,7 @@ int TDBXJDC::WriteDB(PGLOBAL g)
 /***********************************************************************/
 int TDBXJDC::DeleteDB(PGLOBAL g, int irc)
 {
-	strcpy(g->Message, "NO_XJDBC_DELETE");
+	safe_strcpy(g->Message, sizeof(g->Message), "NO_XJDBC_DELETE");
 	return RC_FX;
 } // end of DeleteDB
 
