@@ -36,7 +36,7 @@ class ha_heap: public handler
   my_bool internal_table;
 public:
   ha_heap(handlerton *hton, TABLE_SHARE *table);
-  ~ha_heap() {}
+  ~ha_heap() = default;
   handler *clone(const char *name, MEM_ROOT *mem_root);
   const char *index_type(uint inx)
   {
@@ -65,12 +65,13 @@ public:
   double scan_time()
   { return (double) (stats.records+stats.deleted) / 20.0+10; }
   double read_time(uint index, uint ranges, ha_rows rows)
-  { return (double) rows /  20.0+1; }
-
+  { return (double) rows / 20.0+1; }
+  double keyread_time(uint index, uint ranges, ha_rows rows)
+  { return (double) rows / 20.0+1; }
   int open(const char *name, int mode, uint test_if_locked);
   int close(void);
   void set_keys_for_scanning(void);
-  int write_row(uchar * buf);
+  int write_row(const uchar * buf);
   int update_row(const uchar * old_data, const uchar * new_data);
   int delete_row(const uchar * buf);
   virtual void get_auto_increment(ulonglong offset, ulonglong increment,
