@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 1994, 2016, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2017, 2020, MariaDB Corporation.
+Copyright (c) 2017, 2022, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -579,7 +579,6 @@ dtuple_convert_big_rec(
 	big_rec_t*	vector;
 	dfield_t*	dfield;
 	ulint		size;
-	ulint		n_fields;
 	ulint		local_prefix_len;
 
 	if (!dict_index_is_clust(index)) {
@@ -615,7 +614,7 @@ dtuple_convert_big_rec(
 	a variable-length field that yields the biggest savings when
 	stored externally */
 
-	n_fields = 0;
+	ut_d(ulint n_fields = 0);
 	ulint longest_i;
 
 	const bool mblob = entry->is_alter_metadata();
@@ -686,7 +685,7 @@ dtuple_convert_big_rec(
 				goto skip_field;
 			}
 
-			longest_i = i;
+			longest_i = i + mblob;
 			longest = savings;
 
 skip_field:
@@ -729,9 +728,8 @@ ext_write:
 		dfield_set_data(dfield, data, local_len);
 		dfield_set_ext(dfield);
 
-		n_fields++;
 		(*n_ext)++;
-		ut_ad(n_fields < dtuple_get_n_fields(entry));
+		ut_ad(++n_fields < dtuple_get_n_fields(entry));
 
 		if (upd && !upd->is_modified(longest_i)) {
 
