@@ -94,6 +94,12 @@ struct fil_addr_t;
 #define MAX_PAGE_HASH_LOCKS	1024	/*!< The maximum number of
 					page_hash locks */
 
+/** If LRU list of a buf_pool is less than this size then LRU eviction
+should not happen. This is because when we do LRU flushing we also put
+the blocks on free list. If LRU list is very small then we can end up
+in thrashing. */
+#define BUF_LRU_MIN_LEN		256
+
 extern	buf_pool_t*	buf_pool_ptr;	/*!< The buffer pools
 					of the database */
 
@@ -1394,7 +1400,6 @@ buf_get_nth_chunk_block(
 if needed.
 @param[in]	size	size in bytes
 @return	aligned size */
-UNIV_INLINE
 ulint
 buf_pool_size_align(
 	ulint	size);
@@ -1863,7 +1868,7 @@ public:
 		, m_hp() {}
 
 	/** Destructor */
-	virtual ~HazardPointer() {}
+	virtual ~HazardPointer() = default;
 
 	/** Get current value */
 	buf_page_t* get() const
@@ -1917,7 +1922,7 @@ public:
 		HazardPointer(buf_pool, mutex) {}
 
 	/** Destructor */
-	~FlushHp() override {}
+	~FlushHp() override = default;
 
 	/** Adjust the value of hp. This happens when some
 	other thread working on the same list attempts to
@@ -1938,7 +1943,7 @@ public:
 		HazardPointer(buf_pool, mutex) {}
 
 	/** Destructor */
-	~LRUHp() override {}
+	~LRUHp() override = default;
 
 	/** Adjust the value of hp. This happens when some
 	other thread working on the same list attempts to
@@ -1962,7 +1967,7 @@ public:
 		LRUHp(buf_pool, mutex) {}
 
 	/** Destructor */
-	~LRUItr() override {}
+	~LRUItr() override = default;
 
 	/** Selects from where to start a scan. If we have scanned
 	too deep into the LRU list it resets the value to the tail
@@ -2485,7 +2490,7 @@ struct	CheckUnzipLRUAndLRUList {
 };
 #endif /* UNIV_DEBUG || defined UNIV_BUF_DEBUG */
 
-#include "buf0buf.ic"
+#include "buf0buf.inl"
 
 #endif /* !UNIV_INNOCHECKSUM */
 
