@@ -60,7 +60,7 @@ public:
 		ulint used() const
 			MY_ATTRIBUTE((warn_unused_result))
 		{
-			return(static_cast<ulint>(m_used & ~DYN_BLOCK_FULL_FLAG));
+			return m_used;
 		}
 
 		/**
@@ -153,8 +153,7 @@ public:
 		/** Storage */
 		byte		m_data[MAX_DATA_SIZE];
 
-		/** number of data bytes used in this block;
-		DYN_BLOCK_FULL_FLAG is set when the block becomes full */
+		/** number of data bytes used in this block */
 		uint32_t	m_used;
 
 		friend class mtr_buf_t;
