@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 1996, 2015, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2018, MariaDB Corporation.
+Copyright (c) 2018, 2019, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -175,6 +175,26 @@ operator<<(std::ostream& out, const lock_rec_t& lock)
 #endif
 /* @} */
 
+/**
+Checks if the `mode` is LOCK_S or LOCK_X (possibly ORed with LOCK_WAIT or
+LOCK_REC) which means the lock is a
+Next Key Lock, a.k.a. LOCK_ORDINARY, as opposed to Predicate Lock,
+GAP lock, Insert Intention or Record Lock.
+@param  mode  A mode and flags, of a lock.
+@return true if the only bits set in `mode` are LOCK_S or LOCK_X and optionally
+LOCK_WAIT or LOCK_REC */
+static inline bool lock_mode_is_next_key_lock(ulint mode)
+{
+  static_assert(LOCK_ORDINARY == 0, "LOCK_ORDINARY must be 0 (no flags)");
+  ut_ad((mode & LOCK_TABLE) == 0);
+  mode&= ~(LOCK_WAIT | LOCK_REC);
+  ut_ad((mode & LOCK_WAIT) == 0);
+  ut_ad((mode & LOCK_TYPE_MASK) == 0);
+  ut_ad(((mode & ~(LOCK_MODE_MASK)) == LOCK_ORDINARY) ==
+        (mode == LOCK_S || mode == LOCK_X));
+  return (mode & ~(LOCK_MODE_MASK)) == LOCK_ORDINARY;
+}
+
 /** Lock struct; protected by lock_sys.mutex */
 struct ib_lock_t
 {
@@ -190,10 +210,14 @@ struct ib_lock_t
 					lock. The link node in a singly linked
 					list, used during hashing. */
 
-	/* Statistics for how long lock has been held and time
-	how long this lock had to be waited before it was granted */
-	time_t		requested_time; /*!< Lock request time */
-	ulint		wait_time;	/*!< Time waited this lock or 0 */
+	/** time(NULL) of the lock request creation.
+	Used for computing wait_time and diagnostics only.
+	Note: bogus durations may be reported
+	when the system time is adjusted! */
+	time_t		requested_time;
+	/** Cumulated wait time in seconds.
+	Note: may be bogus when the system time is adjusted! */
+	ulint		wait_time;
 
 	union {
 		lock_table_t	tab_lock;/*!< table lock */
@@ -227,6 +251,13 @@ struct ib_lock_t
 		return(type_mode & LOCK_REC_NOT_GAP);
 	}
 
+	/** @return true if the lock is a Next Key Lock */
+	bool is_next_key_lock() const
+	{
+		return is_record_lock()
+		  && lock_mode_is_next_key_lock(type_mode);
+	}
+
 	bool is_insert_intention() const
 	{
 		return(type_mode & LOCK_INSERT_INTENTION);
