@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 2012, 2015, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2017, MariaDB Corporation.
+Copyright (c) 2017, 2022, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -38,10 +38,10 @@ Created 2012-03-24 Sunny Bains.
 @param[in]	T		The resulting typedef alias */
 #define UT_MUTEX_TYPE(M, P, T) typedef PolicyMutex<M<P> > T;
 
-# ifdef HAVE_IB_LINUX_FUTEX
+# ifdef __linux__
 UT_MUTEX_TYPE(TTASFutexMutex, GenericPolicy, FutexMutex);
 UT_MUTEX_TYPE(TTASFutexMutex, BlockMutexPolicy, BlockFutexMutex);
-# endif /* HAVE_IB_LINUX_FUTEX */
+# endif /* __linux__ */
 
 UT_MUTEX_TYPE(TTASMutex, GenericPolicy, SpinMutex);
 UT_MUTEX_TYPE(TTASMutex, BlockMutexPolicy, BlockSpinMutex);
@@ -111,10 +111,10 @@ in the debug version. */
 class MutexMonitor {
 public:
 	/** Constructor */
-	MutexMonitor() { }
+	MutexMonitor() = default;
 
 	/** Destructor */
-	~MutexMonitor() { }
+	~MutexMonitor() = default;
 
 	/** Enable the mutex monitoring */
 	void enable();
@@ -140,16 +140,10 @@ public:
 
 			/* Some of the slots will be null in non-debug mode */
 
-			if (*it == NULL) {
-				continue;
-			}
-
-			latch_meta_t*	latch_meta = *it;
-
-			bool	ret = callback(*latch_meta);
-
-			if (!ret) {
-				return(ret);
+			if (latch_meta_t* l= *it) {
+				if (!callback(*l)) {
+					return false;
+				}
 			}
 		}
 
