@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 1996, 2016, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2014, 2018, MariaDB Corporation.
+Copyright (c) 2014, 2021, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -58,7 +58,7 @@ lock_wait_table_print(void)
 			(ulong) slot->in_use,
 			(ulong) slot->suspended,
 			slot->wait_timeout,
-			(ulong) difftime(ut_time(), slot->suspend_time));
+			(ulong) difftime(time(NULL), slot->suspend_time));
 	}
 }
 
@@ -155,7 +155,7 @@ lock_wait_table_reserve_slot(
 
 			os_event_reset(slot->event);
 			slot->suspended = TRUE;
-			slot->suspend_time = ut_time();
+			slot->suspend_time = time(NULL);
 			slot->wait_timeout = wait_timeout;
 
 			if (slot == lock_sys.last_slot) {
@@ -184,37 +184,28 @@ lock_wait_table_reserve_slot(
 check if lock timeout was for priority thread,
 as a side effect trigger lock monitor
 @param[in]    trx    transaction owning the lock
-@param[in]    locked true if trx and lock_sys.mutex is ownd
 @return	false for regular lock timeout */
 static
 bool
 wsrep_is_BF_lock_timeout(
-	const trx_t*	trx,
-	bool		locked = true)
+	const trx_t*	trx)
 {
-	if (wsrep_on_trx(trx)
-	    && wsrep_thd_is_BF(trx->mysql_thd, FALSE)
-	    && trx->error_state != DB_DEADLOCK) {
-		ib::info() << "WSREP: BF lock wait long for trx:" << ib::hex(trx->id)
+	bool long_wait= (trx->error_state != DB_DEADLOCK &&
+			 trx->is_wsrep() &&
+			 wsrep_thd_is_BF(trx->mysql_thd, false));
+	bool was_wait= true;
+
+	DBUG_EXECUTE_IF("wsrep_instrument_BF_lock_wait",
+			was_wait=false; long_wait=true;);
+
+	if (long_wait) {
+		ib::info() << "WSREP: BF lock wait long for trx:" << trx->id
 			   << " query: " << wsrep_thd_query(trx->mysql_thd);
-		if (!locked) {
-			lock_mutex_enter();
-		}
 
-		ut_ad(lock_mutex_own());
-
-		trx_print_latched(stderr, trx, 3000);
-
-		if (!locked) {
-			lock_mutex_exit();
-		}
-
-		srv_print_innodb_monitor 	= TRUE;
-		srv_print_innodb_lock_monitor 	= TRUE;
-		os_event_set(srv_monitor_event);
-		return true;
+		return was_wait;
+	} else {
+		return false;
 	}
-	return false;
 }
 #endif /* WITH_WSREP */
 
@@ -231,13 +222,8 @@ lock_wait_suspend_thread(
 				user OS thread */
 {
 	srv_slot_t*	slot;
-	double		wait_time;
 	trx_t*		trx;
 	ibool		was_declared_inside_innodb;
-	int64_t		start_time = 0;
-	int64_t		finish_time;
-	ulint		sec;
-	ulint		ms;
 	ulong		lock_wait_timeout;
 
 	trx = thr_get_trx(thr);
@@ -283,19 +269,18 @@ lock_wait_suspend_thread(
 	lock_wait_mutex_exit();
 	trx_mutex_exit(trx);
 
+	ulonglong start_time = 0;
+
 	if (thr->lock_state == QUE_THR_LOCK_ROW) {
 		srv_stats.n_lock_wait_count.inc();
 		srv_stats.n_lock_wait_current_count++;
-
-		if (ut_usectime(&sec, &ms) == -1) {
-			start_time = -1;
-		} else {
-			start_time = int64_t(sec) * 1000000 + int64_t(ms);
-		}
+		start_time = my_interval_timer();
 	}
 
 	ulint	lock_type = ULINT_UNDEFINED;
-
+#ifndef DBUG_OFF
+	ulint	lock_mode = LOCK_NONE;
+#endif
 	/* The wait_lock can be cleared by another thread when the
 	lock is released. But the wait can only be initiated by the
 	current thread which owns the transaction. Only acquire the
@@ -305,6 +290,9 @@ lock_wait_suspend_thread(
 		wait_lock = trx->lock.wait_lock;
 		if (wait_lock) {
 			lock_type = lock_get_type_low(wait_lock);
+#ifndef DBUG_OFF
+			lock_mode = lock_get_mode(wait_lock);
+#endif
 		}
 		lock_mutex_exit();
 	}
@@ -352,7 +340,16 @@ lock_wait_suspend_thread(
 		thd_wait_begin(trx->mysql_thd, THD_WAIT_TABLE_LOCK);
 	}
 
+	DEBUG_SYNC_C("lock_wait_before_suspend");
 	os_event_wait(slot->event);
+	DBUG_EXECUTE_IF("small_sleep_after_lock_wait",
+		{
+			if (lock_type == LOCK_REC && lock_mode == LOCK_X &&
+			    trx->error_state != DB_DEADLOCK &&
+			    !trx_is_interrupted(trx)) {
+				my_sleep(20000);
+			}
+		});
 
 	thd_wait_end(trx->mysql_thd);
 
@@ -371,28 +368,23 @@ lock_wait_suspend_thread(
 		row_mysql_freeze_data_dictionary(trx);
 	}
 
-	wait_time = ut_difftime(ut_time(), slot->suspend_time);
+	double wait_time = difftime(time(NULL), slot->suspend_time);
 
 	/* Release the slot for others to use */
 
 	lock_wait_table_release_slot(slot);
 
 	if (thr->lock_state == QUE_THR_LOCK_ROW) {
-		int64_t	diff_time;
-		if (start_time == -1 || ut_usectime(&sec, &ms) == -1) {
-			finish_time = -1;
-			diff_time = 0;
-		} else {
-			finish_time = int64_t(sec) * 1000000 + int64_t(ms);
-			diff_time = std::max<int64_t>(
-				0, finish_time - start_time);
-			srv_stats.n_lock_wait_time.add(diff_time);
+		const ulonglong finish_time = my_interval_timer();
 
+		if (finish_time >= start_time) {
+			const ulint diff_time = static_cast<ulint>
+				((finish_time - start_time) / 1000);
+			srv_stats.n_lock_wait_time.add(diff_time);
 			/* Only update the variable if we successfully
 			retrieved the start and finish times. See Bug#36819. */
-			if (ulint(diff_time) > lock_sys.n_lock_max_wait_time) {
-				lock_sys.n_lock_max_wait_time
-					= ulint(diff_time);
+			if (diff_time > lock_sys.n_lock_max_wait_time) {
+				lock_sys.n_lock_max_wait_time = diff_time;
 			}
 			/* Record the lock wait time for this thread */
 			thd_storage_lock_wait(trx->mysql_thd, diff_time);
@@ -412,8 +404,9 @@ lock_wait_suspend_thread(
 	if (lock_wait_timeout < 100000000
 	    && wait_time > (double) lock_wait_timeout
 #ifdef WITH_WSREP
-	    && (!wsrep_on_trx(trx) ||
-	       (!wsrep_is_BF_lock_timeout(trx, false) && trx->error_state != DB_DEADLOCK))
+	    && (!trx->is_wsrep()
+		|| (!wsrep_is_BF_lock_timeout(trx)
+		    && trx->error_state != DB_DEADLOCK))
 #endif /* WITH_WSREP */
 	    ) {
 
@@ -468,19 +461,12 @@ lock_wait_check_and_cancel(
 	const srv_slot_t*	slot)	/*!< in: slot reserved by a user
 					thread when the wait started */
 {
-	trx_t*		trx;
-	double		wait_time;
-	ib_time_t	suspend_time = slot->suspend_time;
-
 	ut_ad(lock_wait_mutex_own());
-
 	ut_ad(slot->in_use);
-
 	ut_ad(slot->suspended);
 
-	wait_time = ut_difftime(ut_time(), suspend_time);
-
-	trx = thr_get_trx(slot->thr);
+	double wait_time = difftime(time(NULL), slot->suspend_time);
+	trx_t* trx = thr_get_trx(slot->thr);
 
 	if (trx_is_interrupted(trx)
 	    || (slot->wait_timeout < 100000000
@@ -515,7 +501,6 @@ lock_wait_check_and_cancel(
 
 		trx_mutex_exit(trx);
 	}
-
 }
 
 /*********************************************************************//**
