@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 2012, 2016, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2017, 2019, MariaDB Corporation.
+Copyright (c) 2017, 2020, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -105,11 +105,17 @@ row_quiesce_write_indexes(
 	FILE*			file,	/*!< in: file to write to */
 	THD*			thd)	/*!< in/out: session */
 {
+	ulint n_indexes = 0;
+	for (const dict_index_t* index = UT_LIST_GET_FIRST(table->indexes);
+	     index; index = UT_LIST_GET_NEXT(indexes, index)) {
+		n_indexes += index->is_committed();
+	}
+
 	{
 		byte		row[sizeof(ib_uint32_t)];
 
 		/* Write the number of indexes in the table. */
-		mach_write_to_4(row, UT_LIST_GET_LEN(table->indexes));
+		mach_write_to_4(row, n_indexes);
 
 		DBUG_EXECUTE_IF("ib_export_io_write_failure_11",
 				close(fileno(file)););
@@ -131,6 +137,12 @@ row_quiesce_write_indexes(
 	     index != 0 && err == DB_SUCCESS;
 	     index = UT_LIST_GET_NEXT(indexes, index)) {
 
+		if (!index->is_committed()) {
+			continue;
+		}
+
+		ut_ad(n_indexes); ut_d(n_indexes--);
+
 		byte*		ptr;
 		byte		row[sizeof(index_id_t)
 				    + sizeof(ib_uint32_t) * 8];
@@ -201,6 +213,7 @@ row_quiesce_write_indexes(
 		err = row_quiesce_write_index_fields(index, file, thd);
 	}
 
+	ut_ad(!n_indexes);
 	return(err);
 }
 
@@ -668,13 +681,18 @@ row_quiesce_set_state(
 			    " FTS auxiliary tables will not be flushed.");
 	}
 
+	dict_index_t* clust_index = dict_table_get_first_index(table);
+
 	row_mysql_lock_data_dictionary(trx);
-	for (dict_index_t* index = dict_table_get_first_index(table);
+
+	for (dict_index_t* index = dict_table_get_next_index(clust_index);
 	     index != NULL;
 	     index = dict_table_get_next_index(index)) {
 		rw_lock_x_lock(&index->lock);
 	}
 
+	rw_lock_x_lock(&clust_index->lock);
+
 	switch (state) {
 	case QUIESCE_START:
 		break;
