@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111-1301 USA */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA */
 
 /* Written by Sergei A. Golubchik, who has a shared copyright to this code
    added support for long options (my_getopt) 22.5.2002 by Jani Tolonen */
@@ -119,11 +119,7 @@ int main(int argc,char *argv[])
     if (subkeys.i >= 0)
       weight= subkeys.f;
 
-#ifdef HAVE_SNPRINTF
     snprintf(buf,MAX_LEN,"%.*s",(int) keylen,info->lastkey_buff+1);
-#else
-    sprintf(buf,"%.*s",(int) keylen,info->lastkey_buff+1);
-#endif
     my_casedn_str(default_charset_info,buf);
     total++;
     lengths[keylen]++;
