@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA */
 
 /*
   TODO: use pthread_join instead of wait_for_thread_count_to_be_zero, like in
@@ -661,14 +661,10 @@ err:
 
 static void *test_thread(void *arg)
 {
-#ifndef DBUG_OFF
-  int param= *((int*) arg);
-#endif
-
   my_thread_init();
   {
   DBUG_ENTER("test_thread");
-  DBUG_PRINT("enter", ("param: %d", param));
+  DBUG_PRINT("enter", ("param: %d", *(int*) arg));
 
   if (!simple_read_write_test() ||
       !simple_read_change_write_read_test() ||
@@ -701,7 +697,7 @@ static char *create_tmpdir(const char *progname)
 {
   static char test_dirname[FN_REFLEN];
   char tmp_name[FN_REFLEN];
-  uint length;
+  size_t length;
 
   /* Create a temporary directory of name TMP-'executable', but without the -t extension */
   fn_format(tmp_name, progname, "", "", MY_REPLACE_DIR | MY_REPLACE_EXT);
@@ -724,7 +720,8 @@ int main(int argc __attribute__((unused)),
 {
   pthread_t tid;
   pthread_attr_t thr_attr;
-  int *param, error, pagen;
+  int *param, error;
+  size_t pagen;
   File tmp_file;
   MY_INIT(argv[0]);
 
@@ -809,7 +806,7 @@ int main(int argc __attribute__((unused)),
             errno);
     exit(1);
   }
-  DBUG_PRINT("info", ("Page cache %d pages", pagen));
+  DBUG_PRINT("info", ("Page cache %zd pages", pagen));
 
   pthread_mutex_lock(&LOCK_thread_count);
   param=(int*) malloc(sizeof(int));
