@@ -1,17 +1,24 @@
 /* Copyright (c) 2010, 2015, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
+  it under the terms of the GNU General Public License, version 2.0,
+  as published by the Free Software Foundation.
+
+  This program is also distributed with certain software (including
+  but not limited to OpenSSL) that is licensed under separate terms,
+  as designated in a particular file or component or in included license
+  documentation.  The authors of MySQL hereby grant you an additional
+  permission to link the program and your derivative works with the
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+  GNU General Public License, version 2.0, for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
-  51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA */
+  51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA */
 
 #ifndef TABLE_EVENTS_STATEMENTS_H
 #define TABLE_EVENTS_STATEMENTS_H
@@ -169,8 +176,7 @@ protected:
 
   table_events_statements_common(const PFS_engine_table_share *share, void *pos);
 
-  ~table_events_statements_common()
-  {}
+  ~table_events_statements_common() = default;
 
   void make_row_part_1(PFS_events_statements *statement,
                        sql_digest_storage *digest);
@@ -202,8 +208,7 @@ protected:
   table_events_statements_current();
 
 public:
-  ~table_events_statements_current()
-  {}
+  ~table_events_statements_current() = default;
 
 private:
   friend class table_events_statements_history;
@@ -238,8 +243,7 @@ protected:
   table_events_statements_history();
 
 public:
-  ~table_events_statements_history()
-  {}
+  ~table_events_statements_history() = default;
 
 private:
   /** Table share lock. */
@@ -271,8 +275,7 @@ protected:
   table_events_statements_history_long();
 
 public:
-  ~table_events_statements_history_long()
-  {}
+  ~table_events_statements_history_long() = default;
 
 private:
   /** Table share lock. */
