@@ -16,21 +16,9 @@
 #ifndef HS_COMPAT_H
 #define HS_COMPAT_H
 
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100213
 #define SPD_INIT_DYNAMIC_ARRAY2(A, B, C, D, E, F) \
   my_init_dynamic_array2(A, B, C, D, E, F)
 #define SPD_INIT_ALLOC_ROOT(A, B, C, D) \
   init_alloc_root(A, "spider", B, C, D)
-#elif defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100000
-#define SPD_INIT_DYNAMIC_ARRAY2(A, B, C, D, E, F) \
-  my_init_dynamic_array2(A, B, C, D, E, F)
-#define SPD_INIT_ALLOC_ROOT(A, B, C, D) \
-  init_alloc_root(A, B, C, D)
-#else
-#define SPD_INIT_DYNAMIC_ARRAY2(A, B, C, D, E, F) \
-  my_init_dynamic_array2(A, B, C, D, E)
-#define SPD_INIT_ALLOC_ROOT(A, B, C, D) \
-  init_alloc_root(A, B, C)
-#endif
 
 #endif
