@@ -17,10 +17,6 @@
 #include <my_global.h>
 #include "mysql_version.h"
 #include "spd_environ.h"
-#if MYSQL_VERSION_ID < 50500
-#include "mysql_priv.h"
-#include <mysql/plugin.h>
-#else
 #include "sql_priv.h"
 #include "probes_mysql.h"
 #include "sql_class.h"
@@ -28,7 +24,6 @@
 #include "sql_base.h"
 #include "sql_servers.h"
 #include "tztime.h"
-#endif
 #include "spd_err.h"
 #include "spd_param.h"
 #include "spd_db_include.h"
@@ -43,9 +38,7 @@
 #include "spd_udf.h"
 #include "spd_malloc.h"
 
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100004
 #define SPIDER_NEED_INIT_ONE_TABLE_FOR_FIND_TEMPORARY_TABLE
-#endif
 
 extern const char **spd_defaults_extra_file;
 extern const char **spd_defaults_file;
@@ -114,23 +107,8 @@ int spider_udf_direct_sql_create_table_list(
     } else
       break;
   }
-#if MYSQL_VERSION_ID < 50500
   if (!(direct_sql->db_names = (char**)
-    spider_bulk_malloc(spider_current_trx, 31, MYF(MY_WME | MY_ZEROFILL),
-      &direct_sql->db_names, sizeof(char*) * table_count,
-      &direct_sql->table_names, sizeof(char*) * table_count,
-      &direct_sql->tables, sizeof(TABLE*) * table_count,
-      &tmp_name_ptr, sizeof(char) * (
-        table_name_list_length +
-        thd->db_length * table_count +
-        2 * table_count
-      ),
-      &direct_sql->iop, sizeof(int) * table_count,
-      NullS))
-  )
-#else
-  if (!(direct_sql->db_names = (char**)
-    spider_bulk_malloc(spider_current_trx, 31, MYF(MY_WME | MY_ZEROFILL),
+    spider_bulk_malloc(spider_current_trx, SPD_MID_UDF_DIRECT_SQL_CREATE_TABLE_LIST_2, MYF(MY_WME | MY_ZEROFILL),
       &direct_sql->db_names, sizeof(char*) * table_count,
       &direct_sql->table_names, sizeof(char*) * table_count,
       &direct_sql->tables, sizeof(TABLE*) * table_count,
@@ -144,7 +122,6 @@ int spider_udf_direct_sql_create_table_list(
       &direct_sql->real_table_bitmap, sizeof(uchar) * ((table_count + 7) / 8),
       NullS))
   )
-#endif
     DBUG_RETURN(HA_ERR_OUT_OF_MEM);
 
   tmp_ptr = table_name_list;
@@ -236,7 +213,7 @@ int spider_udf_direct_sql_create_conn_key(
   }
 #endif
   if (!(direct_sql->conn_key = (char *)
-    spider_malloc(spider_current_trx, 9, direct_sql->conn_key_length + 1,
+    spider_malloc(spider_current_trx, SPD_MID_UDF_DIRECT_SQL_CREATE_CONN_KEY_1, direct_sql->conn_key_length + 1,
       MYF(MY_WME | MY_ZEROFILL)))
   )
     DBUG_RETURN(HA_ERR_OUT_OF_MEM);
@@ -387,6 +364,23 @@ int spider_udf_direct_sql_create_conn_key(
   DBUG_RETURN(0);
 }
 
+static inline void spider_maybe_memcpy_string(
+  char **dest,
+  char *src,
+  char *tmp,
+  uint *dest_len,
+  uint src_len)
+{
+    *dest_len= src_len;
+    if (src_len)
+    {
+      *dest= tmp;
+      memcpy(*dest, src, src_len);
+    } else
+      *dest= NULL;
+}
+
+
 SPIDER_CONN *spider_udf_direct_sql_create_conn(
   const SPIDER_DIRECT_SQL *direct_sql,
   int *error_num
@@ -411,7 +405,7 @@ SPIDER_CONN *spider_udf_direct_sql_create_conn(
   {
 #endif
     if (!(conn = (SPIDER_CONN *)
-      spider_bulk_malloc(spider_current_trx, 32, MYF(MY_WME | MY_ZEROFILL),
+      spider_bulk_malloc(spider_current_trx, SPD_MID_UDF_DIRECT_SQL_CREATE_CONN_1, MYF(MY_WME | MY_ZEROFILL),
         &conn, sizeof(*conn),
         &tmp_name, direct_sql->conn_key_length + 1,
         &tmp_host, direct_sql->tgt_host_length + 1,
@@ -434,11 +428,11 @@ SPIDER_CONN *spider_udf_direct_sql_create_conn(
       *error_num = HA_ERR_OUT_OF_MEM;
       goto error_alloc_conn;
     }
-    conn->default_database.init_calc_mem(138);
+    conn->default_database.init_calc_mem(SPD_MID_UDF_DIRECT_SQL_CREATE_CONN_2);
 #if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
   } else {
     if (!(conn = (SPIDER_CONN *)
-      spider_bulk_malloc(spider_current_trx, 33, MYF(MY_WME | MY_ZEROFILL),
+      spider_bulk_malloc(spider_current_trx, SPD_MID_UDF_DIRECT_SQL_CREATE_CONN_3, MYF(MY_WME | MY_ZEROFILL),
         &conn, sizeof(*conn),
         &tmp_name, direct_sql->conn_key_length + 1,
         &tmp_host, direct_sql->tgt_host_length + 1,
@@ -450,7 +444,7 @@ SPIDER_CONN *spider_udf_direct_sql_create_conn(
       *error_num = HA_ERR_OUT_OF_MEM;
       goto error_alloc_conn;
     }
-    conn->default_database.init_calc_mem(103);
+    conn->default_database.init_calc_mem(SPD_MID_UDF_DIRECT_SQL_CREATE_CONN_4);
   }
 #endif
 
@@ -469,74 +463,36 @@ SPIDER_CONN *spider_udf_direct_sql_create_conn(
   {
 #endif
     conn->tgt_port = direct_sql->tgt_port;
-    conn->tgt_socket_length = direct_sql->tgt_socket_length;
-    conn->tgt_socket = tmp_socket;
-    memcpy(conn->tgt_socket, direct_sql->tgt_socket,
-      direct_sql->tgt_socket_length);
-    conn->tgt_username_length = direct_sql->tgt_username_length;
-    conn->tgt_username = tmp_username;
-    memcpy(conn->tgt_username, direct_sql->tgt_username,
-      direct_sql->tgt_username_length);
-    conn->tgt_password_length = direct_sql->tgt_password_length;
-    conn->tgt_password = tmp_password;
-    memcpy(conn->tgt_password, direct_sql->tgt_password,
-      direct_sql->tgt_password_length);
-    conn->tgt_ssl_ca_length = direct_sql->tgt_ssl_ca_length;
-    if (conn->tgt_ssl_ca_length)
-    {
-      conn->tgt_ssl_ca = tmp_ssl_ca;
-      memcpy(conn->tgt_ssl_ca, direct_sql->tgt_ssl_ca,
-        direct_sql->tgt_ssl_ca_length);
-    } else
-      conn->tgt_ssl_ca = NULL;
-    conn->tgt_ssl_capath_length = direct_sql->tgt_ssl_capath_length;
-    if (conn->tgt_ssl_capath_length)
-    {
-      conn->tgt_ssl_capath = tmp_ssl_capath;
-      memcpy(conn->tgt_ssl_capath, direct_sql->tgt_ssl_capath,
-        direct_sql->tgt_ssl_capath_length);
-    } else
-      conn->tgt_ssl_capath = NULL;
-    conn->tgt_ssl_cert_length = direct_sql->tgt_ssl_cert_length;
-    if (conn->tgt_ssl_cert_length)
-    {
-      conn->tgt_ssl_cert = tmp_ssl_cert;
-      memcpy(conn->tgt_ssl_cert, direct_sql->tgt_ssl_cert,
-        direct_sql->tgt_ssl_cert_length);
-    } else
-      conn->tgt_ssl_cert = NULL;
-    conn->tgt_ssl_cipher_length = direct_sql->tgt_ssl_cipher_length;
-    if (conn->tgt_ssl_cipher_length)
-    {
-      conn->tgt_ssl_cipher = tmp_ssl_cipher;
-      memcpy(conn->tgt_ssl_cipher, direct_sql->tgt_ssl_cipher,
-        direct_sql->tgt_ssl_cipher_length);
-    } else
-      conn->tgt_ssl_cipher = NULL;
-    conn->tgt_ssl_key_length = direct_sql->tgt_ssl_key_length;
-    if (conn->tgt_ssl_key_length)
-    {
-      conn->tgt_ssl_key = tmp_ssl_key;
-      memcpy(conn->tgt_ssl_key, direct_sql->tgt_ssl_key,
-        direct_sql->tgt_ssl_key_length);
-    } else
-      conn->tgt_ssl_key = NULL;
-    conn->tgt_default_file_length = direct_sql->tgt_default_file_length;
-    if (conn->tgt_default_file_length)
-    {
-      conn->tgt_default_file = tmp_default_file;
-      memcpy(conn->tgt_default_file, direct_sql->tgt_default_file,
-        direct_sql->tgt_default_file_length);
-    } else
-      conn->tgt_default_file = NULL;
-    conn->tgt_default_group_length = direct_sql->tgt_default_group_length;
-    if (conn->tgt_default_group_length)
-    {
-      conn->tgt_default_group = tmp_default_group;
-      memcpy(conn->tgt_default_group, direct_sql->tgt_default_group,
-        direct_sql->tgt_default_group_length);
-    } else
-      conn->tgt_default_group = NULL;
+    spider_maybe_memcpy_string(
+      &conn->tgt_socket, direct_sql->tgt_socket, tmp_socket,
+      &conn->tgt_socket_length, direct_sql->tgt_socket_length);
+    spider_maybe_memcpy_string(
+      &conn->tgt_username, direct_sql->tgt_username, tmp_username,
+      &conn->tgt_username_length, direct_sql->tgt_username_length);
+    spider_maybe_memcpy_string(
+      &conn->tgt_password, direct_sql->tgt_password, tmp_password,
+      &conn->tgt_password_length, direct_sql->tgt_password_length);
+     spider_maybe_memcpy_string(
+      &conn->tgt_ssl_ca, direct_sql->tgt_ssl_ca, tmp_ssl_ca,
+      &conn->tgt_ssl_ca_length, direct_sql->tgt_ssl_ca_length);
+    spider_maybe_memcpy_string(
+      &conn->tgt_ssl_capath, direct_sql->tgt_ssl_capath, tmp_ssl_capath,
+      &conn->tgt_ssl_capath_length, direct_sql->tgt_ssl_capath_length);
+    spider_maybe_memcpy_string(
+      &conn->tgt_ssl_cert, direct_sql->tgt_ssl_cert, tmp_ssl_cert,
+      &conn->tgt_ssl_cert_length, direct_sql->tgt_ssl_cert_length);
+    spider_maybe_memcpy_string(
+      &conn->tgt_ssl_cipher, direct_sql->tgt_ssl_cipher, tmp_ssl_cipher,
+      &conn->tgt_ssl_cipher_length, direct_sql->tgt_ssl_cipher_length);
+    spider_maybe_memcpy_string(
+      &conn->tgt_ssl_key, direct_sql->tgt_ssl_key, tmp_ssl_key,
+      &conn->tgt_ssl_key_length, direct_sql->tgt_ssl_key_length);
+    spider_maybe_memcpy_string(
+      &conn->tgt_default_file, direct_sql->tgt_default_file, tmp_default_file,
+      &conn->tgt_default_file_length, direct_sql->tgt_default_file_length);
+    spider_maybe_memcpy_string(
+      &conn->tgt_default_group, direct_sql->tgt_default_group, tmp_default_group,
+      &conn->tgt_default_group_length, direct_sql->tgt_default_group_length);
     conn->tgt_ssl_vsc = direct_sql->tgt_ssl_vsc;
 #if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
   } else {
@@ -582,12 +538,8 @@ SPIDER_CONN *spider_udf_direct_sql_create_conn(
   }
 #endif
 
-#if MYSQL_VERSION_ID < 50500
-  if (pthread_mutex_init(&conn->mta_conn_mutex, MY_MUTEX_INIT_FAST))
-#else
   if (mysql_mutex_init(spd_key_mutex_mta_conn, &conn->mta_conn_mutex,
     MY_MUTEX_INIT_FAST))
-#endif
   {
     *error_num = HA_ERR_OUT_OF_MEM;
     goto error_mta_conn_mutex_init;
@@ -976,150 +928,100 @@ error:
   DBUG_RETURN(error_num);
 }
 
-#define SPIDER_PARAM_STR_LEN(name) name ## _length
-#define SPIDER_PARAM_STR(title_name, param_name) \
-  if (!strncasecmp(tmp_ptr, title_name, title_length)) \
-  { \
-    DBUG_PRINT("info",("spider " title_name " start")); \
-    if (!direct_sql->param_name) \
-    { \
-      if ((direct_sql->param_name = spider_get_string_between_quote( \
-        start_ptr, TRUE, &param_string_parse))) \
-        direct_sql->SPIDER_PARAM_STR_LEN(param_name) = \
-          strlen(direct_sql->param_name); \
-      else { \
-        error_num = param_string_parse.print_param_error(); \
-        goto error; \
-      } \
+#define SPIDER_PARAM_LEN(name) name ## _length
+#define SPIDER_PARAM_STR(title_name, param_name)                        \
+  if (!strncasecmp(parse.start_title, title_name, title_length))        \
+  {                                                                     \
+    DBUG_PRINT("info",("spider " title_name " start"));                 \
+    if (!direct_sql->param_name)                                        \
+    {                                                                   \
+      if ((direct_sql->param_name = spider_create_string(parse.start_value, \
+                                                         value_length))) \
+        direct_sql->SPIDER_PARAM_LEN(param_name) = strlen(direct_sql->param_name); \
+      else {                                                            \
+        error_num= parse.fail(true);                                    \
+        goto error;                                                     \
+      }                                                                 \
       DBUG_PRINT("info",("spider " title_name "=%s", direct_sql->param_name)); \
     } \
     break; \
   }
 #define SPIDER_PARAM_HINT_WITH_MAX(title_name, param_name, check_length, max_size, min_val, max_val) \
-  if (!strncasecmp(tmp_ptr, title_name, check_length)) \
+  if (!strncasecmp(parse.start_title, title_name, check_length)) \
   { \
     DBUG_PRINT("info",("spider " title_name " start")); \
     DBUG_PRINT("info",("spider max_size=%d", max_size)); \
-    int hint_num = atoi(tmp_ptr + check_length) - 1; \
+    int hint_num = atoi(parse.start_title + check_length) - 1; \
     DBUG_PRINT("info",("spider hint_num=%d", hint_num)); \
     DBUG_PRINT("info",("spider direct_sql->param_name=%p", \
-      direct_sql->param_name)); \
+                       direct_sql->param_name)); \
     if (direct_sql->param_name) \
     { \
       if (hint_num < 0 || hint_num >= max_size) \
       { \
-        error_num = param_string_parse.print_param_error(); \
+        error_num= parse.fail(true); \
         goto error; \
       } else if (direct_sql->param_name[hint_num] != -1) \
         break; \
-      char *hint_str = spider_get_string_between_quote(start_ptr, FALSE); \
-      if (hint_str) \
-      { \
-        direct_sql->param_name[hint_num] = atoi(hint_str); \
-        if (direct_sql->param_name[hint_num] < min_val) \
-          direct_sql->param_name[hint_num] = min_val; \
-        else if (direct_sql->param_name[hint_num] > max_val) \
-          direct_sql->param_name[hint_num] = max_val; \
-      } else { \
-        error_num = param_string_parse.print_param_error(); \
-        goto error; \
-      } \
+      direct_sql->param_name[hint_num] = atoi(parse.start_value); \
+      if (direct_sql->param_name[hint_num] < min_val) \
+        direct_sql->param_name[hint_num] = min_val; \
+      else if (direct_sql->param_name[hint_num] > max_val) \
+        direct_sql->param_name[hint_num] = max_val; \
       DBUG_PRINT("info",("spider " title_name "[%d]=%d", hint_num, \
-        direct_sql->param_name[hint_num])); \
+                         direct_sql->param_name[hint_num])); \
     } else { \
-      error_num = param_string_parse.print_param_error(); \
+      error_num= parse.fail(true); \
       goto error; \
     } \
     break; \
   }
 #define SPIDER_PARAM_INT_WITH_MAX(title_name, param_name, min_val, max_val) \
-  if (!strncasecmp(tmp_ptr, title_name, title_length)) \
+  if (!strncasecmp(parse.start_title, title_name, title_length)) \
   { \
     DBUG_PRINT("info",("spider " title_name " start")); \
     if (direct_sql->param_name == -1) \
     { \
-      if ((tmp_ptr2 = spider_get_string_between_quote( \
-        start_ptr, FALSE))) \
-      { \
-        direct_sql->param_name = atoi(tmp_ptr2); \
-        if (direct_sql->param_name < min_val) \
-          direct_sql->param_name = min_val; \
-        else if (direct_sql->param_name > max_val) \
-          direct_sql->param_name = max_val; \
-        param_string_parse.set_param_value(tmp_ptr2, \
-                                           tmp_ptr2 + \
-                                             strlen(tmp_ptr2) + 1); \
-      } else { \
-        error_num = param_string_parse.print_param_error(); \
-        goto error; \
-      } \
+      direct_sql->param_name = atoi(parse.start_value); \
+      if (direct_sql->param_name < min_val) \
+        direct_sql->param_name = min_val; \
+      else if (direct_sql->param_name > max_val) \
+        direct_sql->param_name = max_val; \
       DBUG_PRINT("info",("spider " title_name "=%d", \
-        (int) direct_sql->param_name)); \
+                         (int) direct_sql->param_name)); \
     } \
     break; \
   }
 #define SPIDER_PARAM_INT(title_name, param_name, min_val) \
-  if (!strncasecmp(tmp_ptr, title_name, title_length)) \
+  if (!strncasecmp(parse.start_title, title_name, title_length)) \
   { \
     DBUG_PRINT("info",("spider " title_name " start")); \
     if (direct_sql->param_name == -1) \
     { \
-      if ((tmp_ptr2 = spider_get_string_between_quote( \
-        start_ptr, FALSE))) \
-      { \
-        direct_sql->param_name = atoi(tmp_ptr2); \
-        if (direct_sql->param_name < min_val) \
-          direct_sql->param_name = min_val; \
-        param_string_parse.set_param_value(tmp_ptr2, \
-                                           tmp_ptr2 + \
-                                             strlen(tmp_ptr2) + 1); \
-      } else { \
-        error_num = param_string_parse.print_param_error(); \
-        goto error; \
-      } \
+      direct_sql->param_name = atoi(parse.start_value); \
+      if (direct_sql->param_name < min_val) \
+        direct_sql->param_name = min_val; \
       DBUG_PRINT("info",("spider " title_name "=%d", direct_sql->param_name)); \
     } \
     break; \
   }
 #define SPIDER_PARAM_LONGLONG(title_name, param_name, min_val) \
-  if (!strncasecmp(tmp_ptr, title_name, title_length)) \
+  if (!strncasecmp(parse.start_title, title_name, title_length)) \
   { \
     DBUG_PRINT("info",("spider " title_name " start")); \
     if (direct_sql->param_name == -1) \
     { \
-      if ((tmp_ptr2 = spider_get_string_between_quote( \
-        start_ptr, FALSE))) \
-      { \
-        direct_sql->param_name = \
-          my_strtoll10(tmp_ptr2, (char**) NULL, &error_num); \
-        if (direct_sql->param_name < min_val) \
-          direct_sql->param_name = min_val; \
-        param_string_parse.set_param_value(tmp_ptr2, \
-                                           tmp_ptr2 + \
-                                             strlen(tmp_ptr2) + 1); \
-      } else { \
-        error_num = param_string_parse.print_param_error(); \
-        goto error; \
-      } \
-      DBUG_PRINT("info",("spider " title_name "=%lld", \
-        direct_sql->param_name)); \
+      direct_sql->param_name = my_strtoll10(parse.start_value, (char**) NULL, \
+                                            &error_num); \
+      if (direct_sql->param_name < min_val) \
+        direct_sql->param_name = min_val; \
+      DBUG_PRINT("info",("spider " title_name "=%lld", direct_sql->param_name)); \
     } \
     break; \
   }
 
-int spider_udf_parse_direct_sql_param(
-  SPIDER_TRX *trx,
-  SPIDER_DIRECT_SQL *direct_sql,
-  const char *param,
-  int param_length
-) {
-  int error_num = 0, roop_count;
-  char *param_string = NULL;
-  char *sprit_ptr[2];
-  char *tmp_ptr, *tmp_ptr2, *start_ptr;
-  int title_length;
-  SPIDER_PARAM_STRING_PARSE param_string_parse;
-  DBUG_ENTER("spider_udf_parse_direct_sql_param");
+static void spider_minus_1(SPIDER_DIRECT_SQL *direct_sql)
+{
   direct_sql->tgt_port = -1;
   direct_sql->tgt_ssl_vsc = -1;
   direct_sql->table_loop_mode = -1;
@@ -1132,65 +1034,55 @@ int spider_udf_parse_direct_sql_param(
 #if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
   direct_sql->access_mode = -1;
 #endif
-#if MYSQL_VERSION_ID < 50500
-#else
   direct_sql->use_real_table = -1;
-#endif
   direct_sql->error_rw_mode = -1;
-  for (roop_count = 0; roop_count < direct_sql->table_count; roop_count++)
-    direct_sql->iop[roop_count] = -1;
+  for (int i = 0; i < direct_sql->table_count; i++)
+    direct_sql->iop[i] = -1;
+}
 
+int spider_udf_parse_direct_sql_param(
+  SPIDER_TRX *trx,
+  SPIDER_DIRECT_SQL *direct_sql,
+  const char *param,
+  int param_length
+) {
+  int error_num = 0;
+  char *param_string = NULL;
+  char *start_param;
+  int title_length, value_length;
+  SPIDER_PARAM_STRING_PARSE parse;
+  DBUG_ENTER("spider_udf_parse_direct_sql_param");
+
+  spider_minus_1(direct_sql);
   if (param_length == 0)
     goto set_default;
   DBUG_PRINT("info",("spider create param_string string"));
-  if (
-    !(param_string = spider_create_string(
-      param,
-      param_length))
-  ) {
+  if (!(param_string = spider_create_string(param, param_length)))
+  {
     error_num = HA_ERR_OUT_OF_MEM;
     my_error(ER_OUT_OF_RESOURCES, MYF(0), HA_ERR_OUT_OF_MEM);
     goto error_alloc_param_string;
   }
   DBUG_PRINT("info",("spider param_string=%s", param_string));
 
-  sprit_ptr[0] = param_string;
-  param_string_parse.init(param_string, ER_SPIDER_INVALID_UDF_PARAM_NUM);
-  while (sprit_ptr[0])
+  start_param = param_string;
+  parse.error_num = ER_SPIDER_INVALID_UDF_PARAM_NUM;
+  while (*start_param != '\0')
   {
-    if ((sprit_ptr[1] = strchr(sprit_ptr[0], ',')))
+    if (parse.locate_param_def(start_param))
     {
-      *sprit_ptr[1] = '\0';
-      sprit_ptr[1]++;
+      error_num= parse.fail(false);
+      goto error;
     }
-    tmp_ptr = sprit_ptr[0];
-    sprit_ptr[0] = sprit_ptr[1];
-    while (*tmp_ptr == ' ' || *tmp_ptr == '\r' ||
-      *tmp_ptr == '\n' || *tmp_ptr == '\t')
-      tmp_ptr++;
+    /* Null the end of the parameter value. */
+    *parse.end_value= '\0';
+    value_length= (int) (parse.end_value - parse.start_value);
 
-    if (*tmp_ptr == '\0')
-      continue;
-
-    title_length = 0;
-    start_ptr = tmp_ptr;
-    while (*start_ptr != ' ' && *start_ptr != '\'' &&
-      *start_ptr != '"' && *start_ptr != '\0' &&
-      *start_ptr != '\r' && *start_ptr != '\n' &&
-      *start_ptr != '\t')
-    {
-      title_length++;
-      start_ptr++;
-    }
-    param_string_parse.set_param_title(tmp_ptr, tmp_ptr + title_length);
-
-    switch (title_length)
+    switch (title_length = (int) (parse.end_title - parse.start_title))
     {
       case 0:
-        error_num = param_string_parse.print_param_error();
-        if (error_num)
-          goto error;
-        continue;
+        error_num= parse.fail(true);
+        goto error;
       case 3:
 #if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
         SPIDER_PARAM_INT_WITH_MAX("acm", access_mode, 0, 2);
@@ -1210,117 +1102,96 @@ int spider_udf_parse_direct_sql_param(
         SPIDER_PARAM_STR("srv", server_name);
         SPIDER_PARAM_INT_WITH_MAX("svc", tgt_ssl_vsc, 0, 1);
         SPIDER_PARAM_INT_WITH_MAX("tlm", table_loop_mode, 0, 2);
-#if MYSQL_VERSION_ID < 50500
-#else
         SPIDER_PARAM_INT_WITH_MAX("urt", use_real_table, 0, 1);
-#endif
         SPIDER_PARAM_INT("wto", net_write_timeout, 0);
-        error_num = param_string_parse.print_param_error();
+        error_num= parse.fail(true);
         goto error;
       case 4:
         SPIDER_PARAM_INT_WITH_MAX("erwm", error_rw_mode, 0, 1);
         SPIDER_PARAM_STR("host", tgt_host);
         SPIDER_PARAM_INT_WITH_MAX("port", tgt_port, 0, 65535);
         SPIDER_PARAM_STR("user", tgt_username);
-        error_num = param_string_parse.print_param_error();
+        error_num= parse.fail(true);
         goto error;
       case 6:
         SPIDER_PARAM_STR("server", server_name);
         SPIDER_PARAM_STR("socket", tgt_socket);
         SPIDER_PARAM_HINT_WITH_MAX("iop", iop, 3, direct_sql->table_count, 0, 2);
         SPIDER_PARAM_STR("ssl_ca", tgt_ssl_ca);
-        error_num = param_string_parse.print_param_error();
+        error_num= parse.fail(true);
         goto error;
       case 7:
         SPIDER_PARAM_STR("wrapper", tgt_wrapper);
         SPIDER_PARAM_STR("ssl_key", tgt_ssl_key);
-        error_num = param_string_parse.print_param_error();
+        error_num= parse.fail(true);
         goto error;
       case 8:
         SPIDER_PARAM_STR("database", tgt_default_db_name);
         SPIDER_PARAM_STR("password", tgt_password);
         SPIDER_PARAM_LONGLONG("priority", priority, 0);
         SPIDER_PARAM_STR("ssl_cert", tgt_ssl_cert);
-        error_num = param_string_parse.print_param_error();
+        error_num= parse.fail(true);
         goto error;
       case 10:
         SPIDER_PARAM_STR("ssl_cipher", tgt_ssl_cipher);
         SPIDER_PARAM_STR("ssl_capath", tgt_ssl_capath);
-        error_num = param_string_parse.print_param_error();
+        error_num= parse.fail(true);
         goto error;
       case 11:
 #if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
         SPIDER_PARAM_INT_WITH_MAX("access_mode", access_mode, 0, 2);
 #endif
-        error_num = param_string_parse.print_param_error();
+        error_num= parse.fail(true);
         goto error;
       case 12:
         SPIDER_PARAM_STR("default_file", tgt_default_file);
-        error_num = param_string_parse.print_param_error();
+        error_num= parse.fail(true);
         goto error;
       case 13:
         SPIDER_PARAM_STR("default_group", tgt_default_group);
         SPIDER_PARAM_INT_WITH_MAX("error_rw_mode", error_rw_mode, 0, 1);
-        error_num = param_string_parse.print_param_error();
+        error_num= parse.fail(true);
         goto error;
       case 14:
-#if MYSQL_VERSION_ID < 50500
-#else
         SPIDER_PARAM_INT_WITH_MAX("use_real_table", use_real_table, 0, 1);
-#endif
-        error_num = param_string_parse.print_param_error();
+        error_num= parse.fail(true);
         goto error;
       case 15:
         SPIDER_PARAM_INT_WITH_MAX("table_loop_mode", table_loop_mode, 0, 2);
         SPIDER_PARAM_INT("connect_timeout", connect_timeout, 0);
-        error_num = param_string_parse.print_param_error();
+        error_num= parse.fail(true);
         goto error;
       case 16:
         SPIDER_PARAM_LONGLONG("bulk_insert_rows", bulk_insert_rows, 1);
         SPIDER_PARAM_INT("net_read_timeout", net_read_timeout, 0);
-        error_num = param_string_parse.print_param_error();
+        error_num= parse.fail(true);
         goto error;
       case 17:
         SPIDER_PARAM_INT("net_write_timeout", net_write_timeout, 0);
-        error_num = param_string_parse.print_param_error();
+        error_num= parse.fail(true);
         goto error;
       case 18:
         SPIDER_PARAM_INT_WITH_MAX(
           "connection_channel", connection_channel, 0, 63);
-        error_num = param_string_parse.print_param_error();
+        error_num= parse.fail(true);
         goto error;
       case 22:
         SPIDER_PARAM_INT_WITH_MAX("ssl_verify_server_cert", tgt_ssl_vsc, 0, 1);
-        error_num = param_string_parse.print_param_error();
+        error_num= parse.fail(true);
         goto error;
       default:
-        error_num = param_string_parse.print_param_error();
+        error_num= parse.fail(true);
         goto error;
     }
-
-    /* Verify that the remainder of the parameter value is whitespace */
-    if ((error_num = param_string_parse.has_extra_parameter_values()))
-      goto error;
+    /* Restore delim */
+    *parse.end_value= parse.delim_value;
   }
 
 set_default:
-  if ((error_num = spider_udf_set_direct_sql_param_default(
-    trx,
-    direct_sql
-  )))
-    goto error;
-
-  if (param_string)
-  {
-    spider_free(spider_current_trx, param_string, MYF(0));
-  }
-  DBUG_RETURN(0);
-
+  error_num = spider_udf_set_direct_sql_param_default(trx, direct_sql);
 error:
   if (param_string)
-  {
     spider_free(spider_current_trx, param_string, MYF(0));
-  }
 error_alloc_param_string:
   DBUG_RETURN(error_num);
 }
@@ -1452,25 +1323,10 @@ int spider_udf_set_direct_sql_param_default(
     }
   }
 
-  if (direct_sql->table_loop_mode == -1)
-    direct_sql->table_loop_mode = 0;
   if (direct_sql->priority == -1)
     direct_sql->priority = 1000000;
-  if (direct_sql->connect_timeout == -1)
-    direct_sql->connect_timeout = 6;
-  if (direct_sql->net_read_timeout == -1)
-    direct_sql->net_read_timeout = 600;
-  if (direct_sql->net_write_timeout == -1)
-    direct_sql->net_write_timeout = 600;
-  if (direct_sql->bulk_insert_rows == -1)
-    direct_sql->bulk_insert_rows = 3000;
   if (direct_sql->connection_channel == -1)
     direct_sql->connection_channel = 0;
-#if MYSQL_VERSION_ID < 50500
-#else
-  if (direct_sql->use_real_table == -1)
-    direct_sql->use_real_table = 0;
-#endif
   if (direct_sql->error_rw_mode == -1)
     direct_sql->error_rw_mode = 0;
   for (roop_count = 0; roop_count < direct_sql->table_count; roop_count++)
@@ -1502,14 +1358,11 @@ void spider_udf_free_direct_sql_alloc(
     pthread_mutex_unlock(direct_sql->bg_mutex);
   }
 #endif
-#if MYSQL_VERSION_ID < 50500
-#else
   if (direct_sql->real_table_used && direct_sql->open_tables_thd)
   {
     spider_sys_close_table(direct_sql->open_tables_thd,
       &direct_sql->open_tables_backup);
   }
-#endif
   if (direct_sql->server_name)
   {
     spider_free(spider_current_trx, direct_sql->server_name, MYF(0));
@@ -1593,15 +1446,12 @@ long long spider_direct_sql_body(
   char *sql;
   TABLE_LIST table_list;
   SPIDER_BG_DIRECT_SQL *bg_direct_sql;
-#if MYSQL_VERSION_ID < 50500
-#else
   TABLE_LIST *real_table_list_last = NULL;
   uint use_real_table = 0;
-#endif
   DBUG_ENTER("spider_direct_sql_body");
   SPIDER_BACKUP_DASTATUS;
   if (!(direct_sql = (SPIDER_DIRECT_SQL *)
-    spider_bulk_malloc(spider_current_trx, 34, MYF(MY_WME | MY_ZEROFILL),
+    spider_bulk_malloc(spider_current_trx, SPD_MID_DIRECT_SQL_BODY_1, MYF(MY_WME | MY_ZEROFILL),
       &direct_sql, sizeof(SPIDER_DIRECT_SQL),
       &sql, sizeof(char) * args->lengths[0],
       NullS))
@@ -1688,11 +1538,8 @@ long long spider_direct_sql_body(
 #if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
   }
 #endif
-#if MYSQL_VERSION_ID < 50500
-#else
   use_real_table = spider_param_udf_ds_use_real_table(thd,
     direct_sql->use_real_table);
-#endif
   for (roop_count = 0; roop_count < direct_sql->table_count; roop_count++)
   {
 #ifdef SPIDER_NEED_INIT_ONE_TABLE_FOR_FIND_TEMPORARY_TABLE
@@ -1723,19 +1570,14 @@ long long spider_direct_sql_body(
     if (!(direct_sql->tables[roop_count] =
       spider_find_temporary_table(thd, &table_list)))
     {
-#if MYSQL_VERSION_ID < 50500
-#else
       if (!use_real_table)
       {
-#endif
         error_num = ER_SPIDER_UDF_TMP_TABLE_NOT_FOUND_NUM;
         my_printf_error(ER_SPIDER_UDF_TMP_TABLE_NOT_FOUND_NUM,
           ER_SPIDER_UDF_TMP_TABLE_NOT_FOUND_STR,
           MYF(0), SPIDER_TABLE_LIST_db_str(&table_list),
           SPIDER_TABLE_LIST_table_name_str(&table_list));
         goto error;
-#if MYSQL_VERSION_ID < 50500
-#else
       }
       TABLE_LIST *tables = &direct_sql->table_list[roop_count];
       tables->mdl_request.init(MDL_key::TABLE,
@@ -1751,7 +1593,6 @@ long long spider_direct_sql_body(
       real_table_list_last = tables;
       spider_set_bit(direct_sql->real_table_bitmap, roop_count);
       direct_sql->real_table_used = TRUE;
-#endif
     }
   }
   if ((error_num = spider_udf_direct_sql_create_conn_key(direct_sql)))
@@ -1862,28 +1703,20 @@ my_bool spider_direct_sql_init_body(
   if (bg)
   {
     if (!(bg_direct_sql = (SPIDER_BG_DIRECT_SQL *)
-      spider_malloc(spider_current_trx, 10, sizeof(SPIDER_BG_DIRECT_SQL),
+      spider_malloc(spider_current_trx, SPD_MID_DIRECT_SQL_INIT_BODY_1, sizeof(SPIDER_BG_DIRECT_SQL),
       MYF(MY_WME | MY_ZEROFILL)))
     ) {
       strcpy(message, "spider_bg_direct_sql() out of memory");
       goto error;
     }
-#if MYSQL_VERSION_ID < 50500
-    if (pthread_mutex_init(&bg_direct_sql->bg_mutex, MY_MUTEX_INIT_FAST))
-#else
     if (mysql_mutex_init(spd_key_mutex_bg_direct_sql,
       &bg_direct_sql->bg_mutex, MY_MUTEX_INIT_FAST))
-#endif
     {
       strcpy(message, "spider_bg_direct_sql() out of memory");
       goto error_mutex_init;
     }
-#if MYSQL_VERSION_ID < 50500
-    if (pthread_cond_init(&bg_direct_sql->bg_cond, NULL))
-#else
     if (mysql_cond_init(spd_key_cond_bg_direct_sql,
       &bg_direct_sql->bg_cond, NULL))
-#endif
     {
       strcpy(message, "spider_bg_direct_sql() out of memory");
       goto error_cond_init;
