@@ -17,16 +17,12 @@
 #include <my_global.h>
 #include "mysql_version.h"
 #include "spd_environ.h"
-#if MYSQL_VERSION_ID < 50500
-#include "mysql_priv.h"
-#include <mysql/plugin.h>
-#else
 #include "sql_priv.h"
 #include "probes_mysql.h"
 #include "sql_class.h"
 #include "sql_partition.h"
+#include "sql_select.h"
 #include "ha_partition.h"
-#endif
 #include "sql_common.h"
 #include <errmsg.h>
 #include "spd_err.h"
@@ -46,11 +42,9 @@ extern SPIDER_DBTON spider_dbton[SPIDER_DBTON_SIZE];
 
 spider_fields::spider_fields() :
   dbton_count(0), current_dbton_num(0),
-  table_count(0), current_table_num(0), table_holder(NULL),
+  table_count(0), table_holder(NULL),
   first_link_idx_chain(NULL), last_link_idx_chain(NULL), current_link_idx_chain(NULL),
-  first_conn_holder(NULL), last_conn_holder(NULL), current_conn_holder(NULL),
-  first_field_holder(NULL), last_field_holder(NULL), current_field_holder(NULL),
-  first_field_chain(NULL), last_field_chain(NULL), current_field_chain(NULL)
+  first_conn_holder(NULL), last_conn_holder(NULL), current_conn_holder(NULL)
 {
   DBUG_ENTER("spider_fields::spider_fields");
   DBUG_PRINT("info",("spider this=%p", this));
@@ -69,24 +63,6 @@ spider_fields::~spider_fields()
       spider_free(spider_current_trx, current_link_idx_chain, MYF(0));
     }
   }
-  if (first_field_chain)
-  {
-    while ((current_field_chain = first_field_chain))
-    {
-      first_field_chain = current_field_chain->next;
-      spider_free(spider_current_trx, current_field_chain, MYF(0));
-    }
-  }
-  if (first_field_holder)
-  {
-    while ((current_field_holder = first_field_holder))
-    {
-      first_field_holder = current_field_holder->next;
-      spider_free(spider_current_trx, current_field_holder, MYF(0));
-    }
-  }
-  if (table_holder)
-    spider_free(spider_current_trx, table_holder, MYF(0));
   if (first_conn_holder)
   {
     while ((current_conn_holder = first_conn_holder))
@@ -368,7 +344,7 @@ SPIDER_LINK_IDX_CHAIN *spider_fields::create_link_idx_chain(
   DBUG_ENTER("spider_fields::create_link_idx_chain");
   DBUG_PRINT("info",("spider this=%p", this));
   DBUG_RETURN((SPIDER_LINK_IDX_CHAIN *)
-    spider_malloc(spider_current_trx, 254, sizeof(SPIDER_LINK_IDX_CHAIN),
+    spider_malloc(spider_current_trx, SPD_MID_FIELDS_CREATE_LINK_IDX_CHAIN_1, sizeof(SPIDER_LINK_IDX_CHAIN),
     MYF(MY_WME | MY_ZEROFILL)));
 }
 
@@ -475,7 +451,6 @@ int spider_fields::get_ok_link_idx(
 
 void spider_fields::set_first_link_idx(
 ) {
-  SPIDER_TABLE_HOLDER *table_holder;
   SPIDER_LINK_IDX_HOLDER *link_idx_holder;
   SPIDER_LINK_IDX_CHAIN *link_idx_chain;
   uint dbton_id;
@@ -497,11 +472,10 @@ void spider_fields::set_first_link_idx(
     DBUG_ASSERT(link_idx_chain);
     set_pos_to_first_table_on_link_idx_chain(link_idx_chain);
 
-    set_pos_to_first_table_holder();
-    while ((table_holder = get_next_table_holder()))
+    for (uint i= 0; i < table_count; i++)
     {
       link_idx_holder = get_next_table_on_link_idx_chain(link_idx_chain);
-      spider = table_holder->spider;
+      spider = table_holder[i].spider;
       dbton_hdl = spider->dbton_handler[dbton_id];
       dbton_hdl->first_link_idx = link_idx_holder->link_idx;
     }
@@ -558,7 +532,7 @@ SPIDER_LINK_IDX_HOLDER *spider_fields::create_link_idx_holder(
   DBUG_ENTER("spider_fields::create_link_idx_holder");
   DBUG_PRINT("info",("spider this=%p", this));
   DBUG_RETURN((SPIDER_LINK_IDX_HOLDER *)
-    spider_malloc(spider_current_trx, 253, sizeof(SPIDER_LINK_IDX_HOLDER),
+    spider_malloc(spider_current_trx, SPD_MID_FIELDS_CREATE_LINK_IDX_HOLDER_1, sizeof(SPIDER_LINK_IDX_HOLDER),
     MYF(MY_WME | MY_ZEROFILL)));
 }
 
@@ -636,7 +610,7 @@ SPIDER_CONN_HOLDER *spider_fields::create_conn_holder(
   DBUG_ENTER("spider_fields::create_conn_holder");
   DBUG_PRINT("info",("spider this=%p", this));
   return_conn_holder = (SPIDER_CONN_HOLDER *)
-    spider_bulk_malloc(spider_current_trx, 252, MYF(MY_WME | MY_ZEROFILL),
+    spider_bulk_malloc(spider_current_trx, SPD_MID_FIELDS_CREATE_CONN_HOLDER_1, MYF(MY_WME | MY_ZEROFILL),
       &return_conn_holder, sizeof(SPIDER_CONN_HOLDER),
       &table_link_idx_holder,
         table_count * sizeof(SPIDER_TABLE_LINK_IDX_HOLDER),
@@ -650,24 +624,6 @@ SPIDER_CONN_HOLDER *spider_fields::create_conn_holder(
   DBUG_RETURN(return_conn_holder);
 }
 
-void spider_fields::set_pos_to_first_conn_holder(
-) {
-  DBUG_ENTER("spider_fields::set_pos_to_first_conn_holder");
-  DBUG_PRINT("info",("spider this=%p", this));
-  current_conn_holder = first_conn_holder;
-  DBUG_VOID_RETURN;
-}
-
-SPIDER_CONN_HOLDER *spider_fields::get_next_conn_holder(
-) {
-  SPIDER_CONN_HOLDER *return_conn_holder = current_conn_holder;
-  DBUG_ENTER("spider_fields::get_next_conn_holder");
-  DBUG_PRINT("info",("spider this=%p", this));
-  if (current_conn_holder)
-    current_conn_holder = current_conn_holder->next;
-  DBUG_RETURN(return_conn_holder);
-}
-
 bool spider_fields::has_conn_holder(
 ) {
   DBUG_ENTER("spider_fields::has_conn_holder");
@@ -859,25 +815,24 @@ void spider_fields::free_conn_holder(
       }
     }
   }
-  conn_holder_arg->conn->conn_holder_for_direct_join = NULL;
+  if (conn_holder_arg->conn)
+    conn_holder_arg->conn->conn_holder_for_direct_join = NULL;
   DBUG_PRINT("info",("spider free conn_holder=%p", conn_holder_arg));
   spider_free(spider_current_trx, conn_holder_arg, MYF(0));
   DBUG_VOID_RETURN;
 }
 
-SPIDER_TABLE_HOLDER *spider_fields::add_table(
-  ha_spider *spider_arg
+/* Add the table associated with an ha_spider to a table_holder.
+Return the table_holder. */
+static SPIDER_TABLE_HOLDER *spider_add_table_holder(
+  ha_spider *spider_arg,
+  SPIDER_TABLE_HOLDER *table_holder
 ) {
   spider_string *str;
   uint length;
   char tmp_buf[SPIDER_SQL_INT_LEN + 2];
   SPIDER_TABLE_HOLDER *return_table_holder;
-  SPIDER_FIELD_HOLDER *field_holder;
-  TABLE *table = spider_arg->get_table();
-  Field *field;
   DBUG_ENTER("spider_fields::add_table");
-  DBUG_PRINT("info",("spider this=%p", this));
-  DBUG_PRINT("info",("spider table_count=%u", table_count));
   DBUG_PRINT("info",("spider idx_for_direct_join=%u",
     spider_arg->idx_for_direct_join));
   length = my_sprintf(tmp_buf, (tmp_buf, "t%u",
@@ -896,90 +851,49 @@ SPIDER_TABLE_HOLDER *spider_fields::add_table(
   return_table_holder->spider = spider_arg;
   return_table_holder->alias = str;
 
-  set_pos_to_first_field_holder();
-  while ((field_holder = get_next_field_holder()))
-  {
-    if (!field_holder->spider)
-    {
-      field = field_holder->field;
-      if (
-        field->field_index < table->s->fields &&
-        field == table->field[field->field_index]
-      ) {
-        field_holder->spider = spider_arg;
-        field_holder->alias = str;
-      }
-    }
-  }
   DBUG_RETURN(return_table_holder);
 }
 
-/**
-  Verify that all fields in the query are members of tables that are in the
-  query.
-
-  @return TRUE              All fields in the query are members of tables
-                            that are in the query.
-          FALSE             At least one field in the query is not a
-                            member of a table that is in the query.
-*/
-
-bool spider_fields::all_query_fields_are_query_table_members()
+/* Return the table that field belongs to, or NULL if none exists. */
+SPIDER_TABLE_HOLDER *spider_fields::find_table(Field *field)
 {
-  SPIDER_FIELD_HOLDER *field_holder;
-  DBUG_ENTER("spider_fields::all_query_fields_are_query_table_members");
-  DBUG_PRINT("info",("spider this=%p", this));
-
-  set_pos_to_first_field_holder();
-  while ((field_holder = get_next_field_holder()))
-  {
-    if (!field_holder->spider)
-    {
-      DBUG_PRINT("info", ("spider field is not a member of a query table"));
-      DBUG_RETURN(FALSE);
-    }
-  }
-
-  DBUG_RETURN(TRUE);
+  for (uint i = 0; i < table_count; i++)
+    if (field->table == table_holder[i].table)
+      return &table_holder[i];
+  return NULL;
 }
 
-int spider_fields::create_table_holder(
+void spider_fields::set_table_holder(SPIDER_TABLE_HOLDER *table_holder_arg,
+                                     uint table_count_arg)
+{
+  table_holder= table_holder_arg;
+  table_count= table_count_arg;
+}
+
+/* Allocate space for table_count_arg table holders. */
+static SPIDER_TABLE_HOLDER *spider_create_table_holder(
   uint table_count_arg
 ) {
-  DBUG_ENTER("spider_fields::create_table_holder");
-  DBUG_PRINT("info",("spider this=%p", this));
-  DBUG_ASSERT(!table_holder);
+  SPIDER_TABLE_HOLDER* table_holder;
+  DBUG_ENTER("spider_create_table_holder");
+  if (table_count_arg == 0)
+    DBUG_RETURN(0);
   table_holder = (SPIDER_TABLE_HOLDER *)
-    spider_malloc(spider_current_trx, 249,
+    spider_malloc(spider_current_trx, SPD_MID_CREATE_TABLE_HOLDER_1,
     table_count_arg * sizeof(SPIDER_TABLE_HOLDER),
     MYF(MY_WME | MY_ZEROFILL));
-  if (!table_holder)
-    DBUG_RETURN(HA_ERR_OUT_OF_MEM);
-  table_count = table_count_arg;
-  current_table_num = 0;
-  DBUG_RETURN(0);
+  DBUG_RETURN(table_holder);
 }
 
-void spider_fields::set_pos_to_first_table_holder(
-) {
-  DBUG_ENTER("spider_fields::set_pos_to_first_table_holder");
-  DBUG_PRINT("info",("spider this=%p", this));
-  current_table_num = 0;
-  DBUG_VOID_RETURN;
-}
-
-SPIDER_TABLE_HOLDER *spider_fields::get_next_table_holder(
-) {
-  SPIDER_TABLE_HOLDER *return_table_holder;
-  DBUG_ENTER("spider_fields::get_next_table_holder");
-  DBUG_PRINT("info",("spider this=%p", this));
-  if (current_table_num >= table_count)
-    DBUG_RETURN(NULL);
-  return_table_holder = &table_holder[current_table_num];
-  ++current_table_num;
-  DBUG_RETURN(return_table_holder);
+/* Return pointer to the first table holder. */
+SPIDER_TABLE_HOLDER *spider_fields::get_first_table_holder()
+{
+  DBUG_ENTER("spider_fields::get_first_spider");
+  DBUG_RETURN(table_holder);
 }
 
+/* Return the first table holder associated with a given table, or
+NULL if not found. */
 SPIDER_TABLE_HOLDER *spider_fields::get_table_holder(TABLE *table)
 {
   uint table_num;
@@ -999,117 +913,12 @@ uint spider_fields::get_table_count()
   DBUG_RETURN(table_count);
 }
 
-int spider_fields::add_field(
-  Field *field_arg
-) {
-  SPIDER_FIELD_HOLDER *field_holder;
-  SPIDER_FIELD_CHAIN *field_chain;
-  DBUG_ENTER("spider_fields::add_field");
-  DBUG_PRINT("info",("spider this=%p", this));
-  DBUG_PRINT("info",("spider field=%p", field_arg));
-  if (!first_field_holder)
-  {
-    field_holder = create_field_holder();
-    DBUG_PRINT("info",("spider field_holder=%p", field_holder));
-    if (!field_holder)
-      DBUG_RETURN(HA_ERR_OUT_OF_MEM);
-    field_holder->field = field_arg;
-    first_field_holder = field_holder;
-    last_field_holder = field_holder;
-  } else {
-    field_holder = first_field_holder;
-    do {
-      if (field_holder->field == field_arg)
-        break;
-    } while ((field_holder = field_holder->next));
-    if (!field_holder)
-    {
-      field_holder = create_field_holder();
-      DBUG_PRINT("info",("spider field_holder=%p", field_holder));
-      if (!field_holder)
-        DBUG_RETURN(HA_ERR_OUT_OF_MEM);
-      field_holder->field = field_arg;
-      last_field_holder->next = field_holder;
-      last_field_holder = field_holder;
-    }
-  }
-  field_chain = create_field_chain();
-  DBUG_PRINT("info",("spider field_chain=%p", field_chain));
-  if (!field_chain)
-    DBUG_RETURN(HA_ERR_OUT_OF_MEM);
-  field_chain->field_holder = field_holder;
-  if (!first_field_chain)
-  {
-    first_field_chain = field_chain;
-    last_field_chain = field_chain;
-  } else {
-    last_field_chain->next = field_chain;
-    last_field_chain = field_chain;
-  }
-  DBUG_RETURN(0);
-}
-
-SPIDER_FIELD_HOLDER *spider_fields::create_field_holder(
-) {
-  DBUG_ENTER("spider_fields::create_field_holder");
-  DBUG_PRINT("info",("spider this=%p", this));
-  DBUG_RETURN((SPIDER_FIELD_HOLDER *)
-    spider_malloc(spider_current_trx, 250, sizeof(SPIDER_FIELD_HOLDER),
-    MYF(MY_WME | MY_ZEROFILL)));
-}
-
-void spider_fields::set_pos_to_first_field_holder(
-) {
-  DBUG_ENTER("spider_fields::set_pos_to_first_field_holder");
-  DBUG_PRINT("info",("spider this=%p", this));
-  current_field_holder = first_field_holder;
-  DBUG_VOID_RETURN;
-}
-
-SPIDER_FIELD_HOLDER *spider_fields::get_next_field_holder(
-) {
-  SPIDER_FIELD_HOLDER *return_field_holder = current_field_holder;
-  DBUG_ENTER("spider_fields::get_next_field_holder");
-  DBUG_PRINT("info",("spider this=%p", this));
-  if (current_field_holder)
-    current_field_holder = current_field_holder->next;
-  DBUG_RETURN(return_field_holder);
-}
-
-SPIDER_FIELD_CHAIN *spider_fields::create_field_chain(
-) {
-  DBUG_ENTER("spider_fields::create_field_chain");
-  DBUG_PRINT("info",("spider this=%p", this));
-  DBUG_RETURN((SPIDER_FIELD_CHAIN *)
-    spider_malloc(spider_current_trx, 251, sizeof(SPIDER_FIELD_CHAIN),
-    MYF(MY_WME | MY_ZEROFILL)));
-}
-
-void spider_fields::set_pos_to_first_field_chain(
-) {
-  DBUG_ENTER("spider_fields::set_pos_to_first_field_chain");
-  DBUG_PRINT("info",("spider this=%p", this));
-  current_field_chain = first_field_chain;
-  DBUG_VOID_RETURN;
-}
-
-SPIDER_FIELD_CHAIN *spider_fields::get_next_field_chain(
-) {
-  SPIDER_FIELD_CHAIN *return_field_chain = current_field_chain;
-  DBUG_ENTER("spider_fields::get_next_field_chain");
-  DBUG_PRINT("info",("spider this=%p", this));
-  if (current_field_chain)
-    current_field_chain = current_field_chain->next;
-  DBUG_RETURN(return_field_chain);
-}
-
 void spider_fields::set_field_ptr(
   Field **field_arg
 ) {
   DBUG_ENTER("spider_fields::set_field_ptr");
   DBUG_PRINT("info",("spider this=%p", this));
   DBUG_PRINT("info",("spider field_ptr=%p", field_arg));
-  first_field_ptr = field_arg;
   current_field_ptr = field_arg;
   DBUG_VOID_RETURN;
 }
@@ -1132,15 +941,13 @@ int spider_fields::ping_table_mon_from_table(
   ha_spider *tmp_spider;
   SPIDER_SHARE *tmp_share;
   int tmp_link_idx;
-  SPIDER_TABLE_HOLDER *table_holder;
   SPIDER_LINK_IDX_HOLDER *link_idx_holder;
   DBUG_ENTER("spider_fields::ping_table_mon_from_table");
   set_pos_to_first_table_on_link_idx_chain(link_idx_chain);
-  set_pos_to_first_table_holder();
-  while ((table_holder = get_next_table_holder()))
+  for (uint i= 0; i < table_count; i++)
   {
     link_idx_holder = get_next_table_on_link_idx_chain(link_idx_chain);
-    tmp_spider = table_holder->spider;
+    tmp_spider = table_holder[i].spider;
     tmp_link_idx = link_idx_holder->link_idx;
     tmp_share = tmp_spider->share;
     if (tmp_share->monitoring_kind[tmp_link_idx])
@@ -1177,9 +984,7 @@ spider_group_by_handler::spider_group_by_handler(
   query(*query_arg), fields(fields_arg)
 {
   DBUG_ENTER("spider_group_by_handler::spider_group_by_handler");
-  fields->set_pos_to_first_table_holder();
-  SPIDER_TABLE_HOLDER *table_holder = fields->get_next_table_holder();
-  spider = table_holder->spider;
+  spider = fields->get_first_table_holder()->spider;
   trx = spider->trx;
   DBUG_VOID_RETURN;
 }
@@ -1187,42 +992,21 @@ spider_group_by_handler::spider_group_by_handler(
 spider_group_by_handler::~spider_group_by_handler()
 {
   DBUG_ENTER("spider_group_by_handler::~spider_group_by_handler");
+  spider_free(spider_current_trx, fields->get_first_table_holder(), MYF(0));
   delete fields;
   DBUG_VOID_RETURN;
 }
 
-int spider_group_by_handler::init_scan()
+static int spider_prepare_init_scan(
+  const Query& query, spider_fields *fields, ha_spider *spider,
+  SPIDER_TRX *trx, longlong& offset_limit, THD *thd)
 {
   int error_num, link_idx;
-  uint dbton_id;
-  spider_db_handler *dbton_hdl;
-  st_select_lex *select_lex;
-  longlong select_limit;
-  longlong direct_order_limit;
-  SPIDER_SHARE *share = spider->share;
-  SPIDER_CONN *conn;
   SPIDER_RESULT_LIST *result_list = &spider->result_list;
-  SPIDER_LINK_IDX_CHAIN *link_idx_chain;
-  SPIDER_LINK_IDX_HOLDER *link_idx_holder;
-  DBUG_ENTER("spider_group_by_handler::init_scan");
-  store_error = 0;
-#ifndef DBUG_OFF
-  Field **field;
-  for (
-    field = table->field;
-    *field;
-    field++
-  ) {
-    DBUG_PRINT("info",("spider field_name=%s",
-      SPIDER_field_name_str(*field)));
-  }
-#endif
-
-  if (trx->thd->killed)
-  {
-    my_error(ER_QUERY_INTERRUPTED, MYF(0));
-    DBUG_RETURN(ER_QUERY_INTERRUPTED);
-  }
+  st_select_lex *select_lex;
+  longlong select_limit, direct_order_limit;
+  SPIDER_SHARE *share = spider->share;
+  DBUG_ENTER("spider_prepare_init_scan");
 
   spider->use_fields = TRUE;
   spider->fields = fields;
@@ -1296,86 +1080,87 @@ int spider_group_by_handler::init_scan()
   } else {
     offset_limit = 0;
   }
+  DBUG_RETURN(0);
+}
+
+static int spider_make_query(const Query& query, spider_fields* fields, ha_spider *spider, TABLE *table)
+{
+  uint dbton_id;
+  spider_db_handler* dbton_hdl;
+  SPIDER_RESULT_LIST *result_list = &spider->result_list;
+  int error_num;
+  DBUG_ENTER("spider_make_query");
 
-  /* making a query */
   fields->set_pos_to_first_dbton_id();
   while ((dbton_id = fields->get_next_dbton_id()) < SPIDER_DBTON_SIZE)
   {
     dbton_hdl = spider->dbton_handler[dbton_id];
     result_list->direct_distinct = query.distinct;
-    fields->set_pos_to_first_field_chain();
     if ((error_num = dbton_hdl->reset_sql(SPIDER_SQL_TYPE_SELECT_SQL)))
-    {
       DBUG_RETURN(error_num);
-    }
     if ((error_num = dbton_hdl->append_select_part(SPIDER_SQL_TYPE_SELECT_SQL)))
-    {
       DBUG_RETURN(error_num);
-    }
     fields->set_field_ptr(table->field);
     if ((error_num = dbton_hdl->append_list_item_select_part(
-      query.select, NULL, 0, TRUE, fields, SPIDER_SQL_TYPE_SELECT_SQL)))
-    {
+           query.select, NULL, 0, TRUE, fields, SPIDER_SQL_TYPE_SELECT_SQL)))
       DBUG_RETURN(error_num);
-    }
     if ((error_num = dbton_hdl->append_from_and_tables_part(
-      fields, SPIDER_SQL_TYPE_SELECT_SQL)))
-    {
+           fields, SPIDER_SQL_TYPE_SELECT_SQL)))
       DBUG_RETURN(error_num);
-    }
     if (query.where)
     {
       if ((error_num =
-        dbton_hdl->append_where_part(SPIDER_SQL_TYPE_SELECT_SQL)))
-      {
+           dbton_hdl->append_where_part(SPIDER_SQL_TYPE_SELECT_SQL)))
         DBUG_RETURN(error_num);
-      }
       if ((error_num = dbton_hdl->append_item_type_part(
-        query.where, NULL, 0, TRUE, fields, SPIDER_SQL_TYPE_SELECT_SQL)))
-      {
+             query.where, NULL, 0, TRUE, fields, SPIDER_SQL_TYPE_SELECT_SQL)))
         DBUG_RETURN(error_num);
-      }
     }
     if (query.group_by)
     {
       if ((error_num = dbton_hdl->append_group_by_part(
-        query.group_by, NULL, 0, TRUE, fields, SPIDER_SQL_TYPE_SELECT_SQL)))
-      {
+             query.group_by, NULL, 0, TRUE, fields, SPIDER_SQL_TYPE_SELECT_SQL)))
         DBUG_RETURN(error_num);
-      }
     }
     if (query.having)
     {
       if ((error_num =
-        dbton_hdl->append_having_part(SPIDER_SQL_TYPE_SELECT_SQL)))
-      {
+           dbton_hdl->append_having_part(SPIDER_SQL_TYPE_SELECT_SQL)))
         DBUG_RETURN(error_num);
-      }
       if ((error_num = dbton_hdl->append_item_type_part(
-        query.having, NULL, 0, TRUE, fields, SPIDER_SQL_TYPE_SELECT_SQL)))
-      {
+             query.having, NULL, 0, TRUE, fields, SPIDER_SQL_TYPE_SELECT_SQL)))
         DBUG_RETURN(error_num);
-      }
     }
     if (query.order_by)
     {
       if ((error_num = dbton_hdl->append_order_by_part(
-        query.order_by, NULL, 0, TRUE, fields, SPIDER_SQL_TYPE_SELECT_SQL)))
-      {
+             query.order_by, NULL, 0, TRUE, fields,
+             SPIDER_SQL_TYPE_SELECT_SQL)))
         DBUG_RETURN(error_num);
-      }
     }
-    if ((error_num = dbton_hdl->append_limit_part(result_list->internal_offset,
-      result_list->limit_num, SPIDER_SQL_TYPE_SELECT_SQL)))
-    {
+    if ((error_num = dbton_hdl->append_limit_part(
+           result_list->internal_offset, result_list->limit_num,
+           SPIDER_SQL_TYPE_SELECT_SQL)))
       DBUG_RETURN(error_num);
-    }
     if ((error_num = dbton_hdl->append_select_lock_part(
-      SPIDER_SQL_TYPE_SELECT_SQL)))
-    {
+           SPIDER_SQL_TYPE_SELECT_SQL)))
       DBUG_RETURN(error_num);
-    }
   }
+  DBUG_RETURN(0);
+}
+
+static int spider_send_query(
+  spider_fields *fields, ha_spider *spider, SPIDER_TRX *trx, TABLE *table,
+  int& store_error)
+{
+  int error_num, link_idx;
+  spider_db_handler *dbton_hdl;
+  SPIDER_RESULT_LIST *result_list = &spider->result_list;
+  SPIDER_SHARE *share = spider->share;
+  SPIDER_CONN *conn;
+  SPIDER_LINK_IDX_CHAIN *link_idx_chain;
+  SPIDER_LINK_IDX_HOLDER *link_idx_holder;
+  DBUG_ENTER("spider_send_query");
 
   fields->set_pos_to_first_link_idx_chain();
   while ((link_idx_chain = fields->get_next_link_idx_chain()))
@@ -1395,12 +1180,8 @@ int spider_group_by_handler::init_scan()
         dbton_hdl->first_link_idx, TRUE, FALSE,
         !fields->is_first_link_ok_chain(link_idx_chain))))
       {
-        if (
-          error_num != HA_ERR_END_OF_FILE &&
-          spider->need_mons[link_idx]
-        ) {
+        if (error_num != HA_ERR_END_OF_FILE && spider->need_mons[link_idx])
           error_num = fields->ping_table_mon_from_table(link_idx_chain);
-        }
         if ((error_num = spider->check_error_mode_eof(error_num)) == HA_ERR_END_OF_FILE)
         {
           store_error = HA_ERR_END_OF_FILE;
@@ -1410,63 +1191,53 @@ int spider_group_by_handler::init_scan()
       }
     } else {
 #endif
-      if (dbton_hdl->need_lock_before_set_sql_for_exec(
-        SPIDER_SQL_TYPE_SELECT_SQL))
-      {
-        pthread_mutex_lock(&conn->mta_conn_mutex);
-        SPIDER_SET_FILE_POS(&conn->mta_conn_mutex_file_pos);
-      }
-      if ((error_num =
-        dbton_hdl->set_sql_for_exec(SPIDER_SQL_TYPE_SELECT_SQL, link_idx,
-        link_idx_chain)))
-      {
+      pthread_mutex_assert_not_owner(&conn->mta_conn_mutex);
+      if ((error_num = dbton_hdl->set_sql_for_exec(
+             SPIDER_SQL_TYPE_SELECT_SQL, link_idx, link_idx_chain)))
         DBUG_RETURN(error_num);
-      }
-      if (!dbton_hdl->need_lock_before_set_sql_for_exec(
-        SPIDER_SQL_TYPE_SELECT_SQL))
-      {
-        pthread_mutex_lock(&conn->mta_conn_mutex);
-        SPIDER_SET_FILE_POS(&conn->mta_conn_mutex_file_pos);
-      }
+      pthread_mutex_lock(&conn->mta_conn_mutex);
+      SPIDER_SET_FILE_POS(&conn->mta_conn_mutex_file_pos);
       conn->need_mon = &spider->need_mons[link_idx];
+      DBUG_ASSERT(!conn->mta_conn_mutex_lock_already);
+      DBUG_ASSERT(!conn->mta_conn_mutex_unlock_later);
       conn->mta_conn_mutex_lock_already = TRUE;
       conn->mta_conn_mutex_unlock_later = TRUE;
       if ((error_num = spider_db_set_names(spider, conn,
         link_idx)))
+      if ((error_num = spider_db_set_names(spider, conn, link_idx)))
       {
+        DBUG_ASSERT(conn->mta_conn_mutex_lock_already);
+        DBUG_ASSERT(conn->mta_conn_mutex_unlock_later);
         conn->mta_conn_mutex_lock_already = FALSE;
         conn->mta_conn_mutex_unlock_later = FALSE;
         SPIDER_CLEAR_FILE_POS(&conn->mta_conn_mutex_file_pos);
         pthread_mutex_unlock(&conn->mta_conn_mutex);
-        if (
-          spider->need_mons[link_idx]
-        ) {
+        if (spider->need_mons[link_idx])
           error_num = fields->ping_table_mon_from_table(link_idx_chain);
-        }
-        if ((error_num = spider->check_error_mode_eof(error_num)) == HA_ERR_END_OF_FILE)
+        if ((error_num = spider->check_error_mode_eof(error_num)) ==
+            HA_ERR_END_OF_FILE)
         {
           store_error = HA_ERR_END_OF_FILE;
           error_num = 0;
         }
         DBUG_RETURN(error_num);
       }
-      spider_conn_set_timeout_from_share(conn, link_idx,
-        trx->thd, share);
+      spider_conn_set_timeout_from_share(conn, link_idx, trx->thd, share);
       if (dbton_hdl->execute_sql(
         SPIDER_SQL_TYPE_SELECT_SQL,
         conn,
         spider->result_list.quick_mode,
-        &spider->need_mons[link_idx])
-      ) {
+        &spider->need_mons[link_idx]))
+      {
+        DBUG_ASSERT(conn->mta_conn_mutex_lock_already);
+        DBUG_ASSERT(conn->mta_conn_mutex_unlock_later);
         conn->mta_conn_mutex_lock_already = FALSE;
         conn->mta_conn_mutex_unlock_later = FALSE;
         error_num = spider_db_errorno(conn);
-        if (
-          spider->need_mons[link_idx]
-        ) {
+        if (spider->need_mons[link_idx])
           error_num = fields->ping_table_mon_from_table(link_idx_chain);
-        }
-        if ((error_num = spider->check_error_mode_eof(error_num)) == HA_ERR_END_OF_FILE)
+        if ((error_num = spider->check_error_mode_eof(error_num)) ==
+            HA_ERR_END_OF_FILE)
         {
           store_error = HA_ERR_END_OF_FILE;
           error_num = 0;
@@ -1474,19 +1245,18 @@ int spider_group_by_handler::init_scan()
         DBUG_RETURN(error_num);
       }
       spider->connection_ids[link_idx] = conn->connection_id;
+      DBUG_ASSERT(conn->mta_conn_mutex_lock_already);
+      DBUG_ASSERT(conn->mta_conn_mutex_unlock_later);
       conn->mta_conn_mutex_lock_already = FALSE;
       conn->mta_conn_mutex_unlock_later = FALSE;
       if (fields->is_first_link_ok_chain(link_idx_chain))
       {
         if ((error_num = spider_db_store_result(spider, link_idx, table)))
         {
-          if (
-            error_num != HA_ERR_END_OF_FILE &&
-            spider->need_mons[link_idx]
-          ) {
+          if (error_num != HA_ERR_END_OF_FILE && spider->need_mons[link_idx])
             error_num = fields->ping_table_mon_from_table(link_idx_chain);
-          }
-          if ((error_num = spider->check_error_mode_eof(error_num)) == HA_ERR_END_OF_FILE)
+          if ((error_num = spider->check_error_mode_eof(error_num)) ==
+              HA_ERR_END_OF_FILE)
           {
             store_error = HA_ERR_END_OF_FILE;
             error_num = 0;
@@ -1495,7 +1265,8 @@ int spider_group_by_handler::init_scan()
         }
         spider->result_link_idx = link_idx;
         spider->result_link_idx_chain = link_idx_chain;
-      } else {
+      } else
+      {
         spider_db_discard_result(spider, link_idx, conn);
         SPIDER_CLEAR_FILE_POS(&conn->mta_conn_mutex_file_pos);
         pthread_mutex_unlock(&conn->mta_conn_mutex);
@@ -1504,6 +1275,37 @@ int spider_group_by_handler::init_scan()
     }
 #endif
   }
+  DBUG_RETURN(0);
+}
+
+/*
+ Prepare and send query to data nodes and store the query results.
+*/
+int spider_group_by_handler::init_scan()
+{
+  int error_num;
+  DBUG_ENTER("spider_group_by_handler::init_scan");
+  store_error = 0;
+#ifndef DBUG_OFF
+  for (Field **field = table->field; *field; field++)
+    DBUG_PRINT("info",("spider field_name=%s", SPIDER_field_name_str(*field)));
+#endif
+
+  if (trx->thd->killed)
+  {
+    my_error(ER_QUERY_INTERRUPTED, MYF(0));
+    DBUG_RETURN(ER_QUERY_INTERRUPTED);
+  }
+
+  if ((error_num = spider_prepare_init_scan(
+         query, fields, spider, trx, offset_limit, thd)))
+    DBUG_RETURN(error_num);
+
+  if ((error_num = spider_make_query(query, fields, spider, table)))
+    DBUG_RETURN(error_num);
+
+  if ((error_num = spider_send_query(fields, spider, trx, table, store_error)))
+    DBUG_RETURN(error_num);
 
   first = TRUE;
   DBUG_RETURN(0);
@@ -1614,7 +1416,8 @@ group_by_handler *spider_create_group_by_handler(
   bool keep_going;
   bool find_dbton = FALSE;
   spider_fields *fields = NULL, *fields_arg = NULL;
-  uint table_idx, dbton_id;
+  SPIDER_TABLE_HOLDER *table_holder;
+  uint table_idx, dbton_id, table_count= 0;
   long tgt_link_status;
   DBUG_ENTER("spider_create_group_by_handler");
 
@@ -1630,45 +1433,32 @@ group_by_handler *spider_create_group_by_handler(
       break;
   }
 
-#ifdef WITH_PARTITION_STORAGE_ENGINE
   from = query->from;
   do {
     DBUG_PRINT("info",("spider from=%p", from));
-    if (from->table->const_table)
-      continue;
+    ++table_count;
+#ifdef WITH_PARTITION_STORAGE_ENGINE
     if (from->table->part_info)
     {
       DBUG_PRINT("info",("spider partition handler"));
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
       partition_info *part_info = from->table->part_info;
       uint bits = bitmap_bits_set(&part_info->read_partitions);
       DBUG_PRINT("info",("spider bits=%u", bits));
       if (bits != 1)
       {
         DBUG_PRINT("info",("spider using multiple partitions is not supported by this feature yet"));
-#else
-        DBUG_PRINT("info",("spider partition is not supported by this feature yet"));
-#endif
         DBUG_RETURN(NULL);
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
       }
-#endif
     }
-  } while ((from = from->next_local));
 #endif
+  } while ((from = from->next_local));
+
+  if (!(table_holder= spider_create_table_holder(table_count)))
+    DBUG_RETURN(NULL);
 
   table_idx = 0;
   from = query->from;
-  while (from && from->table->const_table)
-  {
-    from = from->next_local;
-  }
-  if (!from)
-  {
-    /* all tables are const_table */
-    DBUG_RETURN(NULL);
-  }
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
+#ifdef WITH_PARTITION_STORAGE_ENGINE
   if (from->table->part_info)
   {
     partition_info *part_info = from->table->part_info;
@@ -1677,14 +1467,17 @@ group_by_handler *spider_create_group_by_handler(
     handler **handlers = partition->get_child_handlers();
     spider = (ha_spider *) handlers[part];
   } else {
-#endif
     spider = (ha_spider *) from->table->file;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
   }
 #endif
   share = spider->share;
   spider->idx_for_direct_join = table_idx;
   ++table_idx;
+  if (!spider_add_table_holder(spider, table_holder))
+  {
+    DBUG_PRINT("info",("spider can not add a table"));
+    goto skip_free_table_holder;
+  }
   memset(dbton_bitmap, 0, spider_bitmap_size(SPIDER_DBTON_SIZE));
   for (roop_count = 0; roop_count < (int) share->use_dbton_count; ++roop_count)
   {
@@ -1698,9 +1491,7 @@ group_by_handler *spider_create_group_by_handler(
   }
   while ((from = from->next_local))
   {
-    if (from->table->const_table)
-      continue;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
+#ifdef WITH_PARTITION_STORAGE_ENGINE
     if (from->table->part_info)
     {
       partition_info *part_info = from->table->part_info;
@@ -1709,14 +1500,17 @@ group_by_handler *spider_create_group_by_handler(
       handler **handlers = partition->get_child_handlers();
       spider = (ha_spider *) handlers[part];
     } else {
-#endif
       spider = (ha_spider *) from->table->file;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
     }
 #endif
     share = spider->share;
     spider->idx_for_direct_join = table_idx;
     ++table_idx;
+    if (!spider_add_table_holder(spider, table_holder))
+    {
+      DBUG_PRINT("info",("spider can not add a table"));
+      goto skip_free_table_holder;
+    }
     memset(dbton_bitmap_tmp, 0, spider_bitmap_size(SPIDER_DBTON_SIZE));
     for (roop_count = 0; roop_count < (int) share->use_dbton_count; ++roop_count)
     {
@@ -1737,9 +1531,7 @@ group_by_handler *spider_create_group_by_handler(
 
   from = query->from;
   do {
-    if (from->table->const_table)
-      continue;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
+#ifdef WITH_PARTITION_STORAGE_ENGINE
     if (from->table->part_info)
     {
       partition_info *part_info = from->table->part_info;
@@ -1748,9 +1540,7 @@ group_by_handler *spider_create_group_by_handler(
       handler **handlers = partition->get_child_handlers();
       spider = (ha_spider *) handlers[part];
     } else {
-#endif
       spider = (ha_spider *) from->table->file;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
     }
 #endif
     share = spider->share;
@@ -1770,15 +1560,19 @@ group_by_handler *spider_create_group_by_handler(
       {
         fields_arg = new spider_fields();
         if (!fields_arg)
-        {
-          DBUG_RETURN(NULL);
-        }
+          goto skip_free_table_holder;
       }
+      fields_arg->set_table_holder(table_holder, table_count);
       keep_going = TRUE;
       it.init(*query->select);
       while ((item = it++))
       {
         DBUG_PRINT("info",("spider select item=%p", item));
+        if (item->const_item())
+        {
+          DBUG_PRINT("info",("spider const item"));
+          continue;
+        }
         if (spider_db_print_item_type(item, NULL, spider, NULL, NULL, 0,
           roop_count, TRUE, fields_arg))
         {
@@ -1837,6 +1631,8 @@ group_by_handler *spider_create_group_by_handler(
         {
           for (order = query->order_by; order; order = order->next)
           {
+            if ((*order->item)->type() == Item::SUM_FUNC_ITEM)
+              continue;
             if (spider_db_print_item_type((*order->item), NULL, spider, NULL, NULL, 0,
               roop_count, TRUE, fields_arg))
             {
@@ -1873,22 +1669,10 @@ group_by_handler *spider_create_group_by_handler(
     }
   }
   if (!find_dbton)
-  {
-    DBUG_RETURN(NULL);
-  }
-
-  if (fields->create_table_holder(table_idx))
-  {
-    delete fields;
-    DBUG_RETURN(NULL);
-  }
+    goto skip_free_table_holder;
 
   from = query->from;
-  while (from->table->const_table)
-  {
-    from = from->next_local;
-  }
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
+#ifdef WITH_PARTITION_STORAGE_ENGINE
   if (from->table->part_info)
   {
     partition_info *part_info = from->table->part_info;
@@ -1897,9 +1681,7 @@ group_by_handler *spider_create_group_by_handler(
     handler **handlers = partition->get_child_handlers();
     spider = (ha_spider *) handlers[part];
   } else {
-#endif
     spider = (ha_spider *) from->table->file;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
   }
 #endif
   share = spider->share;
@@ -1912,12 +1694,6 @@ group_by_handler *spider_create_group_by_handler(
   }
   DBUG_PRINT("info",("spider s->db=%s", from->table->s->db.str));
   DBUG_PRINT("info",("spider s->table_name=%s", from->table->s->table_name.str));
-  if (!fields->add_table(spider))
-  {
-    DBUG_PRINT("info",("spider can not add a table"));
-    delete fields;
-    DBUG_RETURN(NULL);
-  }
   for (
     roop_count = spider_conn_link_idx_next(share->link_statuses,
       spider->conn_link_idx, -1, share->link_count,
@@ -1931,10 +1707,7 @@ group_by_handler *spider_create_group_by_handler(
     {
       DBUG_PRINT("info",("spider direct_join does not support use_handler"));
       if (lock_mode)
-      {
-        delete fields;
-        DBUG_RETURN(NULL);
-      }
+        goto skip_free_fields;
       continue;
     }
     conn = spider->conns[roop_count];
@@ -1946,8 +1719,7 @@ group_by_handler *spider_create_group_by_handler(
       DBUG_PRINT("info",("spider direct_join does not support with lock tables yet"));
       if (lock_mode)
       {
-        delete fields;
-        DBUG_RETURN(NULL);
+        goto skip_free_fields;
       }
       continue;
     }
@@ -1955,29 +1727,24 @@ group_by_handler *spider_create_group_by_handler(
       share->access_balances[spider->conn_link_idx[roop_count]]))
     {
       DBUG_PRINT("info",("spider can not create conn_holder"));
-      delete fields;
-      DBUG_RETURN(NULL);
+      goto skip_free_fields;
     }
     if (fields->add_link_idx(conn->conn_holder_for_direct_join, spider, roop_count))
     {
       DBUG_PRINT("info",("spider can not create link_idx_holder"));
-      delete fields;
-      DBUG_RETURN(NULL);
+      goto skip_free_fields;
     }
   }
   if (!fields->has_conn_holder())
   {
-    delete fields;
-    DBUG_RETURN(NULL);
+    goto skip_free_fields;
   }
 
   while ((from = from->next_local))
   {
-    if (from->table->const_table)
-      continue;
     fields->clear_conn_holder_from_conn();
 
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
+#ifdef WITH_PARTITION_STORAGE_ENGINE
     if (from->table->part_info)
     {
       partition_info *part_info = from->table->part_info;
@@ -1986,18 +1753,10 @@ group_by_handler *spider_create_group_by_handler(
       handler **handlers = partition->get_child_handlers();
       spider = (ha_spider *) handlers[part];
     } else {
-#endif
       spider = (ha_spider *) from->table->file;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
     }
 #endif
     share = spider->share;
-    if (!fields->add_table(spider))
-    {
-      DBUG_PRINT("info",("spider can not add a table"));
-      delete fields;
-      DBUG_RETURN(NULL);
-    }
     DBUG_PRINT("info",("spider s->db=%s", from->table->s->db.str));
     DBUG_PRINT("info",("spider s->table_name=%s", from->table->s->table_name.str));
     for (
@@ -2014,10 +1773,7 @@ group_by_handler *spider_create_group_by_handler(
       {
         DBUG_PRINT("info",("spider direct_join does not support use_handler"));
         if (lock_mode)
-        {
-          delete fields;
-          DBUG_RETURN(NULL);
-        }
+          goto skip_free_fields;
         continue;
       }
       conn = spider->conns[roop_count];
@@ -2027,17 +1783,13 @@ group_by_handler *spider_create_group_by_handler(
         DBUG_PRINT("info",("spider connection %p can not be used for this query with locking",
           conn));
         if (lock_mode)
-        {
-          delete fields;
-          DBUG_RETURN(NULL);
-        }
+          goto skip_free_fields;
         continue;
       }
       if (fields->add_link_idx(conn->conn_holder_for_direct_join, spider, roop_count))
       {
         DBUG_PRINT("info",("spider can not create link_idx_holder"));
-        delete fields;
-        DBUG_RETURN(NULL);
+        goto skip_free_fields;
       }
     }
 
@@ -2046,30 +1798,20 @@ group_by_handler *spider_create_group_by_handler(
       if (lock_mode)
       {
         DBUG_PRINT("info",("spider some connections can not be used for this query with locking"));
-        delete fields;
-        DBUG_RETURN(NULL);
+        goto skip_free_fields;
       }
     }
     if (!fields->has_conn_holder())
     {
-      delete fields;
-      DBUG_RETURN(NULL);
+      goto skip_free_fields;
     }
   }
 
-  if (!fields->all_query_fields_are_query_table_members())
-  {
-    DBUG_PRINT("info", ("spider found a query field that is not a query table member"));
-    delete fields;
-    DBUG_RETURN(NULL);
-  }
-
   fields->check_support_dbton(dbton_bitmap);
   if (!fields->has_conn_holder())
   {
     DBUG_PRINT("info",("spider all chosen connections can't match dbton_id"));
-    delete fields;
-    DBUG_RETURN(NULL);
+    goto skip_free_fields;
   }
 
   /* choose a connection */
@@ -2081,16 +1823,14 @@ group_by_handler *spider_create_group_by_handler(
   if (fields->make_link_idx_chain(tgt_link_status))
   {
     DBUG_PRINT("info",("spider can not create link_idx_chain"));
-    delete fields;
-    DBUG_RETURN(NULL);
+    goto skip_free_fields;
   }
 
   /* choose link_id */
   if (fields->check_link_ok_chain())
   {
     DBUG_PRINT("info",("spider do not have link ok status"));
-    delete fields;
-    DBUG_RETURN(NULL);
+    goto skip_free_fields;
   }
 
   fields->set_first_link_idx();
@@ -2098,8 +1838,7 @@ group_by_handler *spider_create_group_by_handler(
   if (!(group_by_handler = new spider_group_by_handler(thd, query, fields)))
   {
     DBUG_PRINT("info",("spider can't create group_by_handler"));
-    delete fields;
-    DBUG_RETURN(NULL);
+    goto skip_free_fields;
   }
   query->distinct = FALSE;
   query->where = NULL;
@@ -2107,5 +1846,11 @@ group_by_handler *spider_create_group_by_handler(
   query->having = NULL;
   query->order_by = NULL;
   DBUG_RETURN(group_by_handler);
+
+skip_free_fields:
+  delete fields;
+skip_free_table_holder:
+  spider_free(spider_current_trx, table_holder, MYF(0));
+  DBUG_RETURN(NULL);
 }
 #endif
