@@ -17,17 +17,12 @@
 #include <my_global.h>
 #include "mysql_version.h"
 #include "spd_environ.h"
-#if MYSQL_VERSION_ID < 50500
-#include "mysql_priv.h"
-#include <mysql/plugin.h>
-#else
 #include "sql_priv.h"
 #include "probes_mysql.h"
 #include "sql_class.h"
 #include "key.h"
 #include "sql_base.h"
 #include "tztime.h"
-#endif
 #include "sql_select.h"
 #include "spd_err.h"
 #include "spd_param.h"
@@ -121,17 +116,6 @@ inline int spider_delete_sys_table_row(TABLE *table, int record_number = 0,
   return error_num;
 }
 
-#if MYSQL_VERSION_ID < 50500
-TABLE *spider_open_sys_table(
-  THD *thd,
-  const char *table_name,
-  int table_name_length,
-  bool write,
-  Open_tables_state *open_tables_backup,
-  bool need_lock,
-  int *error_num
-)
-#else
 TABLE *spider_open_sys_table(
   THD *thd,
   const char *table_name,
@@ -141,26 +125,11 @@ TABLE *spider_open_sys_table(
   bool need_lock,
   int *error_num
 )
-#endif
 {
   TABLE *table;
   TABLE_LIST tables;
-#if MYSQL_VERSION_ID < 50500
-  TABLE_SHARE *table_share;
-  char table_key[MAX_DBKEY_LENGTH];
-  uint table_key_length;
-#endif
   DBUG_ENTER("spider_open_sys_table");
 
-#if MYSQL_VERSION_ID < 50500
-  memset(&tables, 0, sizeof(TABLE_LIST));
-  SPIDER_TABLE_LIST_db_str(&tables) = (char*)"mysql";
-  SPIDER_TABLE_LIST_db_length(&tables) = sizeof("mysql") - 1;
-  SPIDER_TABLE_LIST_alias_str(&tables) =
-    SPIDER_TABLE_LIST_table_name_str(&tables) = (char *) table_name;
-  SPIDER_TABLE_LIST_table_name_length(&tables) = table_name_length;
-  tables.lock_type = (write ? TL_WRITE : TL_READ);
-#else
 #ifdef SPIDER_use_LEX_CSTRING_for_database_tablename_alias
   LEX_CSTRING db_name =
   {
@@ -178,18 +147,8 @@ TABLE *spider_open_sys_table(
     "mysql", sizeof("mysql") - 1, table_name, table_name_length, table_name,
     (write ? TL_WRITE : TL_READ));
 #endif
-#endif
 
-#if MYSQL_VERSION_ID < 50500
-  if (need_lock)
-  {
-#endif
-#if MYSQL_VERSION_ID < 50500
-    if (!(table = open_performance_schema_table(thd, &tables,
-      open_tables_backup)))
-#else
     if (!(table = spider_sys_open_table(thd, &tables, open_tables_backup)))
-#endif
     {
       my_printf_error(ER_SPIDER_CANT_OPEN_SYS_TABLE_NUM,
         ER_SPIDER_CANT_OPEN_SYS_TABLE_STR, MYF(0),
@@ -197,165 +156,163 @@ TABLE *spider_open_sys_table(
       *error_num = ER_SPIDER_CANT_OPEN_SYS_TABLE_NUM;
       DBUG_RETURN(NULL);
     }
-#if MYSQL_VERSION_ID < 50500
-  } else {
-    thd->reset_n_backup_open_tables_state(open_tables_backup);
-
-    if (!(table = (TABLE*) spider_malloc(spider_current_trx, 12,
-      sizeof(*table), MYF(MY_WME))))
-    {
-      *error_num = HA_ERR_OUT_OF_MEM;
-      goto error_malloc;
-    }
-
-    table_key_length =
-      create_table_def_key(thd, table_key, &tables, FALSE);
-
-    if (!(table_share = get_table_share(thd,
-      &tables, table_key, table_key_length, 0, error_num)))
-      goto error;
-    if (open_table_from_share(thd, table_share, tables.alias,
-      (uint) (HA_OPEN_KEYFILE | HA_OPEN_RNDFILE | HA_GET_INDEX),
-      READ_KEYINFO | COMPUTE_TYPES | EXTRA_RECORD,
-      (uint) HA_OPEN_IGNORE_IF_LOCKED | HA_OPEN_FROM_SQL_LAYER,
-      table, FALSE)
-    ) {
-      release_table_share(table_share, RELEASE_NORMAL);
-      my_printf_error(ER_SPIDER_CANT_OPEN_SYS_TABLE_NUM,
-        ER_SPIDER_CANT_OPEN_SYS_TABLE_STR, MYF(0),
-        "mysql", table_name);
-      *error_num = ER_SPIDER_CANT_OPEN_SYS_TABLE_NUM;
-      goto error;
-    }
-  }
-#endif
-  if (table_name_length == SPIDER_SYS_XA_TABLE_NAME_LEN)
+  switch (table_name_length)
   {
-    if (
-      !memcmp(table_name,
-        SPIDER_SYS_XA_TABLE_NAME_STR, SPIDER_SYS_XA_TABLE_NAME_LEN) &&
-      table->s->fields != SPIDER_SYS_XA_COL_CNT
-    ) {
-      spider_close_sys_table(thd, table, open_tables_backup, need_lock);
-      table = NULL;
-      my_printf_error(ER_SPIDER_SYS_TABLE_VERSION_NUM,
-        ER_SPIDER_SYS_TABLE_VERSION_STR, MYF(0),
-        SPIDER_SYS_XA_TABLE_NAME_STR);
-      *error_num = ER_SPIDER_SYS_TABLE_VERSION_NUM;
-      goto error_col_num_chk;
-    }
-  } else if (table_name_length == SPIDER_SYS_XA_MEMBER_TABLE_NAME_LEN)
-  {
-    if (
-      !memcmp(table_name,
-        SPIDER_SYS_XA_MEMBER_TABLE_NAME_STR,
-        SPIDER_SYS_XA_MEMBER_TABLE_NAME_LEN) &&
-      table->s->fields != SPIDER_SYS_XA_MEMBER_COL_CNT
-    ) {
-      spider_close_sys_table(thd, table, open_tables_backup, need_lock);
-      table = NULL;
-      my_printf_error(ER_SPIDER_SYS_TABLE_VERSION_NUM,
-        ER_SPIDER_SYS_TABLE_VERSION_STR, MYF(0),
-        SPIDER_SYS_XA_MEMBER_TABLE_NAME_STR);
-      *error_num = ER_SPIDER_SYS_TABLE_VERSION_NUM;
-      goto error_col_num_chk;
-    }
-  } else if (table_name_length == SPIDER_SYS_TABLES_TABLE_NAME_LEN)
-  {
-    if (
-      !memcmp(table_name,
-        SPIDER_SYS_TABLES_TABLE_NAME_STR,
-        SPIDER_SYS_TABLES_TABLE_NAME_LEN) &&
-      table->s->fields != SPIDER_SYS_TABLES_COL_CNT
-    ) {
-      spider_close_sys_table(thd, table, open_tables_backup, need_lock);
-      table = NULL;
-      my_printf_error(ER_SPIDER_SYS_TABLE_VERSION_NUM,
-        ER_SPIDER_SYS_TABLE_VERSION_STR, MYF(0),
-        SPIDER_SYS_TABLES_TABLE_NAME_STR);
-      *error_num = ER_SPIDER_SYS_TABLE_VERSION_NUM;
-      goto error_col_num_chk;
-    }
-  } else if (table_name_length == SPIDER_SYS_LINK_MON_TABLE_NAME_LEN)
-  {
-    if (
-      !memcmp(table_name,
-        SPIDER_SYS_LINK_MON_TABLE_NAME_STR,
-        SPIDER_SYS_LINK_MON_TABLE_NAME_LEN) &&
-      table->s->fields != SPIDER_SYS_LINK_MON_TABLE_COL_CNT
-    ) {
-      spider_close_sys_table(thd, table, open_tables_backup, need_lock);
-      table = NULL;
-      my_printf_error(ER_SPIDER_SYS_TABLE_VERSION_NUM,
-        ER_SPIDER_SYS_TABLE_VERSION_STR, MYF(0),
-        SPIDER_SYS_LINK_MON_TABLE_NAME_STR);
-      *error_num = ER_SPIDER_SYS_TABLE_VERSION_NUM;
-      goto error_col_num_chk;
-    }
-  } else if (table_name_length == SPIDER_SYS_POS_FOR_RECOVERY_TABLE_NAME_LEN)
-  {
-    if (
-      !memcmp(table_name,
-        SPIDER_SYS_POS_FOR_RECOVERY_TABLE_NAME_STR,
-        SPIDER_SYS_POS_FOR_RECOVERY_TABLE_NAME_LEN) &&
-      table->s->fields != SPIDER_SYS_POS_FOR_RECOVERY_TABLE_COL_CNT
-    ) {
-      spider_close_sys_table(thd, table, open_tables_backup, need_lock);
-      table = NULL;
-      my_printf_error(ER_SPIDER_SYS_TABLE_VERSION_NUM,
-        ER_SPIDER_SYS_TABLE_VERSION_STR, MYF(0),
-        SPIDER_SYS_POS_FOR_RECOVERY_TABLE_NAME_STR);
-      *error_num = ER_SPIDER_SYS_TABLE_VERSION_NUM;
-      goto error_col_num_chk;
-    }
+    case 9:
+      if (!memcmp(table_name, SPIDER_SYS_XA_TABLE_NAME_STR,
+        SPIDER_SYS_XA_TABLE_NAME_LEN))
+      {
+        DBUG_PRINT("info",("spider checking for SYS_XA"));
+        if (table->s->fields != SPIDER_SYS_XA_COL_CNT)
+        {
+          spider_close_sys_table(thd, table, open_tables_backup, need_lock);
+          table = NULL;
+          my_printf_error(ER_SPIDER_SYS_TABLE_VERSION_NUM,
+            ER_SPIDER_SYS_TABLE_VERSION_STR, MYF(0),
+            SPIDER_SYS_XA_TABLE_NAME_STR);
+          *error_num = ER_SPIDER_SYS_TABLE_VERSION_NUM;
+          goto error_col_num_chk;
+        }
+        break;
+      }
+      DBUG_ASSERT(0);
+      break;
+    case 13:
+      if (!memcmp(table_name, SPIDER_SYS_TABLES_TABLE_NAME_STR,
+        SPIDER_SYS_TABLES_TABLE_NAME_LEN))
+      {
+        DBUG_PRINT("info",("spider checking for SYS_TABLES"));
+        if (table->s->fields != SPIDER_SYS_TABLES_COL_CNT)
+        {
+          spider_close_sys_table(thd, table, open_tables_backup, need_lock);
+          table = NULL;
+          my_printf_error(ER_SPIDER_SYS_TABLE_VERSION_NUM,
+            ER_SPIDER_SYS_TABLE_VERSION_STR, MYF(0),
+            SPIDER_SYS_TABLES_TABLE_NAME_STR);
+          *error_num = ER_SPIDER_SYS_TABLE_VERSION_NUM;
+          goto error_col_num_chk;
+        }
+        break;
+      }
+      DBUG_ASSERT(0);
+      break;
+    case 16:
+      if (!memcmp(table_name, SPIDER_SYS_XA_MEMBER_TABLE_NAME_STR,
+        SPIDER_SYS_XA_MEMBER_TABLE_NAME_LEN))
+      {
+        DBUG_PRINT("info",("spider checking for SYS_XA_MEMBER"));
+        if (table->s->fields != SPIDER_SYS_XA_MEMBER_COL_CNT)
+        {
+          spider_close_sys_table(thd, table, open_tables_backup, need_lock);
+          table = NULL;
+          my_printf_error(ER_SPIDER_SYS_TABLE_VERSION_NUM,
+            ER_SPIDER_SYS_TABLE_VERSION_STR, MYF(0),
+            SPIDER_SYS_XA_MEMBER_TABLE_NAME_STR);
+          *error_num = ER_SPIDER_SYS_TABLE_VERSION_NUM;
+          goto error_col_num_chk;
+        }
+        break;
+      }
+      DBUG_ASSERT(0);
+      break;
+    case 20:
+      if (!memcmp(table_name, SPIDER_SYS_XA_FAILED_TABLE_NAME_STR,
+        SPIDER_SYS_XA_FAILED_TABLE_NAME_LEN))
+      {
+        DBUG_PRINT("info",("spider checking for SYS_XA_FAILED"));
+        if (table->s->fields != SPIDER_SYS_XA_FAILED_TABLE_COL_CNT)
+        {
+          spider_close_sys_table(thd, table, open_tables_backup, need_lock);
+          table = NULL;
+          my_printf_error(ER_SPIDER_SYS_TABLE_VERSION_NUM,
+            ER_SPIDER_SYS_TABLE_VERSION_STR, MYF(0),
+            SPIDER_SYS_XA_FAILED_TABLE_NAME_STR);
+          *error_num = ER_SPIDER_SYS_TABLE_VERSION_NUM;
+          goto error_col_num_chk;
+        }
+        break;
+      }
+      DBUG_ASSERT(0);
+      break;
+    case 22:
+      if (!memcmp(table_name, SPIDER_SYS_LINK_FAILED_TABLE_NAME_STR,
+        SPIDER_SYS_LINK_FAILED_TABLE_NAME_LEN))
+      {
+        DBUG_PRINT("info",("spider checking for SYS_LINK_FAILED"));
+        if (table->s->fields != SPIDER_SYS_LINK_FAILED_TABLE_COL_CNT)
+        {
+          spider_close_sys_table(thd, table, open_tables_backup, need_lock);
+          table = NULL;
+          my_printf_error(ER_SPIDER_SYS_TABLE_VERSION_NUM,
+            ER_SPIDER_SYS_TABLE_VERSION_STR, MYF(0),
+            SPIDER_SYS_LINK_FAILED_TABLE_NAME_STR);
+          *error_num = ER_SPIDER_SYS_TABLE_VERSION_NUM;
+          goto error_col_num_chk;
+        }
+        break;
+      }
+      DBUG_ASSERT(0);
+      break;
+    case 23:
+      if (!memcmp(table_name, SPIDER_SYS_LINK_MON_TABLE_NAME_STR,
+        SPIDER_SYS_LINK_MON_TABLE_NAME_LEN))
+      {
+        DBUG_PRINT("info",("spider checking for SYS_LINK_MON"));
+        if (table->s->fields != SPIDER_SYS_LINK_MON_TABLE_COL_CNT)
+        {
+          spider_close_sys_table(thd, table, open_tables_backup, need_lock);
+          table = NULL;
+          my_printf_error(ER_SPIDER_SYS_TABLE_VERSION_NUM,
+            ER_SPIDER_SYS_TABLE_VERSION_STR, MYF(0),
+            SPIDER_SYS_LINK_MON_TABLE_NAME_STR);
+          *error_num = ER_SPIDER_SYS_TABLE_VERSION_NUM;
+          goto error_col_num_chk;
+        }
+        break;
+      }
+      DBUG_ASSERT(0);
+      break;
+    case 34:
+      if (!memcmp(table_name, SPIDER_SYS_POS_FOR_RECOVERY_TABLE_NAME_STR,
+        SPIDER_SYS_POS_FOR_RECOVERY_TABLE_NAME_LEN))
+      {
+        DBUG_PRINT("info",("spider checking for SYS_POS_FOR_RECOVERY"));
+        if (table->s->fields != SPIDER_SYS_POS_FOR_RECOVERY_TABLE_COL_CNT)
+        {
+          spider_close_sys_table(thd, table, open_tables_backup, need_lock);
+          table = NULL;
+          my_printf_error(ER_SPIDER_SYS_TABLE_VERSION_NUM,
+            ER_SPIDER_SYS_TABLE_VERSION_STR, MYF(0),
+            SPIDER_SYS_POS_FOR_RECOVERY_TABLE_NAME_STR);
+          *error_num = ER_SPIDER_SYS_TABLE_VERSION_NUM;
+          goto error_col_num_chk;
+        }
+        break;
+      }
+      DBUG_ASSERT(0);
+      break;
+    default:
+      DBUG_ASSERT(0);
+      break;
   }
   DBUG_RETURN(table);
 
-#if MYSQL_VERSION_ID < 50500
-error:
-  spider_free(spider_current_trx, table, MYF(0));
-error_malloc:
-  thd->restore_backup_open_tables_state(open_tables_backup);
-#endif
 error_col_num_chk:
   DBUG_RETURN(NULL);
 }
 
-#if MYSQL_VERSION_ID < 50500
-void spider_close_sys_table(
-  THD *thd,
-  TABLE *table,
-  Open_tables_state *open_tables_backup,
-  bool need_lock
-)
-#else
 void spider_close_sys_table(
   THD *thd,
   TABLE *table,
   Open_tables_backup *open_tables_backup,
   bool need_lock
 )
-#endif
 {
   DBUG_ENTER("spider_close_sys_table");
-#if MYSQL_VERSION_ID < 50500
-  if (need_lock)
-  {
-    close_performance_schema_table(thd, open_tables_backup);
-  } else {
-    table->file->ha_reset();
-    closefrm(table, TRUE);
-    spider_free(spider_current_trx, table, MYF(0));
-    thd->restore_backup_open_tables_state(open_tables_backup);
-  }
-#else
   spider_sys_close_table(thd, open_tables_backup);
-#endif
   DBUG_VOID_RETURN;
 }
 
-#if MYSQL_VERSION_ID < 50500
-#else
 bool spider_sys_open_tables(
   THD *thd,
   TABLE_LIST **tables,
@@ -408,7 +365,6 @@ void spider_sys_close_table(
   thd->restore_backup_open_tables_state(open_tables_backup);
   DBUG_VOID_RETURN;
 }
-#endif
 
 int spider_sys_index_init(
   TABLE *table,
@@ -453,15 +409,9 @@ int spider_check_sys_table(
     table->key_info,
     table->key_info->key_length);
 
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 50200
   DBUG_RETURN(table->file->ha_index_read_idx_map(
     table->record[0], 0, (uchar *) table_key,
     HA_WHOLE_KEY, HA_READ_KEY_EXACT));
-#else
-  DBUG_RETURN(table->file->index_read_idx_map(
-    table->record[0], 0, (uchar *) table_key,
-    HA_WHOLE_KEY, HA_READ_KEY_EXACT));
-#endif
 }
 
 int spider_check_sys_table_with_find_flag(
@@ -477,15 +427,9 @@ int spider_check_sys_table_with_find_flag(
     table->key_info,
     table->key_info->key_length);
 
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 50200
   DBUG_RETURN(table->file->ha_index_read_idx_map(
     table->record[0], 0, (uchar *) table_key,
     HA_WHOLE_KEY, find_flag));
-#else
-  DBUG_RETURN(table->file->index_read_idx_map(
-    table->record[0], 0, (uchar *) table_key,
-    HA_WHOLE_KEY, find_flag));
-#endif
 }
 
 int spider_check_sys_table_for_update_all_columns(
@@ -500,17 +444,16 @@ int spider_check_sys_table_for_update_all_columns(
     table->key_info,
     table->key_info->key_length);
 
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 50200
   DBUG_RETURN(table->file->ha_index_read_idx_map(
     table->record[1], 0, (uchar *) table_key,
     HA_WHOLE_KEY, HA_READ_KEY_EXACT));
-#else
-  DBUG_RETURN(table->file->index_read_idx_map(
-    table->record[1], 0, (uchar *) table_key,
-    HA_WHOLE_KEY, HA_READ_KEY_EXACT));
-#endif
 }
 
+/*
+  Creates a key (`table_key') consisting of `col_count' key parts of
+  `idx'th index of the table, then positions an index cursor to that
+  key.
+*/
 int spider_get_sys_table_by_idx(
   TABLE *table,
   char *table_key,
@@ -543,26 +486,9 @@ int spider_get_sys_table_by_idx(
     key_length);
 
   if (
-/*
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 50200
-    (error_num = table->file->ha_index_read_idx_map(
-      table->record[0], idx, (uchar *) table_key,
-      make_prev_keypart_map(col_count), HA_READ_KEY_EXACT))
-#else
-    (error_num = table->file->index_read_idx_map(
-      table->record[0], idx, (uchar *) table_key,
-      make_prev_keypart_map(col_count), HA_READ_KEY_EXACT))
-#endif
-*/
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 50200
     (error_num = table->file->ha_index_read_map(
       table->record[0], (uchar *) table_key,
       make_prev_keypart_map(col_count), HA_READ_KEY_EXACT))
-#else
-    (error_num = table->file->index_read_map(
-      table->record[0], (uchar *) table_key,
-      make_prev_keypart_map(col_count), HA_READ_KEY_EXACT))
-#endif
   ) {
     spider_sys_index_end(table);
     DBUG_RETURN(error_num);
@@ -575,22 +501,15 @@ int spider_sys_index_next_same(
   char *table_key
 ) {
   DBUG_ENTER("spider_sys_index_next_same");
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 50200
   DBUG_RETURN(table->file->ha_index_next_same(
     table->record[0],
     (const uchar*) table_key,
     table->key_info->key_length));
-#else
-  DBUG_RETURN(table->file->index_next_same(
-    table->record[0],
-    (const uchar*) table_key,
-    table->key_info->key_length));
-#endif
 }
 
 int spider_sys_index_first(
   TABLE *table,
-  const int idx
+  const int idx                 /* which index to use */
 ) {
   int error_num;
   DBUG_ENTER("spider_sys_index_first");
@@ -598,11 +517,7 @@ int spider_sys_index_first(
     DBUG_RETURN(error_num);
 
   if (
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 50200
     (error_num = table->file->ha_index_first(table->record[0]))
-#else
-    (error_num = table->file->index_first(table->record[0]))
-#endif
   ) {
     spider_sys_index_end(table);
     DBUG_RETURN(error_num);
@@ -620,11 +535,7 @@ int spider_sys_index_last(
     DBUG_RETURN(error_num);
 
   if (
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 50200
     (error_num = table->file->ha_index_last(table->record[0]))
-#else
-    (error_num = table->file->index_last(table->record[0]))
-#endif
   ) {
     spider_sys_index_end(table);
     DBUG_RETURN(error_num);
@@ -636,11 +547,7 @@ int spider_sys_index_next(
   TABLE *table
 ) {
   DBUG_ENTER("spider_sys_index_next");
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 50200
   DBUG_RETURN(table->file->ha_index_next(table->record[0]));
-#else
-  DBUG_RETURN(table->file->index_next(table->record[0]));
-#endif
 }
 
 void spider_store_xa_pk(
@@ -811,6 +718,10 @@ void spider_store_xa_member_info(
   DBUG_VOID_RETURN;
 }
 
+/*
+  Store db and table names from `name' to `table's corresponding
+  fields
+*/
 void spider_store_tables_name(
   TABLE *table,
   const char *name,
@@ -1238,45 +1149,6 @@ void spider_store_binlog_pos_gtid(
   DBUG_VOID_RETURN;
 }
 
-void spider_store_table_sts_info(
-  TABLE *table,
-  ha_statistics *stat
-) {
-  MYSQL_TIME mysql_time;
-  DBUG_ENTER("spider_store_table_sts_info");
-  table->field[2]->store((longlong) stat->data_file_length, TRUE);
-  table->field[3]->store((longlong) stat->max_data_file_length, TRUE);
-  table->field[4]->store((longlong) stat->index_file_length, TRUE);
-  table->field[5]->store((longlong) stat->records, TRUE);
-  table->field[6]->store((longlong) stat->mean_rec_length, TRUE);
-  spd_tz_system->gmt_sec_to_TIME(&mysql_time, (my_time_t) stat->check_time);
-  table->field[7]->store_time(&mysql_time);
-  spd_tz_system->gmt_sec_to_TIME(&mysql_time, (my_time_t) stat->create_time);
-  table->field[8]->store_time(&mysql_time);
-  spd_tz_system->gmt_sec_to_TIME(&mysql_time, (my_time_t) stat->update_time);
-  table->field[9]->store_time(&mysql_time);
-  if (stat->checksum_null)
-  {
-    table->field[10]->set_null();
-    table->field[10]->reset();
-  } else {
-    table->field[10]->set_notnull();
-    table->field[10]->store((longlong) stat->checksum, TRUE);
-  }
-  DBUG_VOID_RETURN;
-}
-
-void spider_store_table_crd_info(
-  TABLE *table,
-  uint *seq,
-  longlong *cardinality
-) {
-  DBUG_ENTER("spider_store_table_crd_info");
-  table->field[2]->store((longlong) *seq, TRUE);
-  table->field[3]->store((longlong) *cardinality, FALSE);
-  DBUG_VOID_RETURN;
-}
-
 int spider_insert_xa(
   TABLE *table,
   XID *xid,
@@ -1383,83 +1255,6 @@ int spider_insert_sys_table(
   DBUG_RETURN(error_num);
 }
 
-int spider_insert_or_update_table_sts(
-  TABLE *table,
-  const char *name,
-  uint name_length,
-  ha_statistics *stat
-) {
-  int error_num;
-  char table_key[MAX_KEY_LENGTH];
-  DBUG_ENTER("spider_insert_or_update_table_sts");
-  table->use_all_columns();
-  spider_store_tables_name(table, name, name_length);
-  spider_store_table_sts_info(
-    table,
-    stat
-  );
-
-  if ((error_num = spider_check_sys_table_for_update_all_columns(table, table_key)))
-  {
-    if (error_num != HA_ERR_KEY_NOT_FOUND && error_num != HA_ERR_END_OF_FILE)
-    {
-      table->file->print_error(error_num, MYF(0));
-      DBUG_RETURN(error_num);
-    }
-    if ((error_num = spider_write_sys_table_row(table)))
-    {
-      DBUG_RETURN(error_num);
-    }
-  } else {
-    if ((error_num = spider_update_sys_table_row(table, FALSE)))
-    {
-      table->file->print_error(error_num, MYF(0));
-      DBUG_RETURN(error_num);
-    }
-  }
-
-  DBUG_RETURN(0);
-}
-
-int spider_insert_or_update_table_crd(
-  TABLE *table,
-  const char *name,
-  uint name_length,
-  longlong *cardinality,
-  uint number_of_keys
-) {
-  int error_num;
-  uint roop_count;
-  char table_key[MAX_KEY_LENGTH];
-  DBUG_ENTER("spider_insert_or_update_table_crd");
-  table->use_all_columns();
-  spider_store_tables_name(table, name, name_length);
-
-  for (roop_count = 0; roop_count < number_of_keys; ++roop_count)
-  {
-    spider_store_table_crd_info(table, &roop_count, &cardinality[roop_count]);
-    if ((error_num = spider_check_sys_table_for_update_all_columns(table, table_key)))
-    {
-      if (error_num != HA_ERR_KEY_NOT_FOUND && error_num != HA_ERR_END_OF_FILE)
-      {
-        table->file->print_error(error_num, MYF(0));
-        DBUG_RETURN(error_num);
-      }
-      if ((error_num = spider_write_sys_table_row(table)))
-      {
-        DBUG_RETURN(error_num);
-      }
-    } else {
-      if ((error_num = spider_update_sys_table_row(table, FALSE)))
-      {
-        table->file->print_error(error_num, MYF(0));
-        DBUG_RETURN(error_num);
-      }
-    }
-  }
-  DBUG_RETURN(0);
-}
-
 int spider_log_tables_link_failed(
   TABLE *table,
   char *name,
@@ -1471,11 +1266,6 @@ int spider_log_tables_link_failed(
   table->use_all_columns();
   spider_store_tables_name(table, name, name_length);
   spider_store_tables_link_idx(table, link_idx);
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100000
-#else
-  if (table->field[3] == table->timestamp_field)
-    table->timestamp_field->set_time();
-#endif
   if ((error_num = spider_write_sys_table_row(table)))
   {
     DBUG_RETURN(error_num);
@@ -1508,11 +1298,6 @@ int spider_log_xa_failed(
     (uint) strlen(status),
     system_charset_info);
 
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100000
-#else
-  if (table->field[20] == table->timestamp_field)
-    table->timestamp_field->set_time();
-#endif
   if ((error_num = spider_write_sys_table_row(table)))
   {
     DBUG_RETURN(error_num);
@@ -2121,14 +1906,16 @@ int spider_get_sys_tables(
   DBUG_RETURN(0);
 }
 
+/* Read table info from mysql.spider_tables into a `SPIDER_SHARE' */
 int spider_get_sys_tables_connect_info(
-  TABLE *table,
-  SPIDER_SHARE *share,
-  int link_idx,
+  TABLE *table,                         /* The mysql.spider_tables table */
+  SPIDER_SHARE *share,                  /* The `SPIDER_SHARE' to
+                                        update info */
   MEM_ROOT *mem_root
 ) {
   char *ptr;
   int error_num = 0;
+  const int link_idx= 0;
   DBUG_ENTER("spider_get_sys_tables_connect_info");
   if ((ptr = get_field(mem_root, table->field[3])))
   {
@@ -2354,9 +2141,14 @@ int spider_get_sys_tables_monitoring_binlog_pos_at_failing(
   DBUG_RETURN(error_num);
 }
 
+/*
+  Read the link status from mysql.spider_tables into a `SPIDER_SHARE'
+  with default value 1 (`SPIDER_LINK_STATUS_OK')
+*/
 int spider_get_sys_tables_link_status(
-  TABLE *table,
-  SPIDER_SHARE *share,
+  TABLE *table,                        /* The mysql.spider_tables table */
+  SPIDER_SHARE *share,                 /* The share to read link
+                                       status into */
   int link_idx,
   MEM_ROOT *mem_root
 ) {
@@ -2427,73 +2219,6 @@ int spider_get_sys_tables_static_link_id(
   DBUG_RETURN(error_num);
 }
 
-void spider_get_sys_table_sts_info(
-  TABLE *table,
-  ha_statistics *stat
-) {
-  MYSQL_TIME mysql_time;
-#ifdef MARIADB_BASE_VERSION
-  uint not_used_uint;
-#else
-  my_bool not_used_my_bool;
-#endif
-  long not_used_long;
-  DBUG_ENTER("spider_get_sys_table_sts_info");
-  stat->data_file_length = (ulonglong) table->field[2]->val_int();
-  stat->max_data_file_length = (ulonglong) table->field[3]->val_int();
-  stat->index_file_length = (ulonglong) table->field[4]->val_int();
-  stat->records = (ha_rows) table->field[5]->val_int();
-  stat->mean_rec_length = (ulong) table->field[6]->val_int();
-  table->field[7]->get_date(&mysql_time, SPIDER_date_mode_t(0));
-#ifdef MARIADB_BASE_VERSION
-  stat->check_time = (time_t) my_system_gmt_sec(&mysql_time,
-    &not_used_long, &not_used_uint);
-#else
-  stat->check_time = (time_t) my_system_gmt_sec(&mysql_time,
-    &not_used_long, &not_used_my_bool);
-#endif
-  table->field[8]->get_date(&mysql_time, SPIDER_date_mode_t(0));
-#ifdef MARIADB_BASE_VERSION
-  stat->create_time = (time_t) my_system_gmt_sec(&mysql_time,
-    &not_used_long, &not_used_uint);
-#else
-  stat->create_time = (time_t) my_system_gmt_sec(&mysql_time,
-    &not_used_long, &not_used_my_bool);
-#endif
-  table->field[9]->get_date(&mysql_time, SPIDER_date_mode_t(0));
-#ifdef MARIADB_BASE_VERSION
-  stat->update_time = (time_t) my_system_gmt_sec(&mysql_time,
-    &not_used_long, &not_used_uint);
-#else
-  stat->update_time = (time_t) my_system_gmt_sec(&mysql_time,
-    &not_used_long, &not_used_my_bool);
-#endif
-  if (table->field[10]->is_null())
-  {
-    stat->checksum_null = TRUE;
-    stat->checksum = 0;
-  } else {
-    stat->checksum_null = FALSE;
-    stat->checksum = (ha_checksum) table->field[10]->val_int();
-  }
-  DBUG_VOID_RETURN;
-}
-
-void spider_get_sys_table_crd_info(
-  TABLE *table,
-  longlong *cardinality,
-  uint number_of_keys
-) {
-  uint seq;
-  DBUG_ENTER("spider_get_sys_table_crd_info");
-  seq = (uint) table->field[2]->val_int();
-  if (seq < number_of_keys)
-  {
-    cardinality[seq] = (longlong) table->field[3]->val_int();
-  }
-  DBUG_VOID_RETURN;
-}
-
 int spider_sys_update_tables_link_status(
   THD *thd,
   char *name,
@@ -2504,11 +2229,7 @@ int spider_sys_update_tables_link_status(
 ) {
   int error_num;
   TABLE *table_tables = NULL;
-#if MYSQL_VERSION_ID < 50500
-  Open_tables_state open_tables_backup;
-#else
   Open_tables_backup open_tables_backup;
-#endif
   DBUG_ENTER("spider_sys_update_tables_link_status");
   if (
     !(table_tables = spider_open_sys_table(
@@ -2542,11 +2263,7 @@ int spider_sys_log_tables_link_failed(
 ) {
   int error_num;
   TABLE *table_tables = NULL;
-#if MYSQL_VERSION_ID < 50500
-  Open_tables_state open_tables_backup;
-#else
   Open_tables_backup open_tables_backup;
-#endif
   DBUG_ENTER("spider_sys_log_tables_link_failed");
   if (
     !(table_tables = spider_open_sys_table(
@@ -2581,11 +2298,7 @@ int spider_sys_log_xa_failed(
 ) {
   int error_num;
   TABLE *table_tables = NULL;
-#if MYSQL_VERSION_ID < 50500
-  Open_tables_state open_tables_backup;
-#else
   Open_tables_backup open_tables_backup;
-#endif
   DBUG_ENTER("spider_sys_log_xa_failed");
   if (
     !(table_tables = spider_open_sys_table(
@@ -2608,11 +2321,17 @@ error:
   DBUG_RETURN(error_num);
 }
 
+/* Populate `mon_key' from the current row in `table' */
 int spider_get_sys_link_mon_key(
-  TABLE *table,
-  SPIDER_MON_KEY *mon_key,
+  TABLE *table,                 /* the mysql.spider_link_mon_servers
+                                table */
+  SPIDER_MON_KEY *mon_key,      /* output, to be populated in this
+                                function */
   MEM_ROOT *mem_root,
-  int *same
+  int *same                     /* output, true if the data from the
+                                current row in the table agrees with
+                                existing data in `mon_key' and false
+                                otherwise */
 ) {
   char *db_name, *table_name, *link_id;
   uint db_name_length, table_name_length, link_id_length;
@@ -2628,11 +2347,12 @@ int spider_get_sys_link_mon_key(
     DBUG_RETURN(ER_SPIDER_SYS_TABLE_VERSION_NUM);
   }
 
-  if (
-    !(db_name = get_field(mem_root, table->field[0])) ||
-    !(table_name = get_field(mem_root, table->field[1])) ||
-    !(link_id = get_field(mem_root, table->field[2]))
-  )
+  /* get data for `mon_key' from the table record */
+  if (!(db_name = get_field(mem_root, table->field[0])))
+    DBUG_RETURN(HA_ERR_OUT_OF_MEM);
+  if (!(table_name = get_field(mem_root, table->field[1])))
+    DBUG_RETURN(HA_ERR_OUT_OF_MEM);
+  if (!(link_id = get_field(mem_root, table->field[2])))
     DBUG_RETURN(HA_ERR_OUT_OF_MEM);
 
   db_name_length = strlen(db_name);
@@ -2673,9 +2393,12 @@ int spider_get_sys_link_mon_key(
   DBUG_RETURN(0);
 }
 
+/* Get the server id from the spider_link_mon_servers table field */
 int spider_get_sys_link_mon_server_id(
-  TABLE *table,
-  uint32 *server_id,
+  TABLE *table,                        /* the
+                                       mysql.spider_link_mon_servers
+                                       table */
+  uint32 *server_id,                   /* output to server_id */
   MEM_ROOT *mem_root
 ) {
   char *ptr;
@@ -2688,14 +2411,17 @@ int spider_get_sys_link_mon_server_id(
   DBUG_RETURN(error_num);
 }
 
+/* Get connect info from the spider_link_mon_servers table fields */
 int spider_get_sys_link_mon_connect_info(
-  TABLE *table,
-  SPIDER_SHARE *share,
-  int link_idx,
+  TABLE *table,                           /* The
+                                          mysql.spider_link_mon_servers
+                                          table */
+  SPIDER_SHARE *share,                    /* The output spider_share */
   MEM_ROOT *mem_root
 ) {
   char *ptr;
   int error_num = 0;
+  const int link_idx= 0;
   DBUG_ENTER("spider_get_sys_link_mon_connect_info");
   if (
     !table->field[4]->is_null() &&
@@ -2877,9 +2603,6 @@ int spider_get_link_statuses(
       if (
         (error_num == HA_ERR_KEY_NOT_FOUND || error_num == HA_ERR_END_OF_FILE)
       ) {
-/*
-        table->file->print_error(error_num, MYF(0));
-*/
         DBUG_RETURN(error_num);
       }
     } else if ((error_num =
@@ -2892,282 +2615,6 @@ int spider_get_link_statuses(
   DBUG_RETURN(0);
 }
 
-int spider_sys_insert_or_update_table_sts(
-  THD *thd,
-  const char *name,
-  uint name_length,
-  ha_statistics *stat,
-  bool need_lock
-) {
-  int error_num;
-  TABLE *table_sts = NULL;
-#if MYSQL_VERSION_ID < 50500
-  Open_tables_state open_tables_backup;
-#else
-  Open_tables_backup open_tables_backup;
-#endif
-  DBUG_ENTER("spider_sys_insert_or_update_table_sts");
-  if (
-    !(table_sts = spider_open_sys_table(
-      thd, SPIDER_SYS_TABLE_STS_TABLE_NAME_STR,
-      SPIDER_SYS_TABLE_STS_TABLE_NAME_LEN, TRUE,
-      &open_tables_backup, need_lock, &error_num))
-  ) {
-    goto error;
-  }
-  if ((error_num = spider_insert_or_update_table_sts(
-    table_sts,
-    name,
-    name_length,
-    stat
-  )))
-    goto error;
-  spider_close_sys_table(thd, table_sts, &open_tables_backup, need_lock);
-  table_sts = NULL;
-  DBUG_RETURN(0);
-
-error:
-  if (table_sts)
-    spider_close_sys_table(thd, table_sts, &open_tables_backup, need_lock);
-  DBUG_RETURN(error_num);
-}
-
-int spider_sys_insert_or_update_table_crd(
-  THD *thd,
-  const char *name,
-  uint name_length,
-  longlong *cardinality,
-  uint number_of_keys,
-  bool need_lock
-) {
-  int error_num;
-  TABLE *table_crd = NULL;
-#if MYSQL_VERSION_ID < 50500
-  Open_tables_state open_tables_backup;
-#else
-  Open_tables_backup open_tables_backup;
-#endif
-  DBUG_ENTER("spider_sys_insert_or_update_table_crd");
-  if (
-    !(table_crd = spider_open_sys_table(
-      thd, SPIDER_SYS_TABLE_CRD_TABLE_NAME_STR,
-      SPIDER_SYS_TABLE_CRD_TABLE_NAME_LEN, TRUE,
-      &open_tables_backup, need_lock, &error_num))
-  ) {
-    goto error;
-  }
-  if ((error_num = spider_insert_or_update_table_crd(
-    table_crd,
-    name,
-    name_length,
-    cardinality,
-    number_of_keys
-  )))
-    goto error;
-  spider_close_sys_table(thd, table_crd, &open_tables_backup, need_lock);
-  table_crd = NULL;
-  DBUG_RETURN(0);
-
-error:
-  if (table_crd)
-    spider_close_sys_table(thd, table_crd, &open_tables_backup, need_lock);
-  DBUG_RETURN(error_num);
-}
-
-int spider_sys_delete_table_sts(
-  THD *thd,
-  const char *name,
-  uint name_length,
-  bool need_lock
-) {
-  int error_num;
-  TABLE *table_sts = NULL;
-#if MYSQL_VERSION_ID < 50500
-  Open_tables_state open_tables_backup;
-#else
-  Open_tables_backup open_tables_backup;
-#endif
-  DBUG_ENTER("spider_sys_delete_table_sts");
-  if (
-    !(table_sts = spider_open_sys_table(
-      thd, SPIDER_SYS_TABLE_STS_TABLE_NAME_STR,
-      SPIDER_SYS_TABLE_STS_TABLE_NAME_LEN, TRUE,
-      &open_tables_backup, need_lock, &error_num))
-  ) {
-    goto error;
-  }
-  if ((error_num = spider_delete_table_sts(
-    table_sts,
-    name,
-    name_length
-  )))
-    goto error;
-  spider_close_sys_table(thd, table_sts, &open_tables_backup, need_lock);
-  table_sts = NULL;
-  DBUG_RETURN(0);
-
-error:
-  if (table_sts)
-    spider_close_sys_table(thd, table_sts, &open_tables_backup, need_lock);
-  DBUG_RETURN(error_num);
-}
-
-int spider_sys_delete_table_crd(
-  THD *thd,
-  const char *name,
-  uint name_length,
-  bool need_lock
-) {
-  int error_num;
-  TABLE *table_crd = NULL;
-#if MYSQL_VERSION_ID < 50500
-  Open_tables_state open_tables_backup;
-#else
-  Open_tables_backup open_tables_backup;
-#endif
-  DBUG_ENTER("spider_sys_delete_table_crd");
-  if (
-    !(table_crd = spider_open_sys_table(
-      thd, SPIDER_SYS_TABLE_CRD_TABLE_NAME_STR,
-      SPIDER_SYS_TABLE_CRD_TABLE_NAME_LEN, TRUE,
-      &open_tables_backup, need_lock, &error_num))
-  ) {
-    goto error;
-  }
-  if ((error_num = spider_delete_table_crd(
-    table_crd,
-    name,
-    name_length
-  )))
-    goto error;
-  spider_close_sys_table(thd, table_crd, &open_tables_backup, need_lock);
-  table_crd = NULL;
-  DBUG_RETURN(0);
-
-error:
-  if (table_crd)
-    spider_close_sys_table(thd, table_crd, &open_tables_backup, need_lock);
-  DBUG_RETURN(error_num);
-}
-
-int spider_sys_get_table_sts(
-  THD *thd,
-  const char *name,
-  uint name_length,
-  ha_statistics *stat,
-  bool need_lock
-) {
-  int error_num;
-  char table_key[MAX_KEY_LENGTH];
-  TABLE *table_sts = NULL;
-#if MYSQL_VERSION_ID < 50500
-  Open_tables_state open_tables_backup;
-#else
-  Open_tables_backup open_tables_backup;
-#endif
-  DBUG_ENTER("spider_sys_get_table_sts");
-  if (
-    !(table_sts = spider_open_sys_table(
-      thd, SPIDER_SYS_TABLE_STS_TABLE_NAME_STR,
-      SPIDER_SYS_TABLE_STS_TABLE_NAME_LEN, TRUE,
-      &open_tables_backup, need_lock, &error_num))
-  ) {
-    goto error;
-  }
-
-  table_sts->use_all_columns();
-  spider_store_tables_name(table_sts, name, name_length);
-  if ((error_num = spider_check_sys_table(table_sts, table_key)))
-  {
-    if (error_num != HA_ERR_KEY_NOT_FOUND && error_num != HA_ERR_END_OF_FILE)
-    {
-      table_sts->file->print_error(error_num, MYF(0));
-    }
-    goto error;
-  } else {
-    spider_get_sys_table_sts_info(
-      table_sts,
-      stat
-    );
-  }
-
-  spider_close_sys_table(thd, table_sts, &open_tables_backup, need_lock);
-  table_sts = NULL;
-  DBUG_RETURN(0);
-
-error:
-  if (table_sts)
-    spider_close_sys_table(thd, table_sts, &open_tables_backup, need_lock);
-  DBUG_RETURN(error_num);
-}
-
-int spider_sys_get_table_crd(
-  THD *thd,
-  const char *name,
-  uint name_length,
-  longlong *cardinality,
-  uint number_of_keys,
-  bool need_lock
-) {
-  int error_num;
-  char table_key[MAX_KEY_LENGTH];
-  bool index_inited = FALSE;
-  TABLE *table_crd = NULL;
-#if MYSQL_VERSION_ID < 50500
-  Open_tables_state open_tables_backup;
-#else
-  Open_tables_backup open_tables_backup;
-#endif
-  DBUG_ENTER("spider_sys_get_table_crd");
-  if (
-    !(table_crd = spider_open_sys_table(
-      thd, SPIDER_SYS_TABLE_CRD_TABLE_NAME_STR,
-      SPIDER_SYS_TABLE_CRD_TABLE_NAME_LEN, TRUE,
-      &open_tables_backup, need_lock, &error_num))
-  ) {
-    goto error;
-  }
-
-  table_crd->use_all_columns();
-  spider_store_tables_name(table_crd, name, name_length);
-  if ((error_num = spider_get_sys_table_by_idx(table_crd, table_key, 0,
-    SPIDER_SYS_TABLE_CRD_PK_COL_CNT - 1)))
-  {
-    if (error_num != HA_ERR_KEY_NOT_FOUND && error_num != HA_ERR_END_OF_FILE)
-    {
-      table_crd->file->print_error(error_num, MYF(0));
-    }
-    goto error;
-  } else {
-    index_inited = TRUE;
-    do {
-      spider_get_sys_table_crd_info(
-        table_crd,
-        cardinality,
-        number_of_keys
-      );
-      error_num = spider_sys_index_next_same(table_crd, table_key);
-    } while (error_num == 0);
-  }
-  index_inited = FALSE;
-  if ((error_num = spider_sys_index_end(table_crd)))
-  {
-    table_crd->file->print_error(error_num, MYF(0));
-    goto error;
-  }
-
-  spider_close_sys_table(thd, table_crd, &open_tables_backup, need_lock);
-  table_crd = NULL;
-  DBUG_RETURN(0);
-
-error:
-  if (index_inited)
-    spider_sys_index_end(table_crd);
-  if (table_crd)
-    spider_close_sys_table(thd, table_crd, &open_tables_backup, need_lock);
-  DBUG_RETURN(error_num);
-}
-
 int spider_sys_replace(
   TABLE *table,
   bool *modified_non_trans_table
@@ -3187,12 +2634,8 @@ int spider_sys_replace(
 
     if (table->file->ha_table_flags() & HA_DUPLICATE_POS)
     {
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 50200
       error_num = table->file->ha_rnd_pos(table->record[1],
         table->file->dup_ref);
-#else
-      error_num = table->file->rnd_pos(table->record[1], table->file->dup_ref);
-#endif
       if (error_num)
       {
         if (error_num == HA_ERR_RECORD_DELETED)
@@ -3205,13 +2648,8 @@ int spider_sys_replace(
 
       key_copy((uchar*)table_key, table->record[0],
         table->key_info + key_num, 0);
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 50200
       error_num = table->file->ha_index_read_idx_map(table->record[1], key_num,
         (const uchar*)table_key, HA_WHOLE_KEY, HA_READ_KEY_EXACT);
-#else
-      error_num = table->file->index_read_idx_map(table->record[1], key_num,
-        (const uchar*)table_key, HA_WHOLE_KEY, HA_READ_KEY_EXACT);
-#endif
       if (error_num)
       {
         if (error_num == HA_ERR_RECORD_DELETED)
